// @(#)file      CommonAPIXMLTests.java
// @(#)author    Vincent Perrot, Sun Microsystems, Inc.
// @(#)version   1.12
// @(#)date      02/03/12
//
// Copyright 2002 Sun Microsystems, Inc. All rights reserved.
// This software is the proprietary information of Sun Microsystems, Inc.
// Use is subject to license terms.
//
// Copyright 2002 Sun Microsystems, Inc. Tous droits rservs.
// Ce logiciel est propriet de Sun Microsystems, Inc.
// Distribu par des licences qui en restreignent l'utilisation.
//
package ossj.common.tck;

import ossj.common.tck.SysProps;

import junit.framework.*;

// VP not portable code
// import weblogic.apache.xml.serialize.*;
/*
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
*/
import org.w3c.dom.*;

import org.xml.sax.*;

import java.io.*;

import java.util.*;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

import javax.naming.InitialContext;

import javax.oss.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;


/**
 * CommonAPIXMLTests Class
 *
 * Implement the XML Mesaging Mandatory Test of the JSR144 specification.
 *
 * @author Copyright 2002 Sun Microsystems Inc., Inc. All rights reserved.
 * @version 1.12
 */
public class CommonAPIXMLTests extends JVTTestBase {
    String testName = null;
    public boolean done = false;
    public TestResult result;
    
    /**
     * Creates a new CommonAPIXMLTests object.
     *
     * @param name the test name
     *
     * @throws Exception any
     */
    public CommonAPIXMLTests(String name) throws Exception {
        super(name);
        testName = name;
    }
    
    /**
     * Start the test
     *
     * @return the test result
     */
    public TestResult run() {
        result = createResult();
        super.run(result);
        
        return result;
    }
    
    /**
     * Start the test
     *
     * @param result where to put test result
     */
    public void run(TestResult result) {
        this.result = result;
        super.run(result);
    }
    
    /**
     * Initialize the test
     */
    public void setUp() {
        log("setUp");
        log(getName());
        
        if (runonce == false) {
            log("first Time");
            init();
            runonce = true;
        } else {
            log("not First Time");
        }
    }
    
    /**
     *
     */
    public void tearDown() {
        log("tearDown");
    }
    
    /**
     * TEST <CODE>test_JVTSessionFound</CODE>
     * <P>
     * PRECONDITION:
     * <P>
     * Testing Strategy:
     * <P>
     * Success Conditions:
     *<P>
     **/
    public void test_JVTSessionFound() {
        log("==> test_JVTSessionFound ");
        log("===> Using " + util.getProperty(SysProps.JARNAME));
        
        if (jvtSessionList == null) {
            fail("Cannot find and Session Object implementing JVTSession");
        } else {
            for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--)
                log("[" + ind + "] = " + jvtSessionList.get(ind));
        }
    }
    
    /**
     * TEST <CODE>test_XmlManagedEntityKeyAndValue</CODE>
     * <P>
     * PRECONDITION:
     * <P>
     * Testing Strategy:
     * <P>
     * Success Conditions:
     *<P>
     **/
    public void test_XmlManagedEntityKeyAndValue() {
        JVTSession mySession;
        
        try {
            log("==> test_XmlManagedEntityKeyAndValue ");
            
            for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
                // get the managedEntityValue for this JVTSession:
                // - get an instance of the session
                log(" Current JVTSession =  " + jvtSessionList.get(ind));
                mySession = getInstanceOf((String) jvtSessionList.get(ind));
                
                // 3- get the value types
                String[] typeArray = mySession.getManagedEntityTypes();
                
                for (int typeInd = typeArray.length - 1; typeInd >= 0; typeInd--) {
                    // 4- get an instance of a the value type
                    ManagedEntityValue myMev = mySession.makeManagedEntityValue(typeArray[typeInd]);
                    
                    ManagedEntityKey myKey = makeGenericKey(myMev);
                    
                    // 5- test from to xml on the key
                    //VP validateXml(myKey, "managedEntityKey");
                    
                    myMev.setManagedEntityKey(myKey);
                    myMev.setLastUpdateVersionNumber(new Date().getTime());
                    
                    //TODO: set attributes here
                    //VP validateXml(myMev, "managedEntityValue");
                }
            }
        } catch (Exception ex) {
            log(ex);
            fail("Unexpected Exception :" + ex.getClass().getName() + ", message ="
                    + ex.getMessage());
        }
    }
    
    /**
     * TEST <CODE>test_XmlQueryValue</CODE>
     * <P>
     * PRECONDITION:
     * <P>
     * Testing Strategy:
     * <P>
     * Success Conditions:
     *<P>
     **/
    public void test_XmlQueryValue() {
        JVTSession mySession;
        
        log("==> test_XmlQueryValue ");
        
        try {
            for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
                // get the managedEntityValue for this JVTSession:
                // - get an instance of the session
                mySession = getInstanceOf((String) jvtSessionList.get(ind));
                
                // 3- get the value types
                String[] typeArray = mySession.getNamedQueryTypes();
                
                for (int typeInd = typeArray.length - 1; typeInd >= 0; typeInd--) {
                    // 4- get an instance of a the value type
                    NamedQueryValue myQuv = mySession.makeNamedQueryValue(typeArray[typeInd]);
                    
                    //TODO: set attributes here
                    //VP validateXml(myQuv, "queryValue");
                }
            }
        } catch (Exception ex) {
            log(ex);
            fail("Unexpected Exception :" + ex.getClass().getName() + ", message ="
                    + ex.getMessage());
        }
    }
    
    /**
     * TEST <CODE>test_XmlEvent</CODE>
     * <P>
     * PRECONDITION:
     * <P>
     * Testing Strategy:
     * <P>
     * Success Conditions:
     *<P>
     **/
    public void test_XmlEvent() {
        JVTSession mySession;
        
        log("==> test_XmlEvent ");
        
        try {
            for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
                // get the managedEntityValue for this JVTSession:
                // - get an instance of the session
                mySession = getInstanceOf((String) jvtSessionList.get(ind));
                
                // 3- get the value types
                String[] typeArray = mySession.getEventTypes();
                
                for (int typeInd = typeArray.length - 1; typeInd >= 0; typeInd--) {
                    // 4- get an instance of a the value type
                    EventPropertyDescriptor myEventDescriptor = mySession.getEventDescriptor(typeArray[typeInd]);
                    
                    //TODO: set attributes here
                    // populate(myEventDescriptor);
                    Event myEvent = myEventDescriptor.makeEvent();
                    
                    //VP validateXml(myEvent, "event");
                }
            }
        } catch (Exception ex) {
            log(ex);
            fail("Unexpected Exception :" + ex.getClass().getName() + ", message ="
                    + ex.getMessage());
        }
    }
    
    // Private method use to factorize code
    //VP
    /*
    private void validateXml(javax.oss.SerializerFactory sf, String fType) {
        String[] supportedSerializerTypes = null;
        javax.oss.Serializer serializer = null;
        int i = -1;
     
        supportedSerializerTypes = sf.getSupportedSerializerTypes();
     
        while (++i < supportedSerializerTypes.length) {
            // 6- check only javax.oss.XmlSerializer type
            log("====> Test from/toXml on class [" + sf.getClass().getName() + "]");
            serializer = sf.makeSerializer(supportedSerializerTypes[i]);
            printSerializer(serializer);
     
            if ("javax.oss.XmlSerializer".equals(supportedSerializerTypes[i])) {
                // 7- test the from to xml calls
                XmlSerializer xmlSerializer = (XmlSerializer) serializer;
     
                log("=====> TO XML");
     
                String xml = xmlSerializer.toXml(sf, fType);
                log(xml);
     
                log("=====> GET THE DOCUMENT");
     
                Document doc = getDocument(sf.getClass().getName(), xml);
     
                //outputNiceXML(doc);
                log("=====> FROM XML");
     
                Element element = getElement(doc, fType);
                Object value = xmlSerializer.fromXml(element);
     
                log("=====> TO XML (FROM XML)");
     
                if (fType.equals("managedEntityKey")) {
                    xml = xmlSerializer.toXml((ManagedEntityKey) value, fType);
                } else if (fType.equals("managedEntityValue")) {
                    xml = xmlSerializer.toXml((ManagedEntityValue) value, fType);
                } else if (fType.equals("queryValue")) {
                    xml = xmlSerializer.toXml((QueryValue) value, fType);
                } else if (fType.equals("event")) {
                    xml = xmlSerializer.toXml((javax.oss.Event) value, fType);
                }
     
                log(xml);
                log("=====> OBJ == TO XML (FROM XML)");
     
                if (!valuesEquals(sf, value)) {
                    fail("[" + sf.getClass().getName() + "] of factory type [" + fType
                        + "].fromXml(toXml()) returns different objects");
                }
            }
        }
    }
     */
    
    // -------------------------
    // Generic dispatcher for the comparaison
    // -------------------------
    private boolean valuesEquals(Object value1, Object value2) {
        String commonClassName1 = findCommonClassName(value1);
        String commonClassName2 = findCommonClassName(value2);
        
        if (!(commonClassName1.equals(commonClassName2))) {
            return false;
        }
        
        if (commonClassName1.equals("javax.oss.ManagedEntityKey")) {
            return compareManagedEntityKey((ManagedEntityKey) value1, (ManagedEntityKey) value2);
        } else if (commonClassName1.equals("javax.oss.ManagedEntityValue")) {
            return compareManagedEntityValue((ManagedEntityValue) value1,
                    (ManagedEntityValue) value2);
        } else if (commonClassName1.equals("javax.oss.Event")) {
            return compareEvent((Event) value1, (Event) value2);
        } else if (commonClassName1.equals("javax.oss.QueryValue")) {
            return compareQueryValue((NamedQueryValue) value1, (NamedQueryValue) value2);
        }
        
        return true;
    }
    
    // -------------------------
    // ManagedEntitykey comparaison
    // -------------------------
    private boolean compareManagedEntityKey(ManagedEntityKey key1, ManagedEntityKey key2) {
        return (((key1.getApplicationContext() == null) ? (key2.getApplicationContext() == null)
        : key1.getApplicationContext().equals(key2
                .getApplicationContext()))
                && ((key1.getApplicationDN() == null) ? (key2.getApplicationDN() == null)
                : key1.getApplicationDN().equals(key2
                .getApplicationDN()))
                && ((key1.getType() == null) ? (key2.getType() == null)
                : key1.getType().equals(key2.getType())));
    }
    
    // -------------------------
    // ManagedEntityValue comparaison
    // -------------------------
    private boolean compareManagedEntityValue(ManagedEntityValue value1, ManagedEntityValue value2) {
        return compareManagedEntityKey(value1.getManagedEntityKey(), value2.getManagedEntityKey());
    }
    
    // -------------------------
    // Event comparaison
    // -------------------------
    private boolean compareEvent(Event value1, Event value2) {
        java.util.Date date1 = value1.getEventTime();
        java.util.Date date2 = value2.getEventTime();
        String dn1 = value1.getApplicationDN();
        String dn2 = value2.getApplicationDN();
        
        return (((dn1 == null) ? (dn2 == null) : dn1.equals(dn2)) && true);
    }
    
    // -------------------------
    // QueryValue comparaison
    // -------------------------
    private boolean compareQueryValue(NamedQueryValue value1, NamedQueryValue value2) {
        return true;
    }
    
    // -------------------------
    // Utilities
    // -------------------------
    private ManagedEntityKey makeGenericKey(ManagedEntityValue mev) {
        log("=====> makeGenericKey using ctx with fact =" + JNDI_FACT_CLASS_NAME + " URL="
                + JNDI_URL);
        
        ManagedEntityKey mek = mev.makeManagedEntityKey();
        ApplicationContext ac = mek.makeApplicationContext();
        ac.setFactoryClass(JNDI_FACT_CLASS_NAME);
        ac.setURL(JNDI_URL);
        
        //setSystemProperties();
        mek.setApplicationContext(ac);
        mek.setType("co:ManagedEntityValue");
        mek.setApplicationDN(mev.getClass().getName() + "DN");
        
        return mek;
    }
/*
    private void printSerializer(javax.oss.Serializer serializer) {
        int i = -1;
        int size = -1;
 
        log("		SupportedEncodingStyles");
 
        String[] supportedEncodingStyles = serializer.getSupportedEncodingStyles();
        size = supportedEncodingStyles.length - 1;
        i = -1;
 
        while (i < size) {
            i++;
            log("		                         " + supportedEncodingStyles[i]);
        }
 
        log("		DefaultEncodingStyle   = " + serializer.getDefaultEncodingStyle());
        log("		Type                   = " + serializer.getType());
    }
 */
    private void outputNiceXML(Document doc) {
        /*
        String encoding = "UTF-8";
         
        int LineWidth = 30;
        int Indent = 2;
         
        OutputFormat of = new OutputFormat();
        of.setEncoding(encoding);
        of.setLineWidth(LineWidth);
        of.setIndent(Indent);
         
        XMLSerializer xs = new XMLSerializer(System.out, of);
         */
        log("----START DOCUMENT----");
/*
        try {
            xs.serialize(doc);
        } catch (Exception e) {
            e.printStackTrace();
        }
 */
        log("---END DOCUMENT---");
    }
    
    private String findCommonClassName(Object obj) {
        if (obj instanceof javax.oss.ManagedEntityValue) {
            return "javax.oss.ManagedEntityValue";
        } else if (obj instanceof javax.oss.ManagedEntityKey) {
            return "javax.oss.ManagedEntityKey";
        } else if (obj instanceof javax.oss.Event) {
            return "javax.oss.Event";
        } else if (obj instanceof javax.oss.NamedQueryValue) {
            return "javax.oss.NamedQueryValue";
        }
        
        return null;
    }
    
    /**
     * return the Document corresponding to the given tag and XML String
     *
     * @param tag xml tag
     * @param xmlString xml string to include
     *
     * @return the Document
     */
    public Document getDocument(String tag, String xmlString) {
        Document doc = null;
        
        log("Document getDocument ");
        
        String docTxt = addXmlRootElement(tag, xmlString);
        
        try {
            log("--------document in getDocument-------------");
            log(docTxt);
            log("--------document in getDocument -------------");
            
            //Obtain an instance of DocumentBuilderFactory.
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            
            //Specify a validating parser.
            dbf.setValidating(false);
            
            // dbf.setErrorHandler();
            dbf.setNamespaceAware(true);
            dbf.setCoalescing(false);
            dbf.setIgnoringElementContentWhitespace(true);
            dbf.setIgnoringComments(true);
            
            //Obtain an instance of a DocumentBuilder from the factory.
            DocumentBuilder db = dbf.newDocumentBuilder();
            
            //Parse the document.
            StringReader reader = new StringReader(docTxt);
            InputSource is = new InputSource(reader);
            doc = db.parse(is);
        } catch (javax.xml.parsers.ParserConfigurationException pe) {
            log("Exception in parseXmlString");
            log(pe);
        } catch (Exception ex) {
            log("Exception in parseXmlString");
            log(ex);
        }
        
        return doc;
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param doc DOCUMENT ME!
     * @param tagName DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public org.w3c.dom.Element getElement(Document doc, String tagName) {
        log("tag name:" + tagName);
        
        log("getElement 1");
        
        Node node = null;
        NodeList nodeList = null;
        
        nodeList = doc.getElementsByTagName(tagName);
        
        // nodeList = doc.getElementsByTagNameNS("http://java.sun.com/products/oss/xml/Dummy", tagName);
        log("getElement 2");
        
        if (nodeList != null) {
            log("nodeList != null");
            log("nodeList.getLength()" + nodeList.getLength());
            node = nodeList.item(0);
        } else {
            log("element is null");
        }
        
        log("getElement 3");
        
        if (node == null) {
            log("node is null");
        }
        
        Element element = (Element) node;
        
        log("getElement 4");
        
        return element;
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param rootElementTag DOCUMENT ME!
     * @param xmlDoc DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String addXmlRootElement(String rootElementTag, String xmlDoc) {
        //TODO set XMLSchema the property in file
        StringBuffer strBuffer = new StringBuffer();
        strBuffer.append("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
        strBuffer.append("<" + rootElementTag + "\n");
        strBuffer.append("xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"\n"
                + "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
                + "xmlns:co=\"http://java.sun.com/products/oss/xml/Common/XmlCommonSchema.xsd\"\n"
                + "xsi:schemaLocation=\"" + util.getProperty("XMLSchema") + "\"\n" + ">");
        
        if (xmlDoc != null) {
            strBuffer.append(xmlDoc);
        }
        
        strBuffer.append("</" + rootElementTag + ">");
        
        return strBuffer.toString();
    }
    
    private boolean isCastableWith(Class c, String s) {
        if (s.equals(c.getName())) {
            return true;
        }
        
        // ---------
        // loop on interfaces
        // ---------
        Class[] cl = c.getInterfaces();
        
        for (int ind = cl.length - 1; ind >= 0; ind--) {
            if (cl[ind].getName().equals(s)) {
                return true;
            }
            
            Class[] cli = cl[ind].getInterfaces();
            
            if ((cli != null) && (cli.length > 0)) {
                return isCastableWith(cli[0], s);
            }
        }
        
        // ---------
        // loop on super class
        // ---------
        Class sup = c.getSuperclass();
        
        if (sup != null) {
            return isCastableWith(sup, s);
        }
        
        return false;
    }
}
