// @(#)file      FindRemote.java
// @(#)author    Vincent Perrot, Sun Microsystems, Inc.
// @(#)version   1.5
// @(#)date      02/03/05
// 
// Copyright 2002 Sun Microsystems, Inc. All rights reserved.
// This software is the proprietary information of Sun Microsystems, Inc.
// Use is subject to license terms.
// 
// Copyright 2002 Sun Microsystems, Inc. Tous droits rservs.
// Ce logiciel est propriet de Sun Microsystems, Inc.
// Distribu par des licences qui en restreignent l'utilisation.
// 
package ossj.common.tck;

import ossj.common.tck.StringTable;
import ossj.common.tck.SysProps;

import junit.framework.*;

import java.io.*;

import java.util.*;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

import javax.naming.InitialContext;

import javax.oss.*;


/**
 * FindRemote Class
 *
 * Implement the XML Mesaging Mandatory Test of the JSR144 specification.
 *
 * @author Copyright 2002 Sun Microsystems Inc., Inc. All rights reserved.
 * @version 1.5
 */
public class FindRemote {
    public static InitialContext namingContext = null;
    public static String JNDI_FACT_CLASS_NAME = "";
    public static String JNDI_URL = "";
    public static int EVENT_TIMEOUT = 60;
    public static String TESTFILES_HOME = "";
    public static String LOG_FILE = "";
    public static boolean STDOUT_ENABLED = true;
    public static ossj.common.tck.Util util = null;
    public static ArrayList jvtSessionList;

    /**
     * Creates a new FindRemote object.
     *
     * @throws Exception DOCUMENT ME!
     */
    public FindRemote() throws Exception {
        util = new ossj.common.tck.Util(true);
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws Exception DOCUMENT ME!
     */
    public ArrayList getRemote() throws Exception {
        ArrayList jvtSessionList = new ArrayList();

        // -------------------
        // read the jar and look for classes that should implements JVTSession 
        // -------------------
        //TODO look in the class path for all JVTSession ????
        JarFile jf = new JarFile(util.getProperty(SysProps.JARNAME));
        ZipEntry zf = null;
        String className = null;
        Class c = null;

        for (Enumeration e = jf.entries(); e.hasMoreElements();) {
            zf = (ZipEntry) e.nextElement();

            if (zf.getName().endsWith(".class")) {
                className = zf.getName().substring(0, zf.getName().lastIndexOf('.')).replace('/',
                        '.');

                if (className.equals("javax.oss.JVTSession")) {
                    continue;
                }

                c = Class.forName(className);

                // if (java.lang.reflect.Modifier.isAbstract(c.getModifiers())) continue;
                if (javax.ejb.EJBObject.class.isAssignableFrom(c)
                && javax.oss.JVTSession.class.isAssignableFrom(c)) {
                    int lastDotIndex = className.lastIndexOf('.');
                    jvtSessionList.add(className);
                }
            }
        }

        return jvtSessionList;
    }

    /**
     * DOCUMENT ME!
     *
     * @param jvtSessionList DOCUMENT ME!
     *
     * @throws IOException DOCUMENT ME!
     */
    public void addJvtSession(ArrayList jvtSessionList)
        throws IOException {
        // ---------------
        // add JVTSession at the end of property files 
        // --------------- 
        String propFilename = System.getProperty("PROPERTIES_FILE");

        if (propFilename == null) {
            propFilename = StringTable.SYSPROPERTYFILE;
        }

        FileInputStream propIn = null;
        FileOutputStream propOut = null;

        try {
            propIn = new FileInputStream(propFilename);
        } catch (java.io.FileNotFoundException fnfe) {
        }

        if (propIn == null) {
            throw new IOException("Could not find " + propFilename);
        }

        try {
            propOut = new FileOutputStream(propFilename + ".modified");
        } catch (java.io.FileNotFoundException fnfe) {
            throw new IOException("Could not create " + propFilename + ".modified");
        }

        // ---------------
        // Add customized line
        // ---------------
        System.out.println("# from jar_name = " + util.getProperty(SysProps.JARNAME) + "\n");
        propOut.write("# --------- Customization begin---------\n".getBytes());
        propOut.write(("# from jar_name = " + util.getProperty(SysProps.JARNAME) + "\n").getBytes());
        propOut.write("# For each following Home, you have to provided, its JNDI name.\n".getBytes());
        propOut.write("\n".getBytes());

        int size = jvtSessionList.size();
        String name = null;

        for (int i = size - 1; i >= 0; i--) {
            name = (String) jvtSessionList.get(i);
            propOut.write(name.getBytes());
            propOut.write("_HOME=\"<class name of the corresponding home interface>\"\n".getBytes());
            propOut.write(name.getBytes());
            propOut.write("_JNDINAME=\"<JNDI name>\"\n".getBytes());
            propOut.write("\n".getBytes());
        }

        propOut.write("\n".getBytes());
        propOut.write("# --------- Customization end  ---------\n".getBytes());
        propOut.write("\n".getBytes());
        propOut.write("\n".getBytes());

        // ---------------
        // Add Original property file
        // ---------------
        // int nb = propIn.available() ;
        // byte b[] = new byte[propIn.available()] ;
        // propIn.read(b) ;
        // propOut.write(b) ;
        // 
        // propOut.write("\n".getBytes() ) ;
        propOut.close();
    }

    /**
     * DOCUMENT ME!
     *
     * @param args DOCUMENT ME!
     *
     * @throws Exception DOCUMENT ME!
     */
    public static void main(String[] args) throws Exception {
        FindRemote finder = new FindRemote();
        finder.addJvtSession(finder.getRemote());
    }
}
