package ossj.common.tck;

import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.oss.JVTSession;
import javax.rmi.PortableRemoteObject;
import junit.framework.Assert;
import junit.framework.TestCase;
import junit.framework.TestResult;

/**
 * JVTTestBase Class
 *
 * A Base Class for TTAPIJVTTests
 * This class is both a TT JVT Proxy and a Test Case.
 * The TTAPIJVTTests extends and use the services of this class.
 * The value class implementations used in this class are the one
 * retrieved by calling the make methods on the provider JVT Session Bean
 * @author Copyright 2001-2002 Nortel Networks Limited. All rights reserved.
 * @version 1.0
 * @since December 2001
 */
public class JVTTestBase extends TestCase {
    public static boolean runonce = false;
    public static boolean debug = false;
    public static InitialContext namingContext = null;
    public static String JNDI_FACT_CLASS_NAME = "";
    public static String JNDI_URL = "";
    public static int EVENT_TIMEOUT = 60;
    public static String TESTFILES_HOME = "";
    public static String LOG_FILE = "";
    public static String JMS_FACTORY = "";
    public static String TOPIC_NAME = "";
    public static ArrayList jvtSessionList = null;
    public static ossj.common.tck.Util util = null;

    //Cache for keys used in prompts.
    //public java.util.Vector keyCache 	= new java.util.Vector();
    public static int QUOTECHAR = 34;
    public boolean done = false;

    //provide defaults for these config items (they are not manadatory in the config file)
    public int howmany_key = 10;
    public int howmany_value = 10;
    public boolean return_failed_only = false;
    public TestResult result;

    //VP    public final String configFilePath = "TTClientConfig.txt";
    public JVTTestBase(String name) {
        super(name);
    }
    /**
     * Start the test
     *
     * @return the test result
     */
    public TestResult run() {
        result = createResult();
        super.run(result);

        return result;
    }

    /**
     * Start the test
     *
     * @param result where to put test result
     */
    public void run(TestResult result) {
        this.result = result;
        super.run(result);
    }

    /**
     * Print the string to the log file
     *
     * @param s text to add
     */
    public void log(String s) {
        if (debug) {
            util.log(s);
        }
    }

    /**
     * print the exception to the log file
     *
     * @param e exception to trace
     */
    public void log(Throwable e) {
        if (debug) {
            util.log(e);
        }
    }

    /**
     * Set up the environment
     */
    public void getEnvironment() {
        try {
            util = new ossj.common.tck.Util(true);
        } catch (Exception e) {
            e.printStackTrace();
        }

        if ((LOG_FILE = util.getProperty(ossj.common.tck.SysProps.LOGFILE)) != null) {
            if (util.outputFile == null) {
                try {
                    util.outputFile = new PrintStream(new FileOutputStream(LOG_FILE));
                } catch (java.io.FileNotFoundException e) {
                    e.printStackTrace();
                    util.outputFile = System.out;
                }
            }
        } else {
            util.outputFile = System.out;
        }

        String logValue = util.getProperty(ossj.common.tck.SysProps.LOGGING_ENABLED);
        debug = false;

        if (logValue != null) {
            if (logValue.equals("true")) {
                debug = true;
            }
        }

        JNDI_URL = util.getProperty(ossj.common.tck.SysProps.JNDIURL);
        JNDI_FACT_CLASS_NAME = util.getProperty(ossj.common.tck.SysProps.JNDIFACTORY);
        EVENT_TIMEOUT = Integer.getInteger("EVENT_TIMEOUT", 60).intValue();
        TESTFILES_HOME = util.getProperty(ossj.common.tck.SysProps.EVENTTIMEOUT);
    }

    /**
     * Init the env, to be call by all inheritence
     */
    public void init() {
        if (runonce == false) {
            getEnvironment();
            log("---INIT CALLED----");

            try {
                FindRemote finder = new FindRemote();
                jvtSessionList = finder.getRemote();
            } catch (Exception e) {
                log(e);
            }

            //runonce = true;
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param myClass DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws TCKException DOCUMENT ME!
     */
    public JVTSession getInstanceOf(String myClass) throws TCKException {
        Thread.currentThread().setContextClassLoader(ClassLoader.getSystemClassLoader());

        javax.ejb.EJBHome omHome = null;
        JVTSession result = null;

        String HomeClassName = util.getProperty(myClass + "_HOME");

        try {
            Object obj = util.getInitialContext().lookup(util.getProperty(myClass + "_JNDINAME"));

            Class testClass = ((this.getClass()).getClassLoader()).loadClass(HomeClassName);
            log("VP> the home class for " + myClass + " = " + testClass.getName());
            omHome = (javax.ejb.EJBHome) PortableRemoteObject.narrow(obj, testClass);

            //result=  omHome.create(); 
            // create is not part of the SessionBean Interface
            result = (JVTSession) omHome.getClass().getMethod("create", (Class[])null).invoke(omHome, (Object[])null);
        } catch (NoSuchMethodException nsmex) { //getMethod
            log(nsmex);
            throw new TCKException("The " + HomeClassName
                + " Home interface does not contain the default method without argument create()",
                nsmex);
        } catch (NamingException nex) { //thrown by lookup
            log(nex);
            throw new TCKException("Please check the property "
                + util.getProperty(myClass + "_JNDINAME")
                + " of the configuration file in your server configuration", nex);
        } catch (ClassCastException ccex) {
            log(ccex);

            //thrown by narrow
            throw new TCKException("Client and server classes differ! Please check version numbers",
                ccex);
        } catch (Exception m) {
            log(m);
        }

        return result;
    }


    /**
    overload to make life easier
    */
    public static void assertTrue(String attr, String type, boolean c) {
        Assert.assertTrue(StringTable.ATTRIBUTE + attr + StringTable.TYPE + type, c);
    }

    /**
        overload to make life easier
    */
    public static void assertTrue(String type, boolean c) {
        Assert.assertTrue(StringTable.TYPE + type, c);
    }

    /**
        overload to make life easier
    */
    public static void assertTrueMsg(String msg, boolean c) {
        Assert.assertTrue(msg, c);
    }

    /**
        overload to make life easier
    */
    public static void fail(String attr, String type) {
        Assert.fail(StringTable.ATTRIBUTE + attr + StringTable.TYPE + type);
    }

    /**
        overload to make life easier
    */
    public static void fail(String type) {
        Assert.fail(StringTable.TYPE + type);
    }
}
