package ossj.common.tck;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import java.util.Properties;


/**
 * Properties subclass which ensures that required properties are available
 * @author  BanuPrasad
 * @version
 */
public class SafeProperties extends Properties {
    /**
        creates a new SafeProperties instance
        @param propFilename The file to read the properties from
        @param keys The list of keys which must be present in the property file
        @throws IOException If the file cannot be read
        @throws IllegalArgumentException If a required property is missing
    */
    public SafeProperties(String propFilename, String[] keys)
        throws IOException, IllegalArgumentException {
        super();

        InputStream propIS = null;

        try {
            propIS = new FileInputStream(propFilename);
        } catch (java.io.FileNotFoundException fnfe) {
        }

        if (propIS == null) {
            propIS = ClassLoader.getSystemResourceAsStream(propFilename);
        }

        if (propIS == null) {
            propIS = this.getClass().getClassLoader().getResourceAsStream(propFilename);
        }

        if (propIS == null) {
            throw new IOException("Could not find " + propFilename);
        }

        this.load(propIS);

        boolean oneMissing = false;
        String keyList = new String();

        for (int i = 0; i < keys.length; ++i) {
            if (!this.containsKey(keys[i])) {
                oneMissing = true;
                keyList += (keys[i] + ",");
            }
        }

        if (oneMissing) {
            throw new IllegalArgumentException(propFilename
                + " does not contain all required properties " + keyList);
        }
    }

    /**
        checks if the property file contains all specified keys
        @param keys The list of keys
        @return If the object contains all specified keys
    */
    public boolean containsAllKeys(String[] keys) {
        boolean retVal = true;

        if (null != keys) {
            for (int i = 0; (i < keys.length) && retVal; ++i)
                retVal = this.containsKey(keys[i]);
        }

        return retVal;
    }
}
