/*
 * Starter.java
 *
 * Created on 6. Februar 2002, 12:37
 */
package ossj.common.tck;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.Properties;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * @author aebbert
 * @version
 */
public class Starter {
    public static boolean stopAtFirstFailed = false;
    
    private static final String fileSeperator = System.getProperty("file.separator");
    private static final String classpath = System.getProperty("java.class.path");
    private static final String javaHome = System.getProperty("java.home");
    private static final String javaBin = javaHome + fileSeperator + "bin" + fileSeperator + "java";
    
    private static final String integrationProfile = System.getProperty("INTEGRATION_PROFILE");
    
    //private static final String[] javaArgs = { "-cp", classpath};
    private static final String funcTest = "ossj.common.tck.CommonApiTckTest";
    private static final String eventTest = "ossj.common.tck.tests.TestEvents$EndlessEventTest";
    private static final String junitRunner = "junit.textui.TestRunner";
    private static Runtime myRuntime = Runtime.getRuntime();
    private String[] newJavaArgs;
    private SuiteStarter funcSuiteStarter;
    
    //    private OutputStream funcOut = new BufferedOutputStream(new ByteArrayOutputStream());
    private OutputStream funcOut = new ByteArrayOutputStream();
    private SuiteStarter eventSuiteStarter;
    
    //    private OutputStream eventOut = new BufferedOutputStream(new ByteArrayOutputStream());
    private OutputStream eventOut = new ByteArrayOutputStream();
    
    /** Creates new Starter */
    public Starter(String[] args) {
        newJavaArgs = args;
    }
    
    /**
     * Start the test suite
     *
     * @throws Exception any
     */
    public void startTck() throws Exception {
        List commands = new ArrayList();
        commands.add(javaBin);
        //VP 
        commands.add("com.sun.enterprise.appclient.Main");
        //end VP
        
        for (int i = 0; i < newJavaArgs.length; i++) {
            commands.add(newJavaArgs[i]);
        }
        /*
        for (int i = 0; i < javaArgs.length; i++) {
            commands.add(javaArgs[i]);
        }
         */
        Properties systemp = System.getProperties();
        String namep = null;
        for (Enumeration e = systemp.propertyNames();e.hasMoreElements();){
            namep = (String)e.nextElement();
            if (namep.startsWith("org.omg.") ||
                    namep.startsWith("INTEGRA") ||
                    namep.startsWith("PROPERT") ||
                    namep.startsWith("com.sun.appserv.iiop") ||
                    namep.startsWith("JNDI") ||
                    namep.startsWith("java.util"))
                commands.add("-D"+namep+"="+systemp.getProperty(namep));
        }
        
        commands.add(junitRunner);
        commands.add(eventTest);
        
        //System.err.println("CDE="+commands);
        
        //VP System.err.println("==> VP cde = ["+commands);
        eventSuiteStarter = new SuiteStarter((String[]) commands.toArray(new String[0]));
        eventSuiteStarter.start();
        
        commands.clear();
        
        commands.add(javaBin);
        
        for (int i = 0; i < newJavaArgs.length; i++) {
            commands.add(newJavaArgs[i]);
        }
        
        for (Enumeration e = systemp.propertyNames();e.hasMoreElements();){
            namep = (String)e.nextElement();
            if (namep.startsWith("org.omg.") ||
                    namep.startsWith("INTEGRA") ||
                    namep.startsWith("PROPERT") ||
                    namep.startsWith("com.sun.appserv.iiop") ||
                    namep.startsWith("JNDI") ||
                    namep.startsWith("java.util"))
                commands.add("-D"+namep+"="+systemp.getProperty(namep));
        }
        
        commands.add(junitRunner);
        commands.add(funcTest);
        
        //System.err.println("CDE="+commands);
        
        //VP System.err.println("==> VP event cde = ["+commands);
        funcSuiteStarter = new SuiteStarter((String[]) commands.toArray(new String[0]));
        funcSuiteStarter.start();
        
        while (!funcSuiteStarter.didRan() && !eventSuiteStarter.didRan()) {
            try {
                //System.out.println("wait");
                Thread.sleep(100);
            } catch (InterruptedException ie) {
                ie.printStackTrace();
            }
        }
        
        TestSuite eventSuite = null;
        TestSuite funcSuite = null;
        
        boolean suitesLoaded = false;
        
        try {
            eventSuite = (TestSuite) Class.forName(eventTest).getMethod("suite", new Class[0])
            .invoke(null, new Object[0]);
            funcSuite = (TestSuite) Class.forName(funcTest).getMethod("suite", new Class[0])
            .invoke(null, new Object[0]);
            suitesLoaded = true;
        } catch (ClassNotFoundException cnfe) {
            cnfe.printStackTrace();
        } catch (NoSuchMethodException nsme) {
            nsme.printStackTrace();
        } catch (SecurityException se) {
            se.printStackTrace();
        } catch (IllegalAccessException iae) {
            iae.printStackTrace();
        } catch (IllegalArgumentException iare) {
            iare.printStackTrace();
        } catch (InvocationTargetException ite) {
            ite.printStackTrace();
        }
        
        StreamRedirector eventRedirect = null;
        StreamRedirector eventErrorRedirect = null;
        StreamRedirector funcRedirect = null;
        StreamRedirector funcErrorRedirect = null;
        
        if (!suitesLoaded) {
            System.out.println("could not load test suites");
            funcSuiteStarter.destroyProcess();
        } else {
            eventRedirect = new StreamRedirector(eventSuiteStarter.getOutputStream(), eventOut,
                    "EVNT ", eventSuite);
            eventRedirect.start();
            eventErrorRedirect = new StreamRedirector(eventSuiteStarter.getErrorStream(), eventOut,
                    null, eventSuite);
            eventErrorRedirect.start();
            
            funcRedirect = new StreamRedirector(funcSuiteStarter.getOutputStream(), funcOut,
                    "FUNC ", funcSuite);
            funcRedirect.start();
            funcErrorRedirect = new StreamRedirector(funcSuiteStarter.getErrorStream(), funcOut,
                    null, funcSuite);
            funcErrorRedirect.start();
        }
        
        while (funcSuiteStarter.isAlive()) {
            // wait until tck has finished
            try {
                Thread.sleep(1000);
            } catch (InterruptedException ie) {
                ie.printStackTrace();
            }
        }
        
        if (eventSuiteStarter.isAlive()) {
            eventSuiteStarter.destroyProcess();
        }
        
        //System.out.println("func tck finished");
        if (funcRedirect != null) {
            funcRedirect.stopGentle();
        }
        
        if (eventRedirect != null) {
            eventRedirect.stopGentle();
        }
        
        if (funcErrorRedirect != null) {
            funcErrorRedirect.stopGentle();
        }
        
        if (eventErrorRedirect != null) {
            eventErrorRedirect.stopGentle();
        }
        
        //System.out.println("stopped redirectory");
        while (funcRedirect.isAlive() && eventRedirect.isAlive() && funcErrorRedirect.isAlive()
        && eventErrorRedirect.isAlive()) {
            // wait until tck has finished
            try {
                Thread.sleep(1000);
            } catch (InterruptedException ie) {
                ie.printStackTrace();
            }
        }
        
        // read tck propertyfiles
        Properties tckProps = new SafeProperties(StringTable.SYSPROPERTYFILE, new String[0]);
        
        // create test report
        String spacer = "==========================================================";
        File reportFile = new File("tckResults.txt");
        FileOutputStream fos = new FileOutputStream(reportFile);
        PrintWriter pw = new PrintWriter(fos);
        pw.println("Report from OSS Common API (JSR144) TCK");
        pw.println("TCK Version: 1.4");
        pw.println("Tested Integration Profile: "+integrationProfile);
        pw.println("Creation time is: " + (new Date()).toString());
        pw.println(spacer);
        pw.println(spacer);
        pw.println("\nResults: Event tests");
        pw.println(spacer);
        pw.print(eventOut.toString());
        pw.println("\nResults: Function tests");
        pw.println(spacer);
        pw.print(funcOut.toString());
        pw.println("\nConfiguration: tckproperties.txt");
        pw.println(spacer);
        pw.flush();
        (new SafeProperties(StringTable.SYSPROPERTYFILE, new String[0])).store(fos, null);
        fos.flush();
        pw.println("\nConfiguration: system properties");
        pw.println(spacer);
        pw.flush();
        System.getProperties().store(fos, null);
        pw.println("\n\n" + spacer);
        pw.println("End of report.");
        fos.flush();
        pw.close();
        fos.close();
        System.out.println("TCK Report was written to file:");
        System.out.println(reportFile.getAbsolutePath());
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        if ((args.length > 0)
        && (args[0].equals("-?") || args[0].equals("?") || args[0].equals("/?")
        || args[0].equals("-h") || args[0].equals("/h") || args[0].equals("h")
        || args[0].equals("--help") || args[0].equals("-help") || args[0].equals("/help")
        || args[0].equals("help"))) {
            System.out.println("usage:\n" + "java ossj.common.tck.Starter [options]\n\n"
                    + "options: are passed to the internally started java processes");
        } else {
            String stop = System.getProperty("stopAtFirstFailed");
            if (stop != null && stop.equals("true")){
                stopAtFirstFailed = true;
            }
            try {
                (new Starter(args)).startTck();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
    
    public class SuiteStarter extends Thread {
        private String[] command;
        private Process process;
        private boolean ran = false;
        
        public SuiteStarter(String[] command) {
            this.command = command;
        }
        
        public void run() {
            try {
                //for (int d = 0; d < command.length;d++) System.out.println(" "+command[d]);
                process = myRuntime.exec(command);
                ran = true;
                process.waitFor();
                
                //System.out.println("starter stopped");
            } catch (IOException ioe) {
                ioe.printStackTrace();
            } catch (InterruptedException ie) {
                ie.printStackTrace();
            } catch (SecurityException se) {
                se.printStackTrace();
            }
        }
        
        public boolean didRan() {
            return ran;
        }
        
        public void destroyProcess() {
            if (process != null) {
                process.destroy();
            }
            
            //System.out.println("destroyed externel process");
        }
        
        /**
         * This is the processe' OutputStream!!!
         */
        public OutputStream getInputStream() {
            return process.getOutputStream();
        }
        
        public InputStream getErrorStream() {
            return process.getErrorStream();
        }
        
        /**
         *This is the process' InputStream!!!
         */
        public InputStream getOutputStream() {
            return process.getInputStream();
        }
    }
    
    public class StreamRedirector extends Thread {
        private InputStream source;
        private OutputStream dest;
        private TestSuite suite;
        private String prefix;
        private boolean stop = false;
        private String[] testCaseNames;
        
        public StreamRedirector(InputStream source, OutputStream dest, String prefix,
                TestSuite suite) {
            this.source = source;
            this.dest = dest;
            this.suite = suite;
            this.prefix = prefix;
            
            List testCaseNamesList = new ArrayList();
            createTestCaseNames(suite, testCaseNamesList);
            testCaseNames = (String[]) testCaseNamesList.toArray(new String[0]);
            
            //System.out.println("found "+testCaseNames.length+" TestCases");
        }
        
        private void createTestCaseNames(TestSuite suite, List aList) {
            if (suite != null) {
                int testcases = suite.testCount();
                
                if ((testcases == 1) && (suite.testAt(0).countTestCases() == Integer.MAX_VALUE)) {
                    return;
                }
                
                for (int i = 0; i < testcases; i++) {
                    Test testCase = suite.testAt(i);
                    
                    if (testCase instanceof TestCase) {
                        aList.add(((TestCase) testCase).getName());
                    } else if (testCase instanceof TestSuite) {
                        createTestCaseNames((TestSuite) testCase, aList);
                    }
                }
            }
        }
        
        public void run() {
            try {
                int total = 0;
                int testcases = testCaseNames.length;
                int state = 0; // expect "."
                
                if (suite == null) {
                    state = -1; // just forward output
                }
                
                int available;
                byte[] bytes = new byte[1024];
                Test currentTest;
                String testCaseName = "";
                
                do {
                    available = source.available();
                    
                    synchronized (dest) {
                        while (available > 0) {
                            //System.out.println(available+" bytes in inputstream available");
                            int read;
                            int offset = 0;
                            read = source.read(bytes, 0, Math.min(bytes.length, available));
                            
                            // chage the "."s and "E"s from JUnit to something more meaningful
                            // the endless event test has Integer.MAX_VALUE testcases, but the suite does not know of them
                            // so suite.testAt may not be called
                            while ((state != -1) && ((read - offset) > 0)
                            && ((total < testcases) || (testcases == 0))) {
                                String singleChar = new String(bytes, offset, 1);
                                
                                //System.err.println("found byte "+Byte.toString(bytes[offset])+ "single="+singleChar);
                                if ((bytes[offset] == 13) || (bytes[offset] == 10)
                                ) {
                                    offset++;
                                } else if ( !(singleChar.equals(".")
                                || singleChar.equals("E") 
                                || singleChar.equals("F"))) {
                                    offset++;
                                } else if (state == 0) { // expected char is "."
                                    
                                    if (singleChar.equals(".")) {
                                        if (testcases != 0) {
                                            testCaseName = testCaseNames[total];
                                        } else {
                                            testCaseName = suite.toString() + " " + total;
                                        }
                                        
                                        offset++;
                                        
                                        //System.out.println("found test "+currentTest);
                                        //System.out.println("found test case name "+testCaseName);
                                        state = 1; // parse next char to see if test succeded
                                    } else {
                                        state = -1; // unexpected character - ignore all the rest
                                    }
                                } else if (state == 1) { // expected char is either "E" if test failed or "." for next test
                                    
                                    // if test failed, there is another byte == 'E'
                                    boolean passed = true;
                                    
                                    if (singleChar.equals("E") || singleChar.equals("F")) {
                                        passed = false;
                                        offset++;
                                    }
                                    
                                    total++;
                                    
                                    print( (total<10?"00":(total<100?"0":""))+total + " "+(passed ? "passed" : "FAILED") + ": " + testCaseName
                                            + "\n");
                                    state = 0; // wait for next test "."
                                    if (stopAtFirstFailed && !passed){ stop = true;}
                                }
                                
                                // characters from outputstream possibly are "."s & "E"s
                            }
                            
                            if ((read - offset) > 0) {
                                print(bytes, offset, read - offset);
                            }
                            
                            available -= read;
                        }
                    }
                    
                    try {
                        sleep(1000);
                    } catch (InterruptedException ie) {
                        ie.printStackTrace();
                    }
                } while (!stop);
                
                if (state == 1) {
                    // assume last test succeded - no "E" received anyway
                    print((total<10?"00":(total<100?"0":""))+total + " "+"passed: " + testCaseName + "\n");
                }
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
            
            //System.out.println("redirector stopped");
        }
        
        private void print(String s) throws IOException {
            String withTimeStamp = ("[" + new Date()).toString() + "] " + s;
            print(withTimeStamp.getBytes(), 0, withTimeStamp.length());
        }
        
        private void print(byte[] bytes, int offset, int length)
        throws IOException {
            dest.write(bytes, offset, length);
            
            if (prefix != null) {
                System.out.print(prefix + (new String(bytes, offset, length)));
            }
        }
        
        public void stopGentle() {
            stop = true;
        }
    }
}
