/*
 * util.java
 * Loads of utilities for the TCK
 *
 */
package ossj.common.tck;


import java.beans.*;

import java.io.*;

import java.lang.reflect.*;

import java.util.*;
import java.util.zip.*;

import javax.jms.InvalidDestinationException;
import javax.jms.InvalidSelectorException;
import javax.jms.JMSException;
import javax.jms.JMSSecurityException;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;

import javax.naming.*;

import javax.oss.*;

import javax.rmi.*;


/**
 * Collection of useful methods & metadata
 * @author BanuPrasad Dhanakoti
 * @version
 */
public class Util extends Object {
    private static Hashtable classImpl = new Hashtable();
    
    // this contains the keys of mangedkeys
    private static Hashtable managedKeys = null;
    
    /**Hashtable of Vectors of illegal ManagedEntityKeys per Type*/
    private static Hashtable illegalKeys = null;
    
    //private final int sequences;
    private static SysProps sysProps;
    private static TestProps testProps;
    
    /** for log */
    public static PrintStream outputFile = null;
    private InitialContext ictx = null;
    
    /** The  types available for testing*/
    
    //VP	private  String[] MANAGED_TYPES = null;
    
    /** The property types for all MEVs */
    
    //VP     private static Hashtable mevAttributeTypes = null;
    
    /**The maximum allowed string length*/
    private final int MAXSTRINGLENGTH = 512;
    
    /**the timeout per event poll*/
    private final long eventTimeout;
    
    /**throws total timout to wait for an event*/
    private final long totalTimeout = 5000;
    
    //VP private JVTSession jas;
    
    /**used for random generator*/
    public final Random RANDOM;
    
    /* jms resources */
    private QueueSession qsession = null;
    
    /** Creates new util */
    public Util() throws Exception {
        this(false);
    }
    
    /**
     * Creates a new Util object.
     *
     * @param generic DOCUMENT ME!
     *
     * @throws Exception DOCUMENT ME!
     */
    public Util(boolean generic) throws Exception {
        String propertiesFiles = System.getProperty("PROPERTIES_FILE");
        
        if (propertiesFiles == null) {
            propertiesFiles = StringTable.SYSPROPERTYFILE;
        }
        
        if (null == this.sysProps) {
            this.sysProps = new SysProps(propertiesFiles);
        }
        
        this.RANDOM = new Random();
        
        long time = Integer.parseInt(this.sysProps.getProperty(SysProps.EVENTTIMEOUT));
        this.eventTimeout = Math.max(1, time);
        
        //VP this.jas = this.getJVTSession();
        //VP this.MANAGED_TYPES = this.jas.getManagedEntityTypes() ;
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param s DOCUMENT ME!
     */
    public void log(String s) {
        outputFile.println(s);
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void log(Throwable e) {
        outputFile.println(e.getMessage());
        e.printStackTrace(outputFile);
    }
    
    /**@return The timeout to wait for an event*/
    public long getEventTimeout() {
        return this.eventTimeout;
    }
    
    /**The timeout to wait for all events*/
    public long getTotalTimeout() {
        return this.totalTimeout;
    }
    
    /**
     * @return The type of the attribute. For indexed attributes, the element
     * type is returned.
     * If an attribute of that name doesn't exist, null is returned
     */
    public Class getAttributeType(AttributeAccess mev, String name)
    throws Exception {
        if ((null == name) || (null == mev)) {
            throw new java.lang.IllegalArgumentException("Null argument");
        }
        
        PropertyDescriptor prop = new PropertyDescriptor(name, mev.getClass());
        
        return prop.getPropertyType();
    }
    
    /**
     * @return Hashtable containing pairs of MEV class and
     * a hashtable of attribute name/type pairs
     */
    private Hashtable makeMEVAttributeTypes(JVTSession jas)
    throws Exception {
        //VP JVTSession om = this.getJVTSession();
        String[] mgNames = jas.getManagedEntityTypes();
        
        Hashtable retVal = new Hashtable(mgNames.length);
        
        for (int i = 0; i < mgNames.length; ++i) {
            Class cls = jas.makeManagedEntityValue(mgNames[i]).getClass();
            retVal.put(cls, Util.makeAttributeTypes(cls));
        }
        
        return retVal;
    }
    
    /**
     * @param theClass The class to make the attribute/attribute type hashtable from
     * @return The hashtable
     */
    private static Hashtable makeAttributeTypes(Class theClass)
    throws Exception {
        Hashtable retVal = new Hashtable();
        BeanInfo info = Introspector.getBeanInfo(theClass);
        
        //  System.out.println("***********hi the BeabInfo is *****************" + info);
        PropertyDescriptor[] desc = info.getPropertyDescriptors();
        
        for (int i = 0; i < desc.length; ++i) {
            Class cls = null;
            
            if (desc[i] instanceof IndexedPropertyDescriptor) {
                cls = ((IndexedPropertyDescriptor) desc[i]).getIndexedPropertyType();
            } else {
                cls = desc[i].getPropertyType();
            }
            
            retVal.put(desc[i].getName(), cls);
        }
        
        return retVal;
    }
    
    /**@return A new InitialContext for the server*/
    public InitialContext getInitialContext() throws TCKException {
        
        if (ictx == null) {
            try {
                
                Hashtable env = new Hashtable();
                env.put(Context.INITIAL_CONTEXT_FACTORY,
                        this.sysProps.getProperty(SysProps.JNDIFACTORY));
                env.put(Context.PROVIDER_URL, this.sysProps.getProperty(SysProps.JNDIURL));
                ictx = new InitialContext(env);                
            } catch (NamingException ex) {
                throw new TCKException("Please check the properties " + SysProps.JNDIFACTORY
                        + " and " + SysProps.JNDIURL
                        + " of the configuration file in your server configuration", ex);
            } catch (ClassCastException ccex) {
                ccex.printStackTrace();
                
                //thrown by narrow
                throw new TCKException("Client and server in banue classes differ! Please check version numbers",
                        ccex);
            }
        }
        return ictx;
    }
    
    /** @return A new JVTSession interface */
    
    /* VP obsolete use getInstanceOf ....
     *
     
        public JVTSession getJVTSession() throws TCKException
        {
            Thread.currentThread().setContextClassLoader(ClassLoader.getSystemClassLoader());
     
    //VP      com.nokia.oss.ossj.common.ri.ex.JVTActivationHome omHome ;
     
            javax.ejb.EJBHome omHome;
            JVTSession result = null;
     
            try {
                Object obj = this.getInitialContext().lookup(this.sysProps.getProperty(SysProps.JVTSESSION_JNDINAME));
     
                //Class test = (((EJBHome)obj).getEJBMetaData()).getHomeInterfaceClass();
                Class test1 = ((this.getClass()).getClassLoader()).loadClass(this.sysProps.getProperty(SysProps.JVTSESSION_HOME));
                omHome = (javax.ejb.EJBHome)PortableRemoteObject.narrow(obj, test1);
     
                //result = omHome.create();
                result = (JVTSession) omHome.getClass().getMethod("create",null).invoke(omHome,null);
     
            }
            catch( NamingException nex ){ //thrown by lookup
                throw new TCKException( "Please check the property "+SysProps.JVTSESSION_HOME
                                        +" of the configuration file in your server configuration",
                                        nex );
            }
            catch( ClassCastException ccex ){
                ccex.printStackTrace();
                //thrown by narrow
                throw new TCKException( "Client and server classes differ! Please check version numbers", ccex);
            }
    //        catch( CreateException cex ){ //thrown by create
    //            throw new TCKException( "Internal error creating an JVTSession object", cex);
    //        }
    //        catch( java.rmi.RemoteException rex ){ //thrown by create
    //            throw new TCKException( "Communication error creating an JVTSession object", rex);
    //        }
     
            catch(Exception ex) {
             //   System.out.println("hi some unuall exception");
                ex.printStackTrace();
                throw new TCKException( "When Creating JVTSession "+SysProps.JVTSESSION_JNDINAME+" object", ex);
            }
     
           return result;
        }
     
     */
    
    /**
     * return the name of the topic factory from the properties file
     */
    public String getTopicFactoryName() {
        return this.sysProps.getProperty(SysProps.TOPICFACTORYJNDINAME);
    }
    public String getQueueFactoryName() {
        return this.sysProps.getProperty(SysProps.QUEUEFACTORYJNDINAME);
    }
    public String getRequestQueueName() {
        return this.sysProps.getProperty(SysProps.REQUESTQUEUEJNDINAME);
    }
    
    /**
     * return the name of the topic from the properties file
     *
     * @return DOCUMENT ME!
     */
    public String getTopicName() {
        return this.sysProps.getProperty(SysProps.JVTTOPICJNDINAME);
    }
    
    /**
     * Test if the given JNDI name follows:
     * System/System1/ApplicationType/Common/Application/1-1;1-1;ReferenceImplementation/Comp/name
     * or
     * System/System1/ApplicationType/Common/Application/1-1;1-1;ReferenceImplementation/Resource/name
     *
     * @param name the jndi name to test
     *
     * @return null if the name is valid or the text error found
     */
    public String isValidJndiName(String jndiName) {
        //	 System/System1/ApplicationType/Common/Application/1-1;1-1;ReferenceImplementation/Comp
        StringTokenizer context = new StringTokenizer(jndiName, "/");
        String elem = "";
        
        // Skip root env in case
        while (context.hasMoreTokens() && !elem.equals("System")) {
            elem = (String) context.nextToken();
            
            //log("+++> elem = "+elem);
        }
        
        if (!elem.equals("System")) {
            return ("System string not found in the JNDI NAME [" + jndiName + "] ");
        }
        
        if (!context.hasMoreTokens()) {
            return ("Nothing after System in the JNDI NAME [" + jndiName + "] ");
        }
        
        // skip the next element System1
        elem = (String) context.nextToken();
        
        if (!context.hasMoreTokens()) {
            return ("Nothing after System/xxx/ in the JNDI NAME [" + jndiName + "] ");
        }
        
        // ApplicationType
        elem = (String) context.nextToken();
        
        if (!elem.equals("ApplicationType")) {
            return ("ApplicationType string not found after system instance in the JNDI NAME ["
                    + jndiName + "] ");
        }
        
        if (!context.hasMoreTokens()) {
            return ("Nothing after System/xxx/ApplicationType in the JNDI NAME [" + jndiName + "] ");
        }
        
        // skip the next element Common
        elem = (String) context.nextToken();
        
        if (!context.hasMoreTokens()) {
            return ("Nothing after System/xxx/ApplicationType/yyy in the JNDI NAME [" + jndiName
                    + "] ");
        }
        
        // Application
        elem = (String) context.nextToken();
        
        if (!elem.equals("Application")) {
            return ("Application string not found after application type instance in the JNDI NAME ["
                    + jndiName + "] ");
        }
        
        if (!context.hasMoreTokens()) {
            return ("Nothing after System/xxx/ApplicationType/yyy/Application in the JNDI NAME ["
                    + jndiName + "] ");
        }
        
        // test the next element  x-x;y-y;zzz
        elem = (String) context.nextToken();
        
        StringTokenizer app = new StringTokenizer(elem, ";");
        
        if (app.countTokens() != 3) {
            return ("Invalid application name (x-y;s-d;zzz) in the JNDI NAME [" + jndiName + "] ");
        }
        
        if (!context.hasMoreTokens()) {
            return ("Nothing after System/xxx/ApplicationType/yyy/Application/zzz in the JNDI NAME ["
                    + jndiName + "] ");
        }
        
        // Comp
        elem = (String) context.nextToken();
        
        if (!elem.equals("Comp") && !elem.equals("Resource")) {
            return ("Comp or Resource string not found after application type instance in the JNDI NAME ["
                    + jndiName + "] ");
        }
        
        if (!context.hasMoreTokens()) {
            return ("Nothing after System/xxx/ApplicationType/yyy/Application/zzz/[Comp/Resource] in the JNDI NAME ["
                    + jndiName + "] ");
        }
        
        // can test if the token ends with Home here?
        //
        elem = (String) context.nextToken();
        
        if (context.hasMoreTokens()) {
            return ("The JNDI name shall end after System/xxx/ApplicationType/yyy/Application/zzz/[Comp/Resource]/iiiiHome in the JNDI NAME ["
                    + jndiName + "] ");
        }
        
        return null;
    }
    
    /**
     * @return A new TopicSubscriber on a new TopicConnection to the topic SysProps.TOPICNAME
     * Messages sent by this connection are reeived
     */
    public TopicSubscriber getTopicSubscriber() throws Exception {
        return this.getTopicSubscriber(null);
        
        // return null;
    }
    
    /**
     * @param msgSelector The selector for incoming messages
     * @return A new TopicSubscriber on a new TopicConnection to the topic SysProps.TOPICNAME
     * Messages sent by this connection are reeived
     */
    public TopicSubscriber getTopicSubscriber(String msgSelector)
    throws TCKException {
        TopicSubscriber retVal = null;
        
        try {
            //subscribe to JMS
            InitialContext ictx = this.getInitialContext();
            Object obj = ictx.lookup(this.sysProps.getProperty(SysProps.TOPICFACTORYJNDINAME));
            TopicConnectionFactory tconFactory = (TopicConnectionFactory) PortableRemoteObject
                    .narrow(obj, TopicConnectionFactory.class);
            
            TopicConnection tcon = tconFactory.createTopicConnection();
            TopicSession tSession = tcon.createTopicSession(false, Session.CLIENT_ACKNOWLEDGE);
            
            Topic topic = null;
            
            try {
                obj = ictx.lookup(this.sysProps.getProperty(SysProps.JVTTOPICJNDINAME));
                topic = (Topic) PortableRemoteObject.narrow(obj, Topic.class);
            } catch (NamingException ne) {
                topic = tSession.createTopic(this.sysProps.getProperty(SysProps.JVTTOPICJNDINAME));
                ictx.bind(this.sysProps.getProperty(SysProps.JVTTOPICJNDINAME), topic);
            } catch (ClassCastException ccex) { //thrown from narrow
                throw new TCKException("Client and server classes differ! Please check version numbers",
                        ccex);
            }
            
            retVal = tSession.createSubscriber(topic, msgSelector, false); // TCK does not publish anyway
            
            // start the connection so we can receive messages
            tcon.start();
        } catch (NamingException nex) { //thrown by lookup
            throw new TCKException("Please check the property " + SysProps.TOPICFACTORYJNDINAME
                    + " of the configuration file in your server configuration", nex);
        } catch (ClassCastException ccex) { //thrown by narrow
            throw new TCKException("Client and server classes differ! Please check version numbers",
                    ccex);
        } catch (JMSSecurityException jsex) { // thrown by createTopicConnection
            throw new TCKException("JMS username or password are invalid. Please check your server configuration",
                    jsex);
        } catch (InvalidDestinationException idex) { // thrown by createSubscriber
            throw new TCKException("Please check the property " + SysProps.JVTTOPICJNDINAME
                    + " of the configuration file in your server configuration", idex);
        } catch (InvalidSelectorException isex) { // thrown by createSubscriber
            throw new TCKException("'" + msgSelector + "' is and invalid message selector", isex);
        } catch (JMSException jex) {
            jex.printStackTrace();
            throw new TCKException("Internal JMS error", jex);
        }
        
        return retVal;
    }
    
    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws TCKException DOCUMENT ME!
     */
    public QueueSession createQueueSession() throws TCKException {
        if (qsession == null) {
            try {
                InitialContext ictx = this.getInitialContext();
                Object obj = ictx.lookup(this.sysProps.getProperty(SysProps.QUEUEFACTORYJNDINAME));
                QueueConnectionFactory queueConFactory = (QueueConnectionFactory) PortableRemoteObject
                        .narrow(obj, QueueConnectionFactory.class);
                
                QueueConnection qcon = queueConFactory.createQueueConnection();
                qsession = qcon.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
                qcon.start();
            } catch (NamingException nex) { //thrown by lookup
                throw new TCKException("Please check the property " + SysProps.QUEUEFACTORYJNDINAME
                        + " of the configuration file in your server configuration", nex);
            } catch (JMSException jex) {
                throw new TCKException("Internal JMS error", jex);
            }
        }
        
        return qsession;
    }
    
    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws TCKException DOCUMENT ME!
     */
    public QueueSender createQueueSender() throws TCKException {
        QueueSender retVal = null;
        
        try {
            // subscribe to JMS
            QueueSession qSession = createQueueSession();
            Queue queue = null;
            
            try {
                Object obj = ictx.lookup(this.sysProps.getProperty(SysProps.REQUESTQUEUEJNDINAME));
                queue = (Queue) PortableRemoteObject.narrow(obj, Queue.class);
            } catch (ClassCastException ccex) { //thrown from narrow
                throw new TCKException("Client and server classes differ! Please check version numbers",
                        ccex);
            }
            
            retVal = qSession.createSender(queue);
            
            // start the connection so we can send messages
            //            qcon.start();
        } catch (NamingException nex) { //thrown by lookup
            throw new TCKException("Please check the property " + SysProps.QUEUEFACTORYJNDINAME
                    + " of the configuration file in your server configuration", nex);
        } catch (ClassCastException ccex) { //thrown by narrow
            throw new TCKException("Client and server classes differ! Please check version numbers",
                    ccex);
        } catch (JMSSecurityException jsex) { // thrown by createTopicConnection
            throw new TCKException("JMS username or password are invalid. Please check your server configuration",
                    jsex);
        } catch (InvalidDestinationException idex) { // thrown by createSubscriber
            throw new TCKException("Please check the property " + SysProps.REQUESTQUEUEJNDINAME
                    + " of the configuration file in your server configuration", idex);
        } catch (JMSException jex) {
            throw new TCKException("Internal JMS error", jex);
        }
        
        return retVal;
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param msgSelector DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws TCKException DOCUMENT ME!
     */
    public QueueReceiver createQueueReceiver(String msgSelector)
    throws TCKException {
        QueueReceiver retVal = null;
        
        try {
            //subscribe to JMS
            QueueSession qSession = createQueueSession();
            
            Queue queue = null;
            
            try {
                Object obj = ictx.lookup(this.sysProps.getProperty(SysProps.RESPONSEQUEUEJNDINAME));
                queue = (Queue) PortableRemoteObject.narrow(obj, Queue.class);
            } catch (ClassCastException ccex) { //thrown from narrow
                throw new TCKException("Client and server classes differ! Please check version numbers",
                        ccex);
            }
            
            retVal = qSession.createReceiver(queue, msgSelector);
            
            // start the connection so we can receive messages
            //            qcon.start();
        } catch (NamingException nex) { //thrown by lookup
            throw new TCKException("Please check the property " + SysProps.QUEUEFACTORYJNDINAME
                    + " of the configuration file in your server configuration", nex);
        } catch (ClassCastException ccex) { //thrown by narrow
            throw new TCKException("Client and server classes differ! Please check version numbers",
                    ccex);
        } catch (JMSSecurityException jsex) { // thrown by createTopicConnection
            throw new TCKException("JMS username or password are invalid. Please check your server configuration",
                    jsex);
        } catch (InvalidDestinationException idex) { // thrown by createSubscriber
            throw new TCKException("Please check the property " + SysProps.RESPONSEQUEUEJNDINAME
                    + " of the configuration file in your server configuration", idex);
        } catch (InvalidSelectorException isex) { // thrown by createSubscriber
            throw new TCKException("'" + msgSelector + "' is and invalid message selector", isex);
        } catch (JMSException jex) {
            throw new TCKException("Internal JMS error", jex);
        }
        
        return retVal;
    }
    
    
    
    /**@return The intersection of first and second*/
    public static String[] intersect(String[] first, String[] second) {
        if ((null == first) || (null == second)) {
            return new String[0];
        }
        
        String[] clone = null;
        String[] src = null;
        
        if (first.length < second.length) {
            clone = (String[]) second.clone();
            src = first;
        } else {
            clone = (String[]) first.clone();
            src = second;
        }
        
        ArrayList intersect = new ArrayList(src.length);
        Arrays.sort(clone);
        
        for (int i = 0; i < src.length; ++i) {
            if (0 <= Arrays.binarySearch(clone, src[i])) {
                intersect.add(src[i]);
            }
        }
        
        return (String[]) intersect.toArray(new String[0]);
    }
    
    /**
     * @param subSet The subset
     * @param superSet The superset
     * @return true if both parameters are not null and subSet is subset of superSet
     */
    public static boolean isSubset(String[] subSet, String[] superSet) {
        if ((null == subSet) || (null == superSet)) {
            return false;
        }
        
        String[] superClone = (String[]) superSet.clone();
        Arrays.sort(superClone);
        
        for (int i = 0; i < subSet.length; ++i) {
            if (0 > Arrays.binarySearch(superClone, subSet[i])) {
                return false;
            }
        }
        
        return true;
    }
    
    /**
     * gets first minus second i.e. all Strings of first which are not in second
     * @param first
     * @param second
     * @return The difference of first and second
     */
    public static String[] diff(String[] first, String[] second) {
        if (null == first) {
            first = new String[0];
        }
        
        String[] intersect = Util.intersect(first, second);
        Arrays.sort(intersect);
        
        String[] retVal = new String[first.length - intersect.length];
        int insert = 0;
        
        for (int i = 0; i < first.length; ++i) {
            if (0 > Arrays.binarySearch(intersect, first[i])) {
                retVal[insert++] = first[i];
            }
        }
        
        return retVal;
    }
    
    /** @return if a contains key
     * nota bene:all buckets of a must not be null
     * @throws java.lang.IllegalArgumentException If an argument is null
     */
    public static boolean contains(Object[] a, Object key)
    throws java.lang.IllegalArgumentException {
        if ((a == null) || (key == null)) {
            throw new java.lang.IllegalArgumentException("Null argument");
        }
        
        int i;
        
        for (i = 0; (i < a.length) && !key.equals(a[i]); ++i)
            ;
        
        return i < a.length;
    }
    
    /**
     *  @param a
     *  @param key
     *   @return if a contains key
     *   @throws java.lang.IllegalArgumentException If an argument is null
     */
    public static boolean contains(int[] a, int key) throws java.lang.IllegalArgumentException {
        if (null == a) {
            throw new java.lang.IllegalArgumentException("Null argument");
        }
        
        int i;
        
        for (i = 0; (i < a.length) && (key != a[i]); ++i)
            ;
        
        return i < a.length;
    }
    
    /**
     * @param list List of Keys
     * @param key The Key to be looked for
     * @return Index where key was found, -1 otherwise
     */
    public static int find(ArrayList list, ManagedEntityKey key) {
        return Util.find((ManagedEntityKey[]) list.toArray(new ManagedEntityKey[] {  }), key);
    }
    
    /**
     * @param v Vector of Keys
     * @param key The Key to be looked for
     * @return Index where key was found, -1 otherwise
     */
    public static int find(Vector v, ManagedEntityKey key) {
        return Util.find((ManagedEntityKey[]) v.toArray(new ManagedEntityKey[] {  }), key);
    }
    
    /**
     * @param list List of Keys
     * @param key The Key to be looked for
     * @return Index where key was found, -1 otherwise
     */
    public static int find(ManagedEntityKey[] keys, ManagedEntityKey key) {
        int i;
        
        for (i = 0; i < keys.length; ++i) {
            ManagedEntityKey other = keys[i];
            
            if (Util.equals(other, key)) {
                break;
            }
        }
        
        return (i < keys.length) ? i : (-1);
    }
    
    /**
     * @param keys
     * @param key
     * @return If the keys contains duplicates of key
     */
    public static boolean containsDuplicates(Vector v, ManagedEntityKey key) {
        return Util.containsDuplicates((ManagedEntityKey[]) v.toArray(new ManagedEntityKey[] {  }),
                key);
    }
    
    /**
     * @param keys
     * @param key
     * @return If the keys contains duplicates of key
     */
    private static boolean containsDuplicates(ManagedEntityKey[] keys, ManagedEntityKey key) {
        int i;
        
        for (i = 0; (i < keys.length) && !Util.equals(keys[i], key); ++i)
            ;
        
        for (; (i < keys.length) && !Util.equals(keys[i], key); ++i)
            ;
        
        return i < keys.length;
    }
    
    /**
     *makes a comma separated list of the items String representation
     *@param items The items
     *@return The comma separated list
     */
    public static String makeCSList(Object[] items) {
        StringBuffer bug = new StringBuffer();
        
        for (int i = 0; i < items.length; ++i) {
            bug.append(items[i].toString() + ", ");
        }
        
        bug.setLength(bug.length() - 2);
        
        return bug.toString();
    }
    
    /**performs a deep copy on src
     * @param src The source to be copied
     * @return The deep copy
     */
    public String[] deepCopy(String[] src) {
        String[] retVal = new String[src.length];
        
        for (int i = 0; i < src.length; ++i) {
            retVal[i] = new String(src[i]);
        }
        
        return retVal;
    }
    
    /**
     * shuffles the array
     * @throws IllegalArgumentException If a is null
     */
    public void shuffle(Object[] a) throws java.lang.IllegalArgumentException {
        if (null == a) {
            throw new java.lang.IllegalArgumentException("Null argument");
        }
        
        Object obj;
        int index;
        
        for (int i = 0; i < (a.length - 1); ++i) {
            index = this.RANDOM.nextInt(a.length - i);
            obj = a[i];
            a[i] = a[index + i];
            a[index + i] = obj;
        }
    }
    
    /**
     *@param set The set of strings
     *@return A random subset of set
     */
    public String[] randomSubset(String[] set) {
        return (String[]) this.randomSubset((Object[]) set).toArray(new String[] {  });
    }
    
    /**
     *@param set The set of String
     *@return A random subset of set
     */
    public String[] randomSubsetNotEmpty(String[] set) {
        return (String[]) this.randomSubsetNotEmpty((Object[]) set).toArray(new String[] {  });
    }
    
    /**
     *@param set The set
     *@return A random subset of set
     */
    public ArrayList randomSubsetNotEmpty(Object[] set) {
        ArrayList retVal = new ArrayList();
        
        if ((null != set) && (0 < set.length)) {
            int count = Math.max(1, this.RANDOM.nextInt(set.length));
            Object[] clone = (Object[]) set.clone();
            this.shuffle(clone);
            
            for (int i = 0; i < count; ++i) {
                retVal.add(clone[i]);
            }
        }
        
        return retVal;
    }
    
    /**@return A random subset of set*/
    private ArrayList randomSubset(Object[] set) {
        ArrayList retVal = new ArrayList();
        
        if ((null != set) && (0 < set.length)) {
            int count = this.RANDOM.nextInt(set.length);
            Object[] clone = (Object[]) set.clone();
            this.shuffle(clone);
            
            for (int i = 0; i < count; ++i) {
                retVal.add(clone[i]);
            }
        }
        
        return retVal;
    }
    
    /**
     * @param The class to get the minimum from
     * @return The minimum wrapped in the related java class
     * @throws IllegalArgumentException If !cls.IsPrimitive()
     */
    public static Object getMinAsObject(Class cls) throws java.lang.IllegalArgumentException {
        if ((null == cls) || !cls.isPrimitive()) {
            throw new java.lang.IllegalArgumentException("Class is not primitve or null");
        }
        
        Object retVal = null;
        
        if (cls.equals(Byte.TYPE)) {
            retVal = new Byte(Byte.MIN_VALUE);
        } else if (cls.equals(Character.TYPE)) {
            retVal = new Character(Character.MIN_VALUE);
        } else if (cls.equals(Boolean.TYPE)) {
            retVal = new Boolean(false);
        } else if (cls.equals(Short.TYPE)) {
            retVal = new Short(Short.MIN_VALUE);
        } else if (cls.equals(Integer.TYPE)) {
            retVal = new Integer(Integer.MIN_VALUE);
        } else if (cls.equals(Long.TYPE)) {
            retVal = new Long(Long.MIN_VALUE);
        } else if (cls.equals(Float.TYPE)) {
            retVal = new Float(Float.MIN_VALUE);
        } else if (cls.equals(Double.TYPE)) {
            retVal = new Double(Double.MIN_VALUE);
        }
        
        return retVal;
    }
    
    /**
     * @param The class to get the maximum from
     * @return The maximum wrapped in the related java class
     * @throws java.lang.IllegalArgumentException If !cls.IsPrimitive()
     */
    public static Object getMaxAsObject(Class cls) throws java.lang.IllegalArgumentException {
        if ((null == cls) || !cls.isPrimitive()) {
            throw new java.lang.IllegalArgumentException("Class is not primitive");
        }
        
        Object retVal = null;
        
        if (cls.equals(Byte.TYPE)) {
            retVal = new Byte(Byte.MAX_VALUE);
        } else if (cls.equals(Character.TYPE)) {
            retVal = new Character(Character.MAX_VALUE);
        } else if (cls.equals(Boolean.TYPE)) {
            retVal = new Boolean(true);
        } else if (cls.equals(Short.TYPE)) {
            retVal = new Short(Short.MAX_VALUE);
        } else if (cls.equals(Integer.TYPE)) {
            retVal = new Integer(Integer.MAX_VALUE);
        } else if (cls.equals(Long.TYPE)) {
            retVal = new Long(Long.MAX_VALUE);
        } else if (cls.equals(Float.TYPE)) {
            retVal = new Float(Float.MAX_VALUE);
        } else if (cls.equals(Double.TYPE)) {
            retVal = new Double(Double.MAX_VALUE);
        }
        
        return retVal;
    }
    
    /** gets a value which is different from all input values and not null
     * input array members must not be null
     *@param vals The values to be different from
     *@param count The number of values to return
     *@return The different values
     */
    public static String[] getIllegalValue(String[] vals, int count)
    throws java.lang.IllegalArgumentException {
        if ((0 > count) || (null == vals) || (0 == vals.length)) {
            // Fix Issue #52 and #53
            //throw new java.lang.IllegalArgumentException("Null argument or negative count");
            String[] invalid = new String[count];
            for (int i = count-1;i>=0;i--){
                invalid[i] = "invalid_string_"+i+"_"+i*i;
            }
            return invalid;
        }
        
        String[] retVal = new String[count];
        String[] clone = (String[]) vals.clone();
        Arrays.sort(clone);
        
        int last = clone.length - 1;
        
        for (int i = 0; i < count; ++i) {
            retVal[i] = clone[last] + "_" + clone[last];
            
            for (int j = 0; j < i; j++) {
                retVal[j] += ("_" + clone[last]);
            }
        }
        
        return retVal;
    }
    
    /** gets a value which is different from all input values and not null
     * input array members must not be null
     *@param vals The values to be different from
     *@param count The number of values to return
     *@return The different values
     */
    public static int[] getIllegalValue(int[] vals, int count)
    throws java.lang.IllegalArgumentException {
        if ((0 > count) || (null == vals) || (0 == vals.length)) {
            //FIx issue #52 and #53
            //throw new java.lang.IllegalArgumentException("Null argument or negative count");
            int[] invalid = new int[count];
            for (int i = count -1; i>= 0 ; i--) {
                invalid[i] = i*i;
            }
            return invalid;
        }
        
        int[] retVal = new int[count];
        int[] clone = (int[]) vals.clone();
        Arrays.sort(clone);
        
        int last = clone.length - 1;
        
        for (int i = 0; i < count; ++i) {
            retVal[i] = clone[last] + clone[last];
            
            for (int j = 0; j < i; j++) {
                retVal[j] += clone[last];
            }
        }
        
        return retVal;
    }
    
    /** @return A random Date*/
    public Date randomDate() {
        Date retVal = new Date();
        long s = retVal.getTime();
        retVal.setTime(s + (long) (this.RANDOM.nextGaussian() * this.getTotalTimeout()));
        
        return retVal;
    }
    
    /** @return A random String which is not null*/
    public String randomString() {
        int length = this.RANDOM.nextInt(this.MAXSTRINGLENGTH);
        byte[] b = new byte[length];
        this.RANDOM.nextBytes(b);
        
        return new String(b);
    }
    
    /**
     * Creates a random value for the given attribute name and ManagedEntityValue
     * So far, Date and String are the only non primitve types supported.
     * Other non primitve type must be creatable via newInstance
     * @param name The name of the attribute
     * @param mev The ManagedEntityValue
     * @return The randomly created value
     */
    public Object randomValue(String name, AttributeAccess mev)
    throws Exception {
        Class cls = this.getAttributeType(mev, name);
        
        return randomValueInstance(cls);
    }
    
    /**
     * Creates a random value for the given ManagedEntityValue
     * @param mev The ManagedEntityValue
     * @return The randomly created ManagedEntityValue
     */
    public AttributeAccess randomManagedEntityValueAtrributes(AttributeAccess mev)
    throws Exception {
        String[] names = mev.getAttributeNames();
        
        for (int i = names.length - 1; i >= 0; i--)
            if (!names[i].equals(ManagedEntityValue.KEY)) {
            mev.setAttributeValue(names[i], randomValueInstance(getAttributeType(mev, names[i])));
            }
        
        return mev;
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param cls DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws Exception DOCUMENT ME!
     */
    public MultiValueList randomMultiValue(Class cls) throws Exception {
        Object mvl;
        
        if (cls.isInterface()) {
            mvl = randomValueInstance((Class) (findImplementationOf(cls).get(0)));
        } else {
            mvl = randomValueInstance(cls);
        }
        
        // now populate it using set
        Method[] meths = mvl.getClass().getMethods();
        int i;
        
        for (i = meths.length - 1; (i >= 0) && !"set".equals(meths[i].getName()); i--)
            ;
        
        Class param = meths[i].getParameterTypes()[0];
        
        Object values = randomValueInstance(param);
        meths[i].invoke(mvl, new Object[] { values });
        
        return (MultiValueList) mvl;
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param cls DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws Exception DOCUMENT ME!
     */
    public Object randomValueInstance(Class cls) throws Exception {
        Object retVal = null;
        
        if (cls == null) {
            return null;
        }
        
        if (cls.isInterface()) {
            if (MultiValueList.class.isAssignableFrom(cls)) {
                return randomMultiValue(cls);
            } else {
                
                if (findImplementationOf(cls).size() == 0)
                    throw new Exception("Unable to find or instantiate valid implementation for interface " + cls);
                
                retVal = randomValueInstance((Class) (findImplementationOf(cls).get(0)));
                
                if (retVal == null) {
                    throw new Exception(
                            "Unable to find or instantiate valid implementation for interface " + cls);
                } else {
                    return retVal;
                }
            }
        } else if (cls.isArray()) {
            int numberOfElement = 1; // use only one element to avoid duplication in array
            Object[] retArray = null;
            retArray = (Object[]) Array.newInstance(cls.getComponentType(), numberOfElement);
            while (--numberOfElement >= 0)
                retArray[numberOfElement] = randomValueInstance(cls.getComponentType());
            
            return retArray;
            
        } else if (Date.class.equals(cls)) {
            retVal = randomDate();
        } else if (String.class.equals(cls)) {
            retVal = randomString();
        } else if (cls.equals(Integer.TYPE) || java.lang.Integer.class.isAssignableFrom(cls)) {
            retVal = new Integer(this.RANDOM.nextInt());
        } else if (cls.equals(Long.TYPE) || java.lang.Long.class.isAssignableFrom(cls)) {
            retVal = new Long(this.RANDOM.nextLong());
        } else if (cls.equals(Byte.TYPE) || java.lang.Byte.class.isAssignableFrom(cls)) {
            retVal = new Byte((byte) this.RANDOM.nextInt());
        } else if (cls.equals(Short.TYPE) || java.lang.Short.class.isAssignableFrom(cls)) {
            retVal = new Short((short) this.RANDOM.nextInt());
        } else if (cls.equals(Double.TYPE) || java.lang.Double.class.isAssignableFrom(cls)) {
            retVal = new Double(this.RANDOM.nextDouble());
        } else if (cls.equals(Float.TYPE) || java.lang.Float.class.isAssignableFrom(cls)) {
            retVal = new Float(this.RANDOM.nextFloat());
        } else if (cls.equals(Character.TYPE) || java.lang.Character.class.isAssignableFrom(cls)) {
            retVal = new Character((char) this.RANDOM.nextInt());
        } else if (cls.equals(Boolean.TYPE) || java.lang.Boolean.class.isAssignableFrom(cls)) {
            retVal = new Boolean(this.RANDOM.nextBoolean());
        } else if (!cls.isPrimitive()) {
            try {
                retVal = cls.newInstance();
            } catch (Exception e) {
                System.out.println("--> Unable to instanciate randomValue for cls=" + cls + "name="
                        + cls.getName());
                throw e;
            }
        }
        
        return retVal;
    }
    
    /**  Checks two MEK for per-field equality. If any parameter is null, false is returned
     * @param first
     * @param second
     * @return If both parameters are not null and field wise equal.
     */
    private static boolean equals(ManagedEntityKey first, ManagedEntityKey second) {
        if ((null == first) || (null == second)) {
            return false;
        } else {
            return first.toString().equals(second.toString());
        }
    }
    
    /**
     * tests for equality
     * @param first
     * @param second
     * @return if populated attribute names and their values are equal.
     * If any parameter is null, false is returned
     */
    private static boolean equals(ManagedEntityValue first, ManagedEntityValue second)
    throws java.lang.IllegalArgumentException {
        return 0 == Util.compare(first, second).length;
    }
    
    /**
     * @param first
     * @param second
     * @return All attributes first and second are equal
     */
    private static String[] equal(ManagedEntityValue first, ManagedEntityValue second) {
        ArrayList retVal = new ArrayList();
        String[] pops = first.getPopulatedAttributeNames();
        
        for (int i = 0; i < pops.length; ++i) {
            if (second.isPopulated(pops[i])
            && first.getAttributeValue(pops[i]).equals(second.getAttributeValue(pops[i]))) {
                retVal.add(pops[i]);
            }
        }
        
        return (String[]) retVal.toArray(new String[] {  });
    }
    
    /**
     * gets the difference between first and second
     * @param first
     * @param second
     * @return All attributes which are different in the two MEV.
     * Populated/Unpopulated and vice versa is treated as different
     */
    private static String[] compare(ManagedEntityValue first, ManagedEntityValue second)
    throws java.lang.IllegalArgumentException {
        ArrayList list = new ArrayList(Arrays.asList(Util.comparePopulatedAttributeNames(first,
                second)));
        
        String[] intersect = Util.intersect(first.getPopulatedAttributeNames(),
                second.getPopulatedAttributeNames());
        
        for (int i = 0; i < intersect.length; ++i) {
            Object obj = first.getAttributeValue(intersect[i]);
            
            if (obj instanceof ManagedEntityValue) {
                String[] diff = Util.compare((ManagedEntityValue) first.getAttributeValue(
                        intersect[i]),
                        (ManagedEntityValue) second.getAttributeValue(intersect[i]));
                
                if (0 != diff.length) {
                    list.add(intersect[i]);
                }
            } else if (obj instanceof ManagedEntityValue[]) {
                ManagedEntityValue[] aFirst = (ManagedEntityValue[]) first.getAttributeValue(intersect[i]);
                ManagedEntityValue[] aSecond = (ManagedEntityValue[]) second.getAttributeValue(intersect[i]);
                
                if (aFirst.length == aSecond.length) {
                    int k = 0;
                    
                    for (k = 0; (k < aFirst.length) && Util.equals(aFirst[k], aSecond[k]); ++k)
                        ;
                    
                    if (k < aFirst.length) {
                        list.add(intersect[i]);
                    }
                } else {
                    list.add(intersect[i]);
                }
            } else if (obj instanceof ManagedEntityKey) {
                if (!Util.equals((ManagedEntityKey) first.getAttributeValue(intersect[i]),
                        (ManagedEntityKey) second.getAttributeValue(intersect[i]))) {
                    list.add(intersect[i]);
                }
            } else {
                Object firstVal = first.getAttributeValue(intersect[i]);
                Object secondVal = second.getAttributeValue(intersect[i]);
                
                if (null == firstVal) {
                    if (null != secondVal) {
                        list.add(intersect[i]);
                    }
                } else {
                    if ((null == secondVal) || !firstVal.equals(secondVal)) {
                        list.add(intersect[i]);
                    }
                }
            }
        }
        
        return (String[]) list.toArray(new String[0]);
    }
    
    /**special preparation, see spec of t for details*/
    private static void prepareRemoveEventMEV(ManagedEntityValue monitoredMEV,
            ManagedEntityValue removeEventMEV) throws java.lang.IllegalArgumentException {
        if ((null == monitoredMEV) || (null == removeEventMEV)) {
            throw new java.lang.IllegalArgumentException("Null argument");
        }
        
        String[] eventPops = removeEventMEV.getPopulatedAttributeNames();
        
        for (int i = 0; i < eventPops.length; ++i) {
            if (!monitoredMEV.isPopulated(eventPops[i])
            && (null == removeEventMEV.getAttributeValue(eventPops[i]))) {
                removeEventMEV.unpopulateAttribute(eventPops[i]);
            }
        }
    }
    
    /**
     * @param first
     * @param second
     * @return String[] list of attributes populated in only one of the values
     * but not in second
     */
    private static String[] comparePopulatedAttributeNames(ManagedEntityValue first,
            ManagedEntityValue second) throws java.lang.IllegalArgumentException {
        if ((null == first) || (null == second)) {
            throw new java.lang.IllegalArgumentException("Arguments must not be null");
        }
        
        String[] firstPops = first.getPopulatedAttributeNames();
        String[] secondPops = second.getPopulatedAttributeNames();
        
        ArrayList list = new ArrayList(Arrays.asList(Util.diff(firstPops, secondPops)));
        list.addAll(Arrays.asList(Util.diff(secondPops, firstPops)));
        
        return (String[]) list.toArray(new String[0]);
    }
    
    /**
     * @param first
     * @param second
     * @return String[] list of attributes populated in first but not in second
     * but not in second
     */
    private static String[] diffPopulatedAttributes(ManagedEntityValue first,
            ManagedEntityValue second) throws java.lang.IllegalArgumentException {
        if ((null == first) || (null == second)) {
            throw new java.lang.IllegalArgumentException("Arguments must not be null");
        }
        
        String[] firstPops = first.getPopulatedAttributeNames();
        String[] secondPops = second.getPopulatedAttributeNames();
        
        return Util.diff(firstPops, secondPops);
    }
    
    /**
     * copies attributes which are populated in second but not in first
     * from second to first.
     * @param first
     * @param second
     */
    private static void copyPopDiff(ManagedEntityValue first, ManagedEntityValue second) {
        String[] popDiff = Util.diffPopulatedAttributes(second, first);
        
        for (int i = 0; i < popDiff.length; ++i) {
            first.setAttributeValue(popDiff[i], second.getAttributeValue(popDiff[i]));
        }
    }
    
    /**
     * @param first
     * @param second
     * @return If both MEVs are of the same type
     */
    public static boolean isSameType(ManagedEntityValue first, ManagedEntityValue second) {
        return first.makeManagedEntityKey().getType().equals(second.makeManagedEntityKey().getType());
    }
    
    /**@return A Hashtable containing a legal ( but removed ) Key per Type*/
    public Hashtable getManagedEntityKeys(JVTSession jas)
    throws Exception {
        return this.makeManagedEntityKeys(jas);
    }
    
    /**@return A new Hashtable containing a legal ( but removed ) Key per Type*/
    private synchronized Hashtable makeManagedEntityKeys(JVTSession jas)
    throws Exception {
        //VP        if( managedKeys == null)
        //VP        {
        //VP managedKeys = new Hashtable(this.MANAGED_TYPES.length);
        managedKeys = new Hashtable();
        
        //VP JVTSession jas = this.getJVTSession();
        String[] mgTypes = jas.getManagedEntityTypes();
        
        for (int i = 0; i < mgTypes.length; i++) {
            ManagedEntityKey key = jas.makeManagedEntityValue(mgTypes[i]).makeManagedEntityKey();
            
            //VP                managedKeys.put( this.MANAGED_TYPES[i], key);
            managedKeys.put(mgTypes[i], key);
        }
        
        //VP        }
        //VP            return (Hashtable)managedKeys.clone();
        return managedKeys;
    }
    
    /**return the maximum of elements per bulk operation*/
    
    /* public int getMaxBulkElements(){
         return MAX_BULK_ELEMENTS;
     }
     */
    
    /**@return a Hashtable of Vector of illegal Keys per Type*/
    public Vector getIllegalKeys(ManagedEntityValue mev)
    throws Exception {
        ManagedEntityKey validKey = (ManagedEntityKey) mev.makeManagedEntityKey();
        Vector v = new Vector();
        
        ManagedEntityKey ok2 = (ManagedEntityKey) validKey.clone();
        ok2.setType("");
        v.add(ok2);
        
        ManagedEntityKey ok3 = (ManagedEntityKey) validKey.clone();
        ok3.setType(StringTable.INVALIDATION_PREFIX + ok3.getType());
        v.add(ok3);
        
        ManagedEntityKey ok5 = (ManagedEntityKey) validKey.clone();
        ok5.setPrimaryKey(new Object());
        v.add(ok5);
        
        ManagedEntityKey ok6 = (ManagedEntityKey) validKey.clone();
        ok6.setPrimaryKey(new Object());
        ok6.setType(StringTable.INVALIDATION_PREFIX + ok6.getType());
        v.add(ok6);
        
        return v;
    }
    
    /**
     * @param a Array to select from
     * @return Random element of the array. null if array is empty of null
     */
    public Object randomElement(Object[] a) {
        if ((null == a) || (0 == a.length)) {
            return null;
        }
        
        return a[this.RANDOM.nextInt(a.length)];
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param prop DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getProperty(String prop) {
        return this.sysProps.getProperty(prop);
    }
    
    /*
        private Class findFirstImplOfInterface(Class myInterface) {
            Class c = (Class) classImpl.get(myInterface);
     
            if (c != null) {
                return c;
            }
     
            StringTokenizer tokens = new StringTokenizer(System.getProperty("java.class.path"),
                    File.pathSeparator);
            String currSegment;
            File currFile;
     
            while (tokens.hasMoreTokens()) {
                currSegment = tokens.nextToken();
                currFile = new File(currSegment);
     
                if (currFile.exists()) {
                    if (currFile.isDirectory()) {
                        c = findInDirectory(myInterface, currFile);
                    } else {
                        c = findInJar(myInterface, currFile);
                    }
     
                    //System.err.println("VP>"+currFile+" done with c="+c+"\n");
                    if (c != null) {
                        //System.err.println("VP> found "+c.getName()+" for"+myInterface+"\n");
                        classImpl.put(myInterface, c);
     
                        return c;
                    }
                }
            }
     
            return null;
        }
     
        private Class findInDirectory(Class myInterface, File f) {
            Class c = null;
            File f2;
            String[] list = f.list();
     
            for (int i = 0; i < list.length; i++) {
                f2 = new File(f.getPath() + File.separator + list[i]);
     
                if (f2.isDirectory()) {
                    findInDirectory(myInterface, f2);
     
                    // now add this directory only if it contains children
                } else {
                    if (list[i].endsWith(".class")) {
                        String className = list[i].substring(0, list[i].lastIndexOf('.')).replace('/',
                                '.');
     
                        try {
                            c = Class.forName(className);
                        } catch (Throwable e) {
                            continue;
                        }
     
                        if (!c.isInterface() && isCastableWith(c, myInterface.getName())) {
                            return c;
                        }
                    }
                }
            }
     
            return null;
        }
     
        private Class findInJar(Class myInterface, File f) {
            Class cl = null;
            String className = null;
     
            try {
                ZipInputStream zipFile = new ZipInputStream(new BufferedInputStream(
                            new FileInputStream(f)));
                ZipEntry file;
     
                //------
                while ((file = zipFile.getNextEntry()) != null) {
                    if ((file.isDirectory())
                            || (file.getName().charAt(file.getName().length() - 1) == '/')) {
                        // skip it - we'll parse the dir info from the filename
                    } else {
                        if (file.getName().endsWith(".class")) {
                            className = file.getName().substring(0, file.getName().lastIndexOf('.'))
                                            .replace('/', '.');
     
                            try {
                                cl = Class.forName(className);
                            } catch (Throwable e) {
                                continue;
                            }
     
                            //System.err.println("VP> test "+className+" for "+myInterface.getName());
                            if (!cl.isInterface() && isCastableWith(cl, myInterface.getName())) {
                                //System.err.println("VP> found "+cl.getClass().getName()+" for "+myInterface.getName());
                                return cl;
                            }
                        }
                    }
     
                    zipFile.closeEntry();
                }
     
                zipFile.close();
            } catch (Throwable ex) {
                //                                System.err.println("VP>"+className+" Exception "+ex);
                //                                ex.printStackTrace();
            }
     
            //------
            return null;
        }
     */
    public ArrayList findInterfaceStartsWith(String s) {
        ArrayList interfaces = new ArrayList();
        
        StringTokenizer tokens = new StringTokenizer(System.getProperty("java.class.path"),
                File.pathSeparator);
        File currFile;
        String interfaceName = null;
        
        while (tokens.hasMoreTokens()) {
            currFile = new File((String) tokens.nextToken());
            
            if (currFile.exists()) {
                if (currFile.isDirectory()) {
                    interfaces = findInterfaceStartsWithInDirectory(interfaces, s, currFile);
                } else {
                    interfaces = findInterfaceStartsWithInJar(interfaces, s, currFile);
                }
            }
        }
        
        return interfaces;
    }
    
    private ArrayList findInterfaceStartsWithInDirectory(ArrayList interfaces, String s, File f) {
        File f2;
        String[] list = f.list();
        
        for (int i = 0; i < list.length; i++) {
            f2 = new File(f.getPath() + File.separator + list[i]);
            
            if (f2.isDirectory()) {
                findInterfaceStartsWithInDirectory(interfaces, s, f2);
                
                // now add this directory only if it contains children
            } else {
                if (list[i].endsWith(".class")) {
                    String className = list[i].substring(0, list[i].lastIndexOf('.')).replace('/',
                            '.');
                    
                    if (className.startsWith(s)) {
                        // assume that javax.oss.cbe contains only interfaces
                        if (!interfaces.contains(className)) {
                            //log("++> Add interface "+className);
                            interfaces.add(className);
                        }
                    }
                }
            }
        }
        
        return interfaces;
    }
    
    private ArrayList findInterfaceStartsWithInJar(ArrayList interfaces, String s, File f) {
        String className = null;
        
        // skip none ossj jar
        // TODO: fix this the jar search could be a property....
        if (!(f.getName().startsWith("oss") || f.getName().startsWith("common"))) {
            log("++> skip jar " + f.getName());
            return interfaces;
        }
        
        log("++> look in jar " + f.getName());
        
        try {
            ZipInputStream zipFile = new ZipInputStream(new BufferedInputStream(
                    new FileInputStream(f)));
            ZipEntry file;
            
            //------
            while ((file = zipFile.getNextEntry()) != null) {
                if ((file.isDirectory())
                || (file.getName().charAt(file.getName().length() - 1) == '/')) {
                    // skip it - we'll parse the dir info from the filename
                } else {
                    if (file.getName().endsWith(".class")) {
                        className = file.getName().substring(0, file.getName().lastIndexOf('.'))
                        .replace('/', '.');
                        
                        if (className.startsWith(s)) {
                            // assume that javax.oss.cbe contains only interfaces
                            if (!interfaces.contains(className)) {
                                //log("++> Add interface "+className);
                                interfaces.add(className);
                            }
                        }
                    }
                }
                
                zipFile.closeEntry();
            }
            
            zipFile.close();
        } catch (Throwable ex) {
            //    				System.err.println("VP>"+className+" Exception "+ex);
            //    				ex.printStackTrace();
        }
        
        return interfaces;
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param myInterface DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public ArrayList findImplementationOf(Class myInterface) {
        StringTokenizer tokens = new StringTokenizer(System.getProperty("java.class.path"),
                File.pathSeparator);
        File currFile;
        ArrayList classImpl = new ArrayList();
        
        while (tokens.hasMoreTokens()) {
            currFile = new File((String) tokens.nextToken());
            
            if (currFile.exists()) {
                if (currFile.isDirectory()) {
                    classImpl = findImplementationOfInDirectory(classImpl, myInterface, currFile);
                } else {
                    classImpl = findImplementationOfInJar(classImpl, myInterface, currFile);
                }
            }
        }
        
        log(("++> Final ClassImpl =" + classImpl).replaceAll(",", ",\n"));
        
        return classImpl;
    }
    
    private ArrayList findImplementationOfInDirectory(ArrayList classImpl, Class myInterface, File f) {
        Class c = null;
        File f2;
        String[] list = f.list();
        
        for (int i = 0; i < list.length; i++) {
            f2 = new File(f.getPath() + File.separator + list[i]);
            
            if (f2.isDirectory()) {
                classImpl = findImplementationOfInDirectory(classImpl, myInterface, f2);
                
                // now add this directory only if it contains children
            } else {
                if (list[i].endsWith(".class")) {
                    String className = list[i].substring(0, list[i].lastIndexOf('.')).replace('/',
                            '.');
                    
                    try {
                        c = Class.forName(className);
                    } catch (Throwable e) {
                        continue;
                    }
                    
                    //                    if (!c.isInterface() && isCastableWith(c, myInterface)) {
                    if (!c.isInterface() && myInterface.isAssignableFrom(c)) {
                        //log("++> found impl "+className+" for "+myInterface);
                        classImpl.add(c);
                    }
                }
            }
        }
        
        return classImpl;
    }
    
    private ArrayList findImplementationOfInJar(ArrayList classImpl, Class myInterface, File f) {
        Class cl = null;
        String className = null;
        
        // skip none ossj jars
        // TODO: fix this the jar search could be a property....
        if (!(f.getName().startsWith("oss") || f.getName().startsWith("common"))) {
            log("++> skip jar " + f.getName());
            return classImpl;
        }
        
        //log("++> look " + myInterface + " in jar " + f.getName());
        
        try {
            ZipInputStream zipFile = new ZipInputStream(new BufferedInputStream(
                    new FileInputStream(f)));
            ZipEntry file;
            
            //------
            while ((file = zipFile.getNextEntry()) != null) {
                if ((file.isDirectory())
                || (file.getName().charAt(file.getName().length() - 1) == '/')) {
                    // skip it - we'll parse the dir info from the filename
                } else {
                    if (file.getName().endsWith(".class")) {
                        className = file.getName().substring(0, file.getName().lastIndexOf('.'))
                        .replace('/', '.');
                        
                        try {
                            cl = Class.forName(className);
                        } catch (Throwable e) {
                            continue;
                        }
                        
                        //                      if (!cl.isInterface() && isCastableWith(cl, myInterface)) {
                        if (!cl.isInterface() && myInterface.isAssignableFrom(cl)) {
                            //log("++> found impl "+className+" for "+myInterface);
                            classImpl.add(cl);
                        }
                    }
                }
                
                zipFile.closeEntry();
            }
            
            zipFile.close();
        } catch (Throwable ex) {
            log(ex);
        }
        
        return classImpl;
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param c DOCUMENT ME!
     * @param s DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    
    /*
    public boolean isCastableWith(Class c, String s) {
        // ---------
        // loop on interfaces
        // ---------
        if (c.getName().equals(s)) {
            return true;
        }
     
        Class[] cl = c.getInterfaces();
     
        if ((cl != null) && (cl.length > 0)) {
            for (int ind = cl.length - 1; ind >= 0; ind--) {
                if (cl[ind].getName().equals(s)) {
                    return true;
                }
     
                //if (cl[ind].getSuperclass() != null) return isCastableWith(cl[ind].getSuperclass(),s) ;
                Class[] cli = cl[ind].getInterfaces();
     
                if ((cli != null) && (cli.length > 0)) {
                    for (int i = cli.length - 1; i >= 0; i--)
                        return isCastableWith(cli[i], s);
                }
            }
        }
     
        // ---------
        // loop on super class
        // ---------
        Class sup = c.getSuperclass();
     
        if (sup != null) {
            return isCastableWith(sup, s);
        }
     
        return false;
    }
     */
}
