package ossj.common.tck.tests;

import junit.framework.*;

import ossj.common.tck.*;

import java.util.*;

import javax.jms.*;

import javax.naming.*;

import javax.oss.*;

/**
 *
 * This test checks basic functionality of the JVTSession and the JVTActivationSessionHome
 * as it is needed by all other test suites
 * @author  Vincent Perrot, Sun Microsystems Inc.
 * @version 1.1
 */
public class TestBasics extends ossj.common.tck.JVTTestBase {
    /** Creates new testJVTActivationSession */
    public TestBasics(String name) throws Exception {
        super(name);
    }

    /**
     * Basic tests:
     * <UL>
     * <LI> all the test from this class.
     * </UL>
     * And tests from TestJVTSessionSimple
     * <UL>
     * <LI> test_GetManagedEntityTypes_ContainsACTIVATE: verify that the JVT support at least one managed Entity type using getManagedEntityTypes
     * <LI> test_GetQueryTypes_IsNotNull: verify that JVT support at least one query type using getQueryTypes
     * <LI> test_GetSupportedOperations_IsSubsetOfJVTSessionOption: just call getSupportedOptionalOperations
     * <LI> test_JVT_JNDI_NAME:verify the JNDI name construction
     * </UL>
     * @return DOCUMENT ME!
     *
     * @throws Exception DOCUMENT ME!
     */
    public static TestSuite suite() throws Exception {
        //  System.out.println("hi in TEST BASICS");
        TestSuite suite = new TestSuite(TestBasics.class);
        suite.addTest(new TestJVTSessionSimple("test_GetManagedEntityTypes_ContainsACTIVATE"));
        suite.addTest(new TestJVTSessionSimple("test_GetQueryTypes_IsNotNull"));
        suite.addTest(new TestJVTSessionSimple(
                "test_GetSupportedOperations_IsSubsetOfJVTSessionOption"));
        suite.addTest(new TestJVTSessionSimple("test_JVT_JNDI_NAME"));

        return suite;
    }

    protected void setUp() throws Exception {
        // System.out.println("hi in TEST BASICS setup");
        init();

        //VP ??        Thread.currentThread().setContextClassLoader(EventLoader.getClassLoader());
    }

    protected void tearDown() {
    }

    /**
     * verify if the InitialContext can be created. <br>
     * Validate the connection with the AS.
     *
     * @throws Exception DOCUMENT ME!
     */
    public void test_GetInitialContext() throws Exception {
        /*test*/
        InitialContext ictx = this.util.getInitialContext();
        assertTrue("Could not get JNDI tree root. Please check server configuration", null != ictx);
    }

    /**
     * tries to get an JVTActivationSession implementation
     */
    public void test_GetJVTSession() throws Exception {
        /*test*/
        try {
            log("==> test_GetJVTSession ");

            for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
                // get the managedEntityValue for this JVTSession:
                // - get an instance of the session
                log(" Current JVTSession =  " + jvtSessionList.get(ind));

                JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
                assertTrue(
                    "Could not get the JVTSession interface from the server. Please check server configuration for "
                    + jvtSessionList.get(ind), null != jas);
            }
        } catch (Exception ex) {
            log(ex);
            fail("Unexpected Exception :" + ex.getClass().getName() + ", message ="
                + ex.getMessage());
        }

        assertTrue("JVTSession not found", jvtSessionList.size() != 0);
    }

    /**
     * Test if the declared Topic is reachable and validate the JNDI name used for the factory and topic.
     *
     * @throws Exception DOCUMENT ME!
     */
    public void test_GetTopicSubscriber() throws Exception {
        TopicSubscriber t = this.util.getTopicSubscriber();
        String result = this.util.isValidJndiName(this.util.getTopicFactoryName());
        assertTrue(result, result == null);
        log("+++> topic factory is Valid");
        result = this.util.isValidJndiName(this.util.getTopicName());
        assertTrue(result, result == null);
        log("+++> topic is Valid");
        assertTrue("Cannot subscribe to Topic. Please check server configuration", null != t);
        t.close();
    }

    /**
     * Verify the key factory using the given supported managed entity types.
     * 
     * @throws Exception DOCUMENT ME!
     */
    public void test_GetLegalKeys() throws Exception {
        try {
            for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
                // get the managedEntityValue for this JVTSession:
                // - get an instance of the session
                log(" Current JVTSession =  " + jvtSessionList.get(ind));

                JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));

                Hashtable ht = this.util.getManagedEntityKeys(jas);
                String[] types = jas.getManagedEntityTypes();

                for (int i = 0; i < types.length; ++i) {
                    Object obj = ht.get(types[i]);
                    assertTrue(jvtSessionList.get(ind)
                        + ": Returned ManagedEntityKey is null for ManagedEntityType " + types[i],
                        null != obj);
                    assertTrue(jvtSessionList.get(ind)
                        + ": Returned ManagedEntityKey has the wrong key type for ManagedEntityType "
                        + types[i], obj instanceof ManagedEntityKey);
                }
            }
        } catch (Exception ex) {
            log(ex);
            fail("Unexpected Exception :" + ex.getClass().getName() + ", message ="
                + ex.getMessage());
        }
    }
}
