package ossj.common.tck.tests;


//import javax.jms.*;
//import javax.naming.*;
import junit.framework.*;

import ossj.common.tck.StringTable;
import ossj.common.tck.Util;

import java.beans.*;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;

import java.lang.reflect.*;

/*
 * TestBasics.java
 *
 *
 */
import java.util.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.oss.*;


/**
 *
 * @author  Vincent Perrot, Sun Microsystems Inc.
 *
 */
public class TestCbeBasics extends TestMev {
    /**
     * Test cases
     *
     */
    private static Hashtable cbeImpl = null;
    private static ArrayList cbeInter = null;
    private static boolean completeCbeImplDone = false;

    /** Creates new testJVTActivationSession */
    public TestCbeBasics(String name) throws Exception {
        super(name);
    }

    /**
     * Generate the test cases
     */
    public static TestSuite suite() throws Exception {
        TestSuite retVal = new TestSuite("Cbe tests");

        Class cls = TestCbeBasics.class;
        Method[] methods = cls.getDeclaredMethods();

        for (int i = 0; i < methods.length; ++i) {
            String name = methods[i].getName();

            if (name.startsWith(StringTable.TEST_PREFIX)) {
                if (name.startsWith(StringTable.POSITIVE_TEST_PREFIX)
                        || name.startsWith(StringTable.NEGATIVE_TEST_PREFIX)) {
                    retVal.addTest(new TestCbeBasics(name));
                } else {
                    retVal.addTest(new TestCbeBasics(name));
                }
            }
        }

        return retVal;
    }

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() {
    }

    /**
     * Check presence of CBE interfaces<br>
     * Expected Result:<br>
     *  a ArrayList completed with all CBE interfaces
     */
    public void test_Positive_getCbeInterfaces() throws Exception {
        //check first if cbe table completed
        log("==> Test: test_Positive_getCbeInterfaces");
        cbeInter = util.findInterfaceStartsWith("javax.oss.cbe");
        log("++> Found cbe interfaces " + cbeInter.toString().replaceAll(",", ",\n"));
        assertTrue("CBE interfaces not found", null != cbeInter);
    }

    /**
     * Check presence of CBE interfaces implementations <br>
     * Search only in directoryies and jar file name starting with "oss" <br>
     * Expected Result:<br>
     *  a hashtable completed with all CBE interfaces from test_Positive_getCbeInterfaces
     *  as key and ArrayList that contains all implementation Classes as value
     */
    public void test_Positive_getCbeImplementation() throws Exception {
        //check first if cbe table completed
        log("==> Test: test_Positive_getCbeImplementation");
        searchCbeImplementation();

        // note all the interface is not necessarily implemented, so don;t need to check
        // classImpl == null;
        assertTrue("CBE interfaces not found", null != cbeImpl);
    }

    /*****************************************************************************************
     **
     **   genericTest
     **
     ******************************************************************************************/
    protected void doGeneric(String testName) throws Exception {
        log("==> CbeBasics Test: " + testName);

        if (!completeCbeImplDone) {
            searchCbeImplementation();
        }

        ArrayList classImpl = null;
        String inter = null;
        Object mev = null;

        //check first if cbe table completed
        if (cbeImpl == null) {
            searchCbeImplementation();
        }

        // test BeanInfo stuff
        for (Enumeration e = cbeImpl.keys(); e.hasMoreElements();) {
            inter = (String) e.nextElement();
            classImpl = (ArrayList) cbeImpl.get(inter);

            for (int i = classImpl.size() - 1; i >= 0; i--) {
                //	        		if (util.isCastableWith((Class)classImpl.get(i),"javax.oss.ManagedEntityValue")) {
                if (((Class) classImpl.get(i)).getName().indexOf("$") > 0) {
                    log("--> skip [" + ((Class) classImpl.get(i)).getName() + "] $ stuff");

                    continue;
                }

                try {
                    mev = ((Class) classImpl.get(i)).newInstance();
                } catch (java.lang.InstantiationException ie) {
                    log("--> skip [" + ((Class) classImpl.get(i)).getName()
                        + "] InstanciationException:\n");
                    log(ie);

                    continue;
                } catch (java.lang.IllegalAccessException iae) {
                    log("--> skip [" + ((Class) classImpl.get(i)).getName()
                        + "] IllegalAccessException (private ctor):\n");
                    log(iae);

                    continue;
                }

                if (mev instanceof javax.oss.ManagedEntityValue) {
                    log("++> for mev [" + mev.getClass().getName() + "]");

                    this.getClass().getSuperclass()
                        .getDeclaredMethod(testName, new Class[] { AttributeAccess.class }).invoke(this,
                        new Object[] { mev });
                } else {
                    Method meth = null;

                    try {
                        meth = mev.getClass().getDeclaredMethod("clone", (Class[])null);
                    } catch (Exception ex) {
                    }

                    if ((meth != null)
					&& !mev.getClass().isInterface()
                            && testName.equals("test_Positive_testCloneEqualsImplementation")) {
                        log("++> for cloneable [" + mev.getClass().getName() + "]");
                        this.getClass().getSuperclass()
                            .getDeclaredMethod(testName, new Class[] { Object.class })
                            .invoke(this, new Object[] { mev });
                    } else {
                        log("++> [" + mev.getClass().getName() + "] is not a ManagedEntityValue");
                    }
                }
            }
        }
        log("+++> Close test "+testName);
    }

    /*****************************************************************************************
    *
    *   Test Value and Value type and key pattern<br>
    * <UL>
    * <LI>test if VALUE_TYPE field is declared
    * <LI>test it VALUE_TYPE field is public final static String
    * <LI>test if the declared type exist (call forName)
    * <LI>test if the class is of the declared type (decalreType.isAssignableFrom)
    * <LI>test if the VALUE_TYPE ends with "Value"
    * <LI>test if the VALUE_TYPE corresponding interface exists
    * </UL>
    */
    public void test_cbe_ClassNameAndTypeAndValueKeyPair()
        throws Exception {
        doGeneric("test_ClassNameAndTypeAndValueKeyPair");
    }

    /*****************************************************************************************
    **
    **   getAttributeNames
    **
    ******************************************************************************************/
    /**
     * tests that attributes<br>
     *  <UL>
     * <LI>names have length > 0
     *  <LI>names start with lowercase letter
     *  <LI>get/set methods exist
     * </UL>
    */
    public void test_cbe_GetAttributeNames_NamesStartWithLowerCase()
        throws Exception {
        doGeneric("test_GetAttributeNames_NamesStartWithLowerCase");
    }

    /**
    tests that all supported optional attibutes are also listed in all attributes and
    attributes names have length > 0 and start with lowercase letter
    */
    public void test_cbe_GetOptionalAttributeNames_InAllAttributes()
        throws Exception {
        doGeneric("test_GetOptionalAttributeNames_InAllAttributes");
    }

    /*****************************************************************************************
    **
    **   isPopulated
    **
    ******************************************************************************************/
    /**
     * tests isPopulated for all attributes in getAttributeNames()
    */
    public void test_Positive_cbe_IsPopulated_AttributeNameAllDomainValues()
        throws Exception {
        doGeneric("test_Positive_IsPopulated_AttributeNameAllDomainValues");
    }

    /**
     * tests failure of isPopulated(null)
    */
    public void test_Negative_cbe_IsPopulated_AttributeNameNullValue()
        throws Exception {
        doGeneric("test_Negative_IsPopulated_AttributeNameNullValue");
    }

    /**
     * tests failure of isPopulated(a) for String a not in getAttributeNames()
    */
    public void test_Negative_cbe_IsPopulated_AttributeNameIllegalValue()
        throws Exception {
        doGeneric("test_Negative_IsPopulated_AttributeNameIllegalValue");
    }

    /*****************************************************************************************
    **
    **   getSettableAttributeNames
    **
    ******************************************************************************************/
    /**
     * tests that getSettableAttributesNames is a subset of getAttributeNames
    */
    public void test_cbe_GetSettableAttributeNames_SubsetOfGetAttributeNames()
        throws Exception {
        doGeneric("test_GetSettableAttributeNames_SubsetOfGetAttributeNames");
    }

    /*****************************************************************************************
    **
    **   unpopulateAttribute
    **
    ******************************************************************************************/
    /**
     * tests succesful invokation of unpopulateAttribute(a) for all a in getAttributeNames()
    */
    public void test_Positive_cbe_UnpopulateAttribute_AttributeNameAllDomainValues()
        throws Exception {
        doGeneric("test_Positive_UnpopulateAttribute_AttributeNameAllDomainValues");
    }

    /**
     * tests failure of unpopulatedAttribute(null)
    */
    public void test_Negative_cbe_UnpopulateAttribute_AttributeNameNullValue()
        throws Exception {
        doGeneric("test_Negative_UnpopulateAttribute_AttributeNameNullValue");
    }

    /**
     * tests failure of unpopulatedAttribute(a) for a not in getAttributeNames
    */
    public void test_Negative_cbe_UnpopulateAttribute_AttributeNameIllegalValue()
        throws Exception {
        doGeneric("test_Negative_UnpopulateAttribute_AttributeNameIllegalValue");
    }

    /*****************************************************************************************
    **
    **   unpopulateAllAttributes
    **
    ******************************************************************************************/
    /**
     * tests that unpopulateAlAttributes implies !isPopulated(a) for all a in getAttributeNames()
    */
    public void test_cbe_UnpopulateAllAttributes_IsPopulatedFalseForAll()
        throws Exception {
        doGeneric("test_UnpopulateAllAttributes_IsPopulatedFalseForAll");
    }

    /**
     * tests that unpopulateAlAttributes implies getPopulatedAttributesNamesIsEmpty() is empty
    */
    public void test_cbe_UnpopulateAllAttributes_GetPopulatedAttributesNamesIsEmpty()
        throws Exception {
        doGeneric("test_UnpopulateAllAttributes_GetPopulatedAttributesNamesIsEmpty");
    }

    /**
     * tests that unpopulateAlAttributes implies failure of getAttributeValue(a) for all a in getAttributeNames
    */
    public void test_cbe_UnpopulateAllAttributes_GetAttributeValueFailsForAll()
        throws Exception {
        doGeneric("test_UnpopulateAllAttributes_GetAttributeValueFailsForAll");
    }

    /*****************************************************************************************
    **
    **   setAttributeValue
    **
    ******************************************************************************************/
    /**
     * tests failure of setAttributeValue(a) for null
    */
    public void test_Negative_cbe_SetAttributeValue_AttributeNameNullValue()
        throws Exception {
        doGeneric("test_Negative_SetAttributeValue_AttributeNameNullValue");
    }

    /**
     * tests failure of setAttributeValue(a) for a not in getAttributeNames()
    */
    public void test_Negative_cbe_SetAttributeValue_AttributeNameIllegalValue()
        throws Exception {
        doGeneric("test_Negative_SetAttributeValue_AttributeNameIllegalValue");
    }

    /*****************************************************************************************
    **
    **   getAttributeValue
    **
    ******************************************************************************************/
    /**
     * tests failure of getAttributeValue(null)
    */
    public void test_Negative_cbe_GetAttributeValue_AttributeNameNullValue()
        throws Exception {
        doGeneric("test_Negative_GetAttributeValue_AttributeNameNullValue");
    }

    /**
     * tests failure of getAttributeValue(a) for a not in getAttributeValues();
    */
    public void test_Negative_cbe_GetAttributeValue_AttributeNameIllegalValue()
        throws Exception {
        doGeneric("test_Negative_GetAttributeValue_AttributeNameIllegalValue");
    }

    /*****************************************************************************************
    **
    **   setManagedEntityKey
    **
    ******************************************************************************************/
    /**
     * tests a domain value
    */
    public void test_Positive_cbe_SetManagedEntityKey_DomainValue()
        throws Exception {
        doGeneric("test_Positive_SetManagedEntityKey_DomainValue");
    }

    /**
     * tests a illegal key
    */

    //VP key can be tested only on the server side 

    /*
        public void test_Positive_cbe_SetManagedEntityKey_IllegalKey()
            throws Exception {
            doGeneric("test_Positive_SetManagedEntityKey_IllegalKey");
        }
    */

    /**
     * tests a illegal key
    */

    //  VP key can be tested only on the server side 

    /*
    public void test_Positive_cbe_SetAttributeValueKEY_IllegalKey()
        throws Exception {
        doGeneric("test_Positive_SetAttributeValueKEY_IllegalKey");
    }
    */

    /**
     * tests that succesful setManagedEntityKey(val) implies KEY in getPopulatedAttributeNames()
    */
    public void test_cbe_SetManagedEntityKey_GetPopulatedAttributeNamesContainsKEY()
        throws Exception {
        doGeneric("test_SetManagedEntityKey_GetPopulatedAttributeNamesContainsKEY");
    }

    /**
     * tests that succesful setManagedEntityKey(val) implies isPopulated(KEY)
    */
    public void test_cbe_SetManagedEntityKey_IsPopulatedKEY()
        throws Exception {
        doGeneric("test_SetManagedEntityKey_IsPopulatedKEY");
    }

    /*****************************************************************************************
    **
    **   setManagedEntityKey
    **
    ******************************************************************************************/
    /**
     * checks weather the getattributes return key as attribute or not
     */
    public void test_cbe_GetAttributeNames_ContainsKEY()
        throws Exception {
        doGeneric("test_GetAttributeNames_ContainsKEY");
    }

    /**
     * Checks if the KEY attribute is not settable
     */
    public void test_cbe_GetSettableAttributeNames_NotContainsKEY()
        throws Exception {
        doGeneric("test_GetSettableAttributeNames_NotContainsKEY");
    }

    /**
     * checks for invalid managedentitykey
     */
    public void test_Positive_cbe_ManagedEntityKey_IllegalKey()
        throws Exception {
        doGeneric("test_Positive_ManagedEntityKey_IllegalKey");
    }

    /*****************************************************************************************
    **
    **   setAttributeValue
    **
    ******************************************************************************************/
    /**
     * tests setAttributeValue(min/max) for primitive attributes that are not enums
    */

    // problem with creation date 
    public void test_Positive_cbe_SetAttributeValue_PrimitiveMINMAXValues()
        throws Exception {
        doGeneric("test_Positive_SetAttributeValue_PrimitiveMINMAXValues");
    }

    /**
     * tests Illegal null primitive attribute value
     */
    public void test_Negative_cbe_SetAttributeValue_PrimitiveNullValue()
        throws Exception {
        doGeneric("test_Negative_SetAttributeValue_PrimitiveNullValue");
    }

    /**
     * this is a generic method to test the setattribute value function based on the type of the field
    */
    public void test_Positive_cbe_SetAttributeValue_DomainValue()
        throws Exception {
        doGeneric("test_Positive_SetAttributeValue_DomainValue");
    }

    /**
     * tests that setAttributeValue(a) => a in getPopulatedAttributes()
    */
    public void test_cbe_SetAttributeValue_GetPopulatedAttributesNamesContainsAttributeName()
        throws Exception {
        log("==> Run test_SetAttributeValue_GetPopulatedAttributesNamesContainsAttributeName");
        doGeneric("test_SetAttributeValue_GetPopulatedAttributesNamesContainsAttributeName");
    }

    /**
     * tests that setAttributeValue(a) => isPopulated(a)
    */
    public void test_cbe_SetAttributeValue_IsPopulated()
        throws Exception {
        doGeneric("test_SetAttributeValue_IsPopulated");
    }

    /**
     * tests failure of setAttributeValue(a) for an illegal object()
    */
    public void test_Negative_cbe_SetAttributeValue_IllegalObject()
        throws Exception {
        doGeneric("test_Negative_SetAttributeValue_IllegalObject");
    }

    /**
     * Test clone and equals methods CBE implementations
     * Expected Result:
     *  a = b.clone()
     *  a.equals(a) == true
     *  b.equals(b) == true
     *  a.equals(b) == true
     *  b.equals(a) == true
     *
     */
    public void test_Positive_cbe_testCloneEqualsImplementation()
        throws Exception {
        doGeneric("test_Positive_testCloneEqualsImplementation");
    }

    /**
     * private methods
     */
    private void searchCbeImplementation() throws Exception {
        if (cbeInter == null) {
            log("++> list of interface null, search cbe interfaces");
            cbeInter = util.findInterfaceStartsWith("javax.oss.cbe");
        }

        if (cbeImpl == null) {
            cbeImpl = new Hashtable();
        }

        String inter = null;

        for (int j = cbeInter.size() - 1; j >= 0; j--) {
            inter = (String) cbeInter.get(j);
            log("++> search impl of " + inter);
            cbeImpl.put(inter, util.findImplementationOf(Class.forName(inter)));
        }

        completeCbeImplDone = true;
    }
}
