package ossj.common.tck.tests;

import ossj.common.tck.StringTable;
import ossj.common.tck.Util;

//import javax.jms.*;
//import javax.naming.*;
import junit.framework.*;

import java.beans.*;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;

import java.lang.reflect.*;

/*
 * TestBasics.java
 *
 *
 */
import java.util.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.oss.*;


/**
 *
 * @author  Vincent Perrot
 *
 */
public class TestEventValue extends TestMev {
    /**
     * Test cases
     *
     */
    private static Hashtable eventImpl = null;
    private static boolean completeEventImplDone = false;

    /** Creates new testJVTActivationSession */
    public TestEventValue(String name) throws Exception {
        super(name);
    }

    /**
     * Generate the test cases
     */
    public static TestSuite suite() throws Exception {
        TestSuite retVal = new TestSuite("Event tests");

        Class cls = TestCbeBasics.class;
        Method[] methods = cls.getDeclaredMethods();

        for (int i = 0; i < methods.length; ++i) {
            String name = methods[i].getName();

            if (name.startsWith(StringTable.TEST_PREFIX)) {
                if (name.startsWith(StringTable.POSITIVE_TEST_PREFIX)
                        || name.startsWith(StringTable.NEGATIVE_TEST_PREFIX)) {
                    retVal.addTest(new TestCbeBasics(name));
                } else {
                    retVal.addTest(new TestCbeBasics(name));
                }
            }
        }

        return retVal;
    }

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() {
    }


    /**
     * Check presence of CBE interfaces implementations
     * Search only in directoryies and jar file name starting with "oss"
     * Expected Result:
     *  a hashtable completed with all CBE interfaces from test_Positive_getCbeInterfaces
     *  as key and ArrayList that contains all implementation Classes as value
     */
    public void test_Positive_getEventImplementation() throws Exception {
        //check first if cbe table completed
        log("==> Test: test_Positive_getEventImplementation");
        eventImpl.put("javax.oss.Event", util.findImplementationOf(Class.forName("javax.oss.Event")));
        completeEventImplDone = true;
        // note all the interface is not necessarily implemented, so don;t need to check
        // classImpl == null;
        assertTrue("Event Implementation not found", null != eventImpl);
    }

    /*****************************************************************************************
     **
     **   genericTest
     **
     ******************************************************************************************/
    protected void doGeneric(String testName) throws Exception {
        log("==> Event Test: " + testName);

        if (eventImpl == null || !completeEventImplDone) {
        	test_Positive_getEventImplementation();
        }

        ArrayList classImpl = null;
        String inter = null;
        Object mev = null;

        // test BeanInfo stuff
        for (Enumeration e = eventImpl.keys(); e.hasMoreElements();) {
            inter = (String) e.nextElement();
            classImpl = (ArrayList) eventImpl.get(inter);

            for (int i = classImpl.size() - 1; i >= 0; i--) {
                if (((Class) classImpl.get(i)).getName().indexOf("$") > 0) {
                    log("--> skip [" + ((Class) classImpl.get(i)).getName() + "] $ stuff");

                    continue;
                }

                try {
                    mev = ((Class) classImpl.get(i)).newInstance();
                } catch (java.lang.InstantiationException ie) {
                    log("--> skip [" + ((Class) classImpl.get(i)).getName()
                        + "] InstanciationException:\n");
                    log(ie);

                    continue;
                } catch (java.lang.IllegalAccessException iae) {
                    log("--> skip [" + ((Class) classImpl.get(i)).getName()
                        + "] IllegalAccessException (private ctor):\n");
                    log(iae);

                    continue;
                }

                if (mev instanceof javax.oss.Event) {
                    log("++> for Event [" + mev.getClass().getName() + "]");

                    this.getClass().getSuperclass()
                        .getDeclaredMethod(testName, new Class[] { AttributeAccess.class }).invoke(this,
                        new Object[] { mev });
                } else {
                    log("++> [" + mev.getClass().getName() + "] is not a ManagedEntityValue");
                }
            }
        }
    }

    /*****************************************************************************************
    **
    **   getAttributeNames
    **
    ******************************************************************************************/
    /**
     * tests that attributes
     *  - names have length > 0
     *  - names start with lowercase letter
     *  - get/set methods exist
    */
    public void test_event_GetAttributeNames_NamesStartWithLowerCase()
        throws Exception {
        doGeneric("test_GetAttributeNames_NamesStartWithLowerCase");
    }

    /**
    tests that all supported optional attibutes are also listed in all attributes and
    attributes names have length > 0 and start with lowercase letter
    */
    public void test_event_GetOptionalAttributeNames_InAllAttributes()
        throws Exception {
        doGeneric("test_GetOptionalAttributeNames_InAllAttributes");
    }

    /*****************************************************************************************
    **
    **   isPopulated
    **
    ******************************************************************************************/
    /**
     * tests isPopulated for all attributes in getAttributeNames()
    */
    public void test_Positive_event_IsPopulated_AttributeNameAllDomainValues()
        throws Exception {
        doGeneric("test_Positive_IsPopulated_AttributeNameAllDomainValues");
    }

    /**
     * tests failure of isPopulated(null)
    */
    public void test_Negative_event_IsPopulated_AttributeNameNullValue()
        throws Exception {
        doGeneric("test_Negative_IsPopulated_AttributeNameNullValue");
    }

    /**
     * tests failure of isPopulated(a) for String a not in getAttributeNames()
    */
    public void test_Negative_event_IsPopulated_AttributeNameIllegalValue()
        throws Exception {
        doGeneric("test_Negative_IsPopulated_AttributeNameIllegalValue");
    }

    /*****************************************************************************************
    **
    **   getSettableAttributeNames
    **
    ******************************************************************************************/
    /**
     * tests that getSettableAttributesNames is a subset of getAttributeNames
    */
    public void test_event_GetSettableAttributeNames_SubsetOfGetAttributeNames()
        throws Exception {
        doGeneric("test_GetSettableAttributeNames_SubsetOfGetAttributeNames");
    }

    /*****************************************************************************************
    **
    **   unpopulateAttribute
    **
    ******************************************************************************************/
    /**
     * tests succesful invokation of unpopulateAttribute(a) for all a in getAttributeNames()
    */
    public void test_Positive_event_UnpopulateAttribute_AttributeNameAllDomainValues()
        throws Exception {
        doGeneric("test_Positive_UnpopulateAttribute_AttributeNameAllDomainValues");
    }

    /**
     * tests failure of unpopulatedAttribute(null)
    */
    public void test_Negative_event_UnpopulateAttribute_AttributeNameNullValue()
        throws Exception {
        doGeneric("test_Negative_UnpopulateAttribute_AttributeNameNullValue");
    }

    /**
     * tests failure of unpopulatedAttribute(a) for a not in getAttributeNames
    */
    public void test_Negative_event_UnpopulateAttribute_AttributeNameIllegalValue()
        throws Exception {
        doGeneric("test_Negative_UnpopulateAttribute_AttributeNameIllegalValue");
    }

    /*****************************************************************************************
    **
    **   unpopulateAllAttributes
    **
    ******************************************************************************************/
    /**
     * tests that unpopulateAlAttributes implies !isPopulated(a) for all a in getAttributeNames()
    */
    public void test_event_UnpopulateAllAttributes_IsPopulatedFalseForAll()
        throws Exception {
        doGeneric("test_UnpopulateAllAttributes_IsPopulatedFalseForAll");
    }

    /**
     * tests that unpopulateAlAttributes implies getPopulatedAttributesNamesIsEmpty() is empty
    */
    public void test_event_UnpopulateAllAttributes_GetPopulatedAttributesNamesIsEmpty()
        throws Exception {
        doGeneric("test_UnpopulateAllAttributes_GetPopulatedAttributesNamesIsEmpty");
    }

    /**
     * tests that unpopulateAlAttributes implies failure of getAttributeValue(a) for all a in getAttributeNames
    */
    public void test_event_UnpopulateAllAttributes_GetAttributeValueFailsForAll()
        throws Exception {
        doGeneric("test_UnpopulateAllAttributes_GetAttributeValueFailsForAll");
    }

    /*****************************************************************************************
    **
    **   setAttributeValue
    **
    ******************************************************************************************/
    /**
     * tests failure of setAttributeValue(a) for null
    */
    public void test_Negative_event_SetAttributeValue_AttributeNameNullValue()
        throws Exception {
        doGeneric("test_Negative_SetAttributeValue_AttributeNameNullValue");
    }

    /**
     * tests failure of setAttributeValue(a) for a not in getAttributeNames()
    */
    public void test_Negative_event_SetAttributeValue_AttributeNameIllegalValue()
        throws Exception {
        doGeneric("test_Negative_SetAttributeValue_AttributeNameIllegalValue");
    }

    /*****************************************************************************************
    **
    **   getAttributeValue
    **
    ******************************************************************************************/
    /**
     * tests failure of getAttributeValue(null)
    */
    public void test_Negative_event_GetAttributeValue_AttributeNameNullValue()
        throws Exception {
        doGeneric("test_Negative_GetAttributeValue_AttributeNameNullValue");
    }

    /**
     * tests failure of getAttributeValue(a) for a not in getAttributeValues();
    */
    public void test_Negative_event_GetAttributeValue_AttributeNameIllegalValue()
        throws Exception {
        doGeneric("test_Negative_GetAttributeValue_AttributeNameIllegalValue");
    }

    
    /*****************************************************************************************
    **
    **   setAttributeValue
    **
    ******************************************************************************************/
    /**
     * tests setAttributeValue(min/max) for primitive attributes that are not enums
    */

    // problem with creation date 
    public void test_Positive_event_SetAttributeValue_PrimitiveMINMAXValues()
        throws Exception {
        doGeneric("test_Positive_SetAttributeValue_PrimitiveMINMAXValues");
    }

    /**
     * tests Illegal null primitive attribute value
     */
    public void test_Negative_event_SetAttributeValue_PrimitiveNullValue()
        throws Exception {
        doGeneric("test_Negative_SetAttributeValue_PrimitiveNullValue");
    }

    /**
     * this is a generic method to test the setattribute value function based on the type of the field
    */
    public void test_Positive_event_SetAttributeValue_DomainValue()
        throws Exception {
        doGeneric("test_Positive_SetAttributeValue_DomainValue");
    }

    /**
     * tests that setAttributeValue(a) => a in getPopulatedAttributes()
    */
    public void test_event_SetAttributeValue_GetPopulatedAttributesNamesContainsAttributeName()
        throws Exception {
        log("==> Run test_SetAttributeValue_GetPopulatedAttributesNamesContainsAttributeName");
        doGeneric("test_SetAttributeValue_GetPopulatedAttributesNamesContainsAttributeName");
    }

    /**
     * tests that setAttributeValue(a) => isPopulated(a)
    */
    public void test_event_SetAttributeValue_IsPopulated()
        throws Exception {
        doGeneric("test_SetAttributeValue_IsPopulated");
    }

    /**
     * tests failure of setAttributeValue(a) for an illegal object()
    */
    public void test_Negative_event_SetAttributeValue_IllegalObject()
        throws Exception {
        doGeneric("test_Negative_SetAttributeValue_IllegalObject");
    }

    /**
     * Test clone and equals methods CBE implementations
     * Expected Result:
     *  a = b.clone()
     *  a.equals(a) == true
     *  b.equals(b) == true
     *  a.equals(b) == true
     *  b.equals(a) == true
     *
     */
    public void test_Positive_event_testCloneEqualsImplementation()
        throws Exception {
        doGeneric("test_Positive_testCloneEqualsImplementation");
    }

 
}
