package ossj.common.tck.tests;

import ossj.common.tck.*;

import junit.extensions.*;

import junit.framework.*;

import java.util.*;

import javax.jms.*;

import javax.oss.*;


/**
 * Test class for all events sent by the implementation
 * As soon as an error occurs, testing is ended
 * @author  BanuPrasad Dhanakoti
 * @version
 */
public class TestEvents extends TestCase {
    private TopicSubscriber tSubscriber;

    /**maps ManagedEntityKeys to ClientIds*/
    private Hashtable clientIds;

    /** Creates new testEvents */
    public TestEvents(String name) {
        super(name);
        this.clientIds = new Hashtable();
    }

    protected void setUp() throws Exception {
        this.tSubscriber = new Util().getTopicSubscriber();

        //VP       Thread.currentThread().setContextClassLoader(EventLoader.getClassLoader());
    }

    protected void tearDown() throws Exception {
        this.tSubscriber.close();
    }

    /**
     * Infinit Loop<br>
     * For all the messages received via the topic:
     * <UL>
     * <LI>verify they are of the type javax.oss.Event
     * <LI>contains time and applicationDN properties
     * <li>is cloneable
     * </UL>
     * 
     * @throws Exception DOCUMENT ME!
     */
    public void testAllEvents() throws Exception {
        Message msg = this.tSubscriber.receive();

        if (msg instanceof ObjectMessage) {
            ObjectMessage omsg = (ObjectMessage) msg;
            Object obj = omsg.getObject();

            if (obj instanceof javax.oss.Event) {
                javax.oss.Event baseEvent = (javax.oss.Event) obj;

                assertTrue("javax.oss.Event.getEventTime() returned null for event "
                    + baseEvent.toString(), null != baseEvent.getEventTime());

                assertTrue(
                    "Event property OSS_APPLICATION_DN_PROP_NAME and javax.oss.Event.getApplicationDN() differ for event"
                    + baseEvent.toString(),
                    msg.getStringProperty(EventPropertyDescriptor.OSS_APPLICATION_DN_PROP_NAME)
                       .equals(baseEvent.getApplicationDN()));

                Event clone = (javax.oss.Event)baseEvent.clone();
                assertTrue("javax.oss.Event.clone() return null" + baseEvent.toString(),
                    null != clone);

                //TODO test equals also ?????
                
            } else {
                fail("Received ObjectMessage which is not of type javax.oss.Event");
            }
        } else {
            fail("Received message which is not of type ObjectMessage");
        }
    }

    public static class EndlessEventTest extends TestDecorator {
        public EndlessEventTest() {
            super(new TestSuite(TestEvents.class));
        }

        public static TestSuite suite() {
            TestSuite retVal = new TestSuite("Endless Event test");
            retVal.addTest(new TestEvents.EndlessEventTest());

            return retVal;
        }

        public int countTestCases() {
            return 0;
        }

        public void run(TestResult result) {
            while (true) {
                if (result.shouldStop()) {
                    break;
                }

                super.run(result);
            }
        }

        public String toString() {
            return super.toString() + "(endless)";
        }
    }
}
