package ossj.common.tck.tests;

import junit.framework.*;

import ossj.common.tck.*;

import java.lang.reflect.*;

import java.util.*;

import javax.oss.*;


/**
 * tests everything not related to ManagedEntity
 * @author  Vincent Perrot Sun Microsystems
 * @version 1.2
 */
public class TestJVTSessionSimple extends ossj.common.tck.JVTTestBase {
    //VP   JVTSession adaptor = null;
    
    /** Creates new TestJVTSessionSimple */
    public TestJVTSessionSimple(String name) throws Exception {
        super(name);
    }
    
    protected void setUp() throws Exception {
        //   Thread.currentThread().setContextClassLoader(EventLoader.getClassLoader());
        init();
        
        //VP this.adaptor = util.getJVTSession();
    }
    
    protected void tearDown() {
    }
    
    /**
     * Creates the suite of tests
     *
     * @return the object encapsulating the suite of test
     *
     * @throws Exception
     */
    public static TestSuite suite() throws Exception {
        TestSuite retVal = new TestSuite("Domain tests");
        Class cls = TestJVTSessionSimple.class;
        Method[] methods = cls.getMethods();
        
        for (int i = 0; i < methods.length; ++i) {
            String name = methods[i].getName();
            
            if (name.startsWith(StringTable.TEST_PREFIX)) {
                if (name.startsWith(StringTable.POSITIVE_TEST_PREFIX)
                || name.startsWith(StringTable.NEGATIVE_TEST_PREFIX)) {
                    retVal.addTest(new TestJVTSessionSimple(name));
                }
            }
        }
        
        return retVal;
    }
    
    /**
     * Test if the given JNDI name follows:
     * System/System1/ApplicationType/Common/Application/1-1;1-1;ReferenceImplementation/Comp/name
     * or
     * System/System1/ApplicationType/Common/Application/1-1;1-1;ReferenceImplementation/Resource/name
     *
     * @throws Exception
     *
     */
    public void test_JVT_JNDI_NAME() throws Exception {
        /*test*/
        log("==> test_JVT_JNDI_NAME");
        
        String myClass = null;
        String jndiName = null;
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            myClass = (String) jvtSessionList.get(ind);
            jndiName = util.getProperty(myClass + "_JNDINAME");
            log("++> Current JVTSession =  " + myClass);
            log("+++> Test JNDI name [" + jndiName + "]");
            
            String result = util.isValidJndiName(jndiName);
            assertTrue(result, result == null);
            log("+++> Valid");
        }
    }
    
    /**
     *
     * getManagedEntityTypes, instantiated from TestBasics <br>
     * verify that the JVT support at least one managed Entity type using getManagedEntityTypes
     * <P>
     * Success if the the return array is not null and not empty
     */
    public void test_GetManagedEntityTypes_ContainsACTIVATE()
    throws Exception {
        /*test*/
        log("==> test_GetManagedEntityTypes_ContainsACTIVATE ");
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] managedEntityTypes = jas.getManagedEntityTypes();
            assertTrue(jvtSessionList.get(ind) + ".getManagedEntityTypes returns null ", managedEntityTypes != null);
            assertTrue(jvtSessionList.get(ind) + ".getManagedEntityTypes returns empty types list ",
                    managedEntityTypes.length != 0);
        }
    }
    
    /**
     *
     * verify that JVT support at least one query type using getNamedQueryTypes.<br>
     * getNamedQueryTypes instantiated from TestBasics
     * <P>
     * Success if the the return array is not null and not empty
     */
    public void test_GetQueryTypes_IsNotNull() throws Exception {
        /*tests*/
        log("==> test_GetQueryTypes_IsNotNull ");
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] types = jas.getNamedQueryTypes();
            assertTrue(jvtSessionList.get(ind) + ".getNamedQueryTypes returns null", types != null);
            //Fix Issue #51
            // assume that test length >=0 is the same as not testing it...
            //assertTrue(jvtSessionList.get(ind) + ".getNamedQueryTypes returns empty types list", types.length >= 0);
        }
    }
    
    /**
     *
     * verify that JVT support at least one query type using getUpdateProcedureTypes.<br>
     * getUpdateProcedureTypes instantiated from TestBasics
     * <P>
     * Success if the the return array is not null
     */
    public void test_GetUpvTypes_IsNotNull() throws Exception {
        /*tests*/
        log("==> test_GetUpvTypes_IsNotNull ");
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] types = jas.getUpdateProcedureTypes();
            assertTrue(jvtSessionList.get(ind) + ".getUpdateProcedureTypes returns null", types != null);
            //assertTrue(jvtSessionList.get(ind) + ".getUpdateProcedureTypes returns empty types list", types.length >= 0);
        }
    }
    
    /**
     * GetSupportedOperations instantiated from TestBasics.<br>
     * just call getSupportedOptionalOperations
     * <P>
     * Success if no exceptiuon is thrown.
     */
    public void test_GetSupportedOperations_IsSubsetOfJVTSessionOption()
    throws Exception {
        //    	tests
        log("==> test_GetSupportedOperations_IsSubsetOfJVTSessionOption ");
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] operations = jas.getSupportedOptionalOperations();
            
            //        	String[] optionalOperations = this.util.getOptionalOperations();
            //        	assertTrue("getSupportedOperations returned null", null!=operations);
            //        	for( int i=0;i<operations.length;++i ){
            //            	assertTrue( "Supported operation "+operations[i]+" is not in JVTActivationSessionOption",
            //                        this.util.contains(optionalOperations, operations[i]) );
            //        	}
        }
    }
    
    /**
     * newManagedEntityValue Basically test for makeManagedEntityValue
     * Success if a valid value instance is returned:<br>
     * no null and with Key field populated
     */
    public void test_Positive_NewManagedEntityValue_AllDomainValues()
    throws Exception {
        log("==> test_Positive_NewManagedEntityValue_AllDomainValues ");
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] managedTypes = jas.getManagedEntityTypes();
            int i = 0;
            
            /*test*/
            for (i = 0; i < managedTypes.length; ++i) {
                try {
                    ManagedEntityValue val = jas.makeManagedEntityValue(managedTypes[i]);
                    assertTrue(StringTable.TYPE + managedTypes[i] + " value is null", null != val);
                    assertTrue(StringTable.TYPE + managedTypes[i] + " KEY is populated",
                            !val.isPopulated(ManagedEntityValue.KEY));
                } catch (javax.oss.OssIllegalArgumentException ex) {
                    fail(StringTable.TYPE + managedTypes[i]);
                } catch (Exception other) {
                    throw new TCKException(null, managedTypes[i], other);
                }
            }
        }
    }
    
    /**
     * test managed entity value factory using null type<br>
     * Sucess is makeManagedEntityValue(null) throws OssIllegalArgumentException
     *
     * @throws Exception DOCUMENT ME!
     */
    public void test_Negative_NewManagedEntityValue_NullValue()
    throws Exception {
        /*test*/
        log("==> test_Negative_NewManagedEntityValue_NullValue ");
        
        int ind = jvtSessionList.size() - 1;
        
        for (; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            
            try {
                ManagedEntityValue val = jas.makeManagedEntityValue(null);
            } catch (javax.oss.OssIllegalArgumentException ex) {
                continue;
            }
            
            fail(jvtSessionList.get(ind)
            + " :expected IllegalArgumentException when makeManagedEntityValue(null)");
        }
    }
    
    /**
     * Test managed entity value using unsupported type
     * <br>
     * Success if OssIllegalArgumentException is thrown when an invalid (not returned by getTypes) type is given
     *
     * @throws Exception DOCUMENT ME!
     */
    public void test_Negative_NewManagedEntityValue_IllegalValue()
    throws Exception {
        log("==> test_Negative_NewManagedEntityValue_IllegalValue ");
        
        int ind = jvtSessionList.size() - 1;
        
        for (; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] managedTypes = jas.getManagedEntityTypes();
            
            /*test*/
            try {
                ManagedEntityValue val = jas.makeManagedEntityValue(this.util.getIllegalValue(
                        managedTypes, 1)[0]);
            } catch (javax.oss.OssIllegalArgumentException ex) {
                continue;
            }
            
            fail(jvtSessionList.get(ind)
            + " :expected IllegalArgumentException when makeManagedEntityValue(illegal)");
        }
    }
    
    /**
     * new Upv Value Basically test for makeUpdateProcedureValue
     * Success if a valid value instance is returned:<br>
     * no null
     */
    public void test_Positive_NewUpvValue_AllDomainValues()
    throws Exception {
        log("==> test_Positive_NewUpvValue_AllDomainValues ");
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] managedTypes = jas.getUpdateProcedureTypes();
            int i = 0;
            
            /*test*/
            for (i = 0; i < managedTypes.length; ++i) {
                UpdateProcedureValue val = null;
                try {
                    val = jas.makeUpdateProcedureValue(managedTypes[i]);
                    assertTrue(StringTable.TYPE + managedTypes[i] + " value is null", null != val);
                } catch (javax.oss.OssIllegalArgumentException ex) {
                    fail(StringTable.TYPE + managedTypes[i]);
                } catch (Exception other) {
                    throw new TCKException(null, managedTypes[i], other);
                }
                try {
                    javax.oss.UpdateProcedureResponse resp = jas.update(val);
                    resp.isSuccessful();
                } catch (javax.oss.OssIllegalArgumentException ex) {
                    fail(StringTable.TYPE + managedTypes[i]);
                } catch (Exception other) {
                    throw new TCKException(null, managedTypes[i], other);
                }
            }
        }
    }
    
    /**
     * test Upv value factory using null type<br>
     * Sucess is makeUpdateProcedureValue(null) throws OssIllegalArgumentException
     *
     * @throws Exception DOCUMENT ME!
     */
    public void test_Negative_NewUpvValue_NullValue() throws Exception {
        /*test*/
        log("==> test_Negative_NewUpvValue_NullValue ");
        
        int ind = jvtSessionList.size() - 1;
        
        for (; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            
            try {
                UpdateProcedureValue val = jas.makeUpdateProcedureValue(null);
            } catch (javax.oss.OssIllegalArgumentException ex) {
                continue;
            }
            
            fail(jvtSessionList.get(ind)
            + " :expected IllegalArgumentException when makeUpdateProcedureValue(null)");
        }
    }
    
    /**
     * Test Upv value using unsupported type
     * <br>
     * Success if OssIllegalArgumentException is thrown when an invalid (not returned by getTypes) type is given
     *
     * @throws Exception DOCUMENT ME!
     */
    public void test_Negative_NewUpvValue_IllegalValue()
    throws Exception {
        log("==> test_Negative_NewUpvValue_IllegalValue ");
        
        int ind = jvtSessionList.size() - 1;
        
        for (; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] managedTypes = jas.getUpdateProcedureTypes();
            
            /*test*/
            try {
                UpdateProcedureValue val = jas.makeUpdateProcedureValue(this.util.getIllegalValue(
                        managedTypes, 1)[0]);
            } catch (javax.oss.OssIllegalArgumentException ex) {
                continue;
            }
            
            fail(jvtSessionList.get(ind)
            + " :expected IllegalArgumentException when makeUpdateProcedureValue(illegal)");
        }
    }
    
    /**
     *  Verify the query value factory to create new query values
     *  <BR>
     *  Success if makeNamedQueryValue return none null instance
     */
    public void test_Positive_NewQueryValue_AllDomainValues()
    throws Exception {
        log("==> test_Positive_NewQueryValue_AllDomainValues ");
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] types = jas.getNamedQueryTypes();
            assertTrue(jvtSessionList.get(ind) + ".getNamedQueryTypes returns null", types != null);
            util.shuffle(types);
            
            NamedQueryValue qv;
            
            /*tests*/
            for (int i = 0; i < types.length; ++i) {
                try {
                    qv = jas.makeNamedQueryValue(types[i]);
                    assertTrue(types[i] + " returns null", qv != null);
                } catch (javax.oss.OssIllegalArgumentException iaex) {
                    fail(types[i]);
                } catch (java.lang.Exception other) {
                    throw new TCKException(null, types[i], other);
                }
            }
        }
    }
    
    /**
     * try to create new query value using the factory and null query type
     * <BR> Success if OssIllegalArgument is thrown
     *
     * @throws Exception DOCUMENT ME!
     */
    public void test_Negative_NewQueryValue_NullValue()
    throws Exception {
        /*tests*/
        log("==> test_Negative_NewQueryValue_NullValue ");
        
        int ind = jvtSessionList.size() - 1;
        
        for (; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            
            try {
                NamedQueryValue qv = jas.makeNamedQueryValue(null);
            } catch (javax.oss.OssIllegalArgumentException ex) {
                continue;
            }
            
            fail(jvtSessionList.get(ind)
            + " :expected IllegalArgumentException when makeNamedQueryValue(null)");
        }
    }
    
    /**
     * try to create new query value using the factory and invalid query type
     * <br>
     * Success if OssIllegalArgument is thrown
     *
     * @throws Exception DOCUMENT ME!
     */
    public void test_Negative_NewQueryValue_IllegalValue()
    throws Exception {
        log("==> test_Negative_NewQueryValue_NullValue ");
        
        int ind = jvtSessionList.size() - 1;
        
        for (; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] types = jas.getNamedQueryTypes();
            assertTrue(jvtSessionList.get(ind) + ".getNamedQueryTypes returns null", types != null);
            String IllegalValue = util.getIllegalValue(types, 1)[0];
            
            /*tests*/
            try {
                NamedQueryValue qv = jas.makeNamedQueryValue(IllegalValue);
            } catch (javax.oss.OssIllegalArgumentException ex) {
                continue;
            }
            
            fail(jvtSessionList.get(ind)
            + " :expected IllegalArgumentException when makeNamedQueryValue(IllegalValue)");
        }
    }
    
    /**
     * query using null attribute name list
     * Note the exception for this method shall only concern the query value parameter
     * none of the Exception raised by the attribute names shall arrive.
     */
    
    /* deprecated method
    public void test_Positive_queryManagedEntities_NullAttributeNames()
        throws Exception {
        log("==> test_Positive_queryManagedEntities_NullAttributeNames ");
     
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
     
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] types = jas.getNamedQueryTypes();
            util.shuffle(types);
     
            NamedQueryValue qv = null;
     
            ManagedEntityValueIterator mevi = null;
     
            for (int i = 0; i < types.length; ++i) {
                qv = jas.makeNamedQueryValue(types[i]);
     
                // Test 1: query with null attributeNames[]
                try {
                    mevi = jas.queryManagedEntities(qv, null);
                    assertTrue("queryManagedEntities null as attributes for " + types[i]
                        + " returns null Iterator", mevi != null);
     
                    // try to get some values...
                    ManagedEntityValue[] mevs = mevi.getNext(10);
     
                    // Ok remove it now
                    mevi.remove();
                } catch (javax.oss.OssIllegalArgumentException iaex) {
                    log(iaex);
     
                    // This exception shall be thrown only if the query is not valid
                    fail(types[i]);
                } catch (java.lang.Exception other) {
                    throw new TCKException(null, types[i], other);
                }
            }
        }
    }
     */
    
    /**
     * Query managed entities using null query type
     * <br>
     * Success if querry(null) throws OssIllegalArgumentException
     *
     * @throws Exception DOCUMENT ME!
     * @throws TCKException DOCUMENT ME!
     */
    public void test_Negative_query_NullQuery()
    throws Exception {
        log("==> test_Negative_query_NullQuery ");
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            NamedQueryResponse mevi = null;
            
            try {
                mevi = jas.query(null);
                fail("Expected OssIllegalArgumentException for query(null)");
            } catch (javax.oss.OssIllegalArgumentException iaex) {
                //OK
            } catch (java.lang.Exception other) {
                throw new TCKException(null, null, other);
            }
        }
    }
    /**
     * Update procedure using null  type
     * <br>
     * Success if update(null) throws OssIllegalArgumentException
     *
     * @throws Exception DOCUMENT ME!
     * @throws TCKException DOCUMENT ME!
     */
    public void test_Negative_update_NullUpv()
    throws Exception {
        log("==> test_Negative_update_NullUpv ");
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            UpdateProcedureResponse upvr = null;
            
            try {
                upvr = jas.update(null);
                fail("Expected OssIllegalArgumentException for update(null)");
            } catch (javax.oss.OssIllegalArgumentException iaex) {
                //OK
            } catch (java.lang.Exception other) {
                throw new TCKException(null, null, other);
            }
        }
    }
    
    /**
     * Query managed entities using and empty attribute name list
     *
     * @throws Exception DOCUMENT ME!
     * @throws TCKException DOCUMENT ME!
     */
    
    /* deprecated
    public void test_Positive_query_EmptyAttributeNames()
        throws Exception {
        log("==> test_Positive_query_EmptyAttributeNames ");
     
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
     
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] types = jas.getNamedQueryTypes();
            util.shuffle(types);
     
            String[] names = null;
            NamedQueryValue qv = null;
     
            ManagedEntityValueIterator mevi = null;
     
            for (int i = 0; i < types.length; ++i) {
                qv = jas.makeNamedQueryValue(types[i]);
     
                names = new String[0];
     
                try {
                    //illegal array of attribute shall not throw exception...
                    mevi = jas.query(qv, names);
                    assertTrue("query using attributes = new String[1] for "
                        + types[i] + " returns null Iterator", mevi != null);
     
                    // try to get some values...
                    ManagedEntityValue[] mevs = mevi.getNext(10);
     
                    // Ok remove it now
                    mevi.remove();
                } catch (javax.oss.OssIllegalArgumentException iaex) {
                    log(iaex);
                    fail(types[i] + ": get IllegalArgument Exception ");
                } catch (java.lang.Exception other) {
                    log(other);
                    throw new TCKException(null, types[i], other);
                }
            }
        }
    }
     */
    
    /**
     * Query managed entities using an invalid attribute name list
     *
     * @throws Exception DOCUMENT ME!
     * @throws TCKException DOCUMENT ME!
     */
    
    /* deprecated
    public void test_Negative_query_IllegalAttributeName()
        throws Exception {
        log("==> test_Positive_query_EmptyAttributeNames ");
     
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
     
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] types = jas.getNamedQueryTypes();
            String[] mevTypes = jas.getManagedEntityTypes();
            util.shuffle(types);
     
            String[] names = null;
            NamedQueryValue qv = null;
     
            ManagedEntityValueIterator mevi = null;
     
            for (int i = 0; i < types.length; ++i) {
                for (int j = 0; j < mevTypes.length; ++j) {
                    qv = jas.makeNamedQueryValue(types[i]);
     
                    names = new String[1];
                    names[0] = util.randomString();
     
                    try {
                        //illegal array of attribute shall not throw exception...
                        mevi = jas.query(qv, names);
     
                        // the ilegal attribute name shall be ignore
                        assertTrue("query using attributes = one illegalValue for "
                            + types[i] + " returns null Iterator", mevi != null);
     
                        // try to get some values...
                        ManagedEntityValue[] mevs = mevi.getNext(10);
     
                        // Ok remove it now
                        mevi.remove();
                    } catch (javax.oss.OssIllegalArgumentException iaex) {
                        log(iaex);
                        fail(types[i] + ": get IllegalArgument Exception ");
                    } catch (java.lang.Exception other) {
                        log(other);
                        throw new TCKException(null, types[i], other);
                    }
                }
            }
        }
    }
     */
    
    /**
     * Query managed entities using a full attribute name list
     *
     * @throws Exception DOCUMENT ME!
     * @throws TCKException DOCUMENT ME!
     */
    
    /* deprecated
        public void test_Positive_query_AllAttributeNames()
            throws Exception {
            log("==> test_Positive_query_AllAttributeNames ");
     
            for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
                // get the managedEntityValue for this JVTSession:
                // - get an instance of the session
                log(" Current JVTSession =  " + jvtSessionList.get(ind));
     
                JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
                String[] types = jas.getNamedQueryTypes();
                String[] mevTypes = jas.getManagedEntityTypes();
                util.shuffle(types);
     
                String[] names = null;
                NamedQueryValue qv = null;
     
                ManagedEntityValueIterator mevi = null;
     
                for (int i = 0; i < types.length; ++i) {
                    for (int j = 0; j < mevTypes.length; ++j) {
                        qv = jas.makeNamedQueryValue(types[i]);
     
                        //                                log("VP>"+ type="+types[i]);
                        names = jas.makeManagedEntityValue(mevTypes[j]).getAttributeNames();
     
                        try {
                            //                                log("VP> type="+types[i]+" query="+qv.toString());
                            mevi = jas.query(qv, names);
     
                            //                                log("VP> mevi="+mevi.toString());
                            assertTrue("query for " + types[i]
                                + " returns null Iterator", mevi != null);
     
                            // try to get some values...
                            ManagedEntityValue[] mevs = mevi.getNext(10);
     
                            // Ok remove it now
                            mevi.remove();
                        } catch (javax.oss.OssIllegalArgumentException iaex) {
                            log(iaex);
                            fail(types[i] + ": get IllegalArgument Exception ");
                        } catch (java.lang.Exception other) {
                            log(other);
                            throw new TCKException(null, types[i], other);
                        }
                    }
                }
            }
        }
     */
    
    /**
     * Query managed entities using a null query but a valid attribute name list
     *
     * @throws Exception DOCUMENT ME!
     */
    
    /* deprecated
        public void test_Negative_query_NullValue()
            throws Exception {
            log("==> test_Negative_query_NullValue ");
     
            for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
                // get the managedEntityValue for this JVTSession:
                // - get an instance of the session
                log(" Current JVTSession =  " + jvtSessionList.get(ind));
     
                JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
                String[] types = jas.getNamedQueryTypes();
                String[] mevTypes = jas.getManagedEntityTypes();
                util.shuffle(types);
     
                for (int i = 0; i < types.length; ++i) {
                    for (int j = 0; j < mevTypes.length; ++j) {
                        try {
                            String[] names = jas.makeManagedEntityValue(mevTypes[j]).getAttributeNames();
                            ManagedEntityValueIterator mevi = jas.query(null, names);
                        } catch (javax.oss.OssIllegalArgumentException ex) {
                            continue;
                        }
     
                        fail(jvtSessionList.get(ind)
                            + " :expected IllegalArgumentException when query(null, names)");
                    }
                }
            }
        }
     */
    
    /**
     * Try to find and test methods with the format ManagedEntityKey methodname(ManagedEntityValue)
     * <br>
     * Success if at leat one method with the profile ManagedEntityKey <methodname>(ManagedEntityValue) found
     * (the method is called anyway (exception or valid retrn value are correct)
     * @throws Exception DOCUMENT ME!
     */
    public void test_Positive_callMethodKeyCreateWithValue()
    throws Exception {
        /*tests*/
        log("==> test_Positive_callMethodKeyCreateWithValue ");
        
        boolean getOne = false;
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            Method[] meths = jas.getClass().getDeclaredMethods();
            Method current = null;
            
            for (int j = meths.length - 1; j >= 0; j--) {
                current = meths[j];
                
                Class[] param = current.getParameterTypes();
                
                if ((param.length == 1) && ManagedEntityValue.class.isAssignableFrom(param[0])
                && ManagedEntityKey.class.isAssignableFrom(meths[j].getReturnType())) {
                    log("++> Found " + meths[j].getReturnType().getName() + " " + current.getName()
                    + "(" + param[0].getName() + ")");
                    
                    Object[] args = new Object[1];
                    String[] mevTypes = jas.getManagedEntityTypes();
                    
                    for (int k = 0; k < mevTypes.length; ++k) {
                        // try first with the default mev
                        args[0] = jas.makeManagedEntityValue(mevTypes[k]);
                        
                        try {
                            getOne = true; // what ever the result of the invocation....
                            
                            Object res = current.invoke(jas, args);
                            log("++> Call one OK " + meths[j].getReturnType().getName() + " "
                                    + current.getName() + "(" + param[0].getName() + ")");
                        } catch (java.lang.IllegalArgumentException iae) {
                            // did my best but .....
                            log("--> got exception when trying to call ManagedEntityKey "
                                    + current.getName() + "(" + param[0].getName() + ")");
                            log(iae);
                        } catch (Exception e) {
                            // did my best but .....
                            log("--> got exception when trying to call ManagedEntityKey "
                                    + current.getName() + "(" + param[0].getName() + ")");
                            log(e);
                        }
                        
                        // try now with a random value....
                        try {
                            args[0] = util.randomManagedEntityValueAtrributes(jas
                                    .makeManagedEntityValue(mevTypes[k]));
                        } catch (Exception ex) {
                            log("--> got exception when trying to generate the random mev "
                                    + param[0].getName());
                            log(ex);
                            ex.printStackTrace();
                            
                            continue;
                        }
                        
                        try {
                            getOne = true; // what ever the result of the invocation....
                            
                            Object res = current.invoke(jas, args);
                            log("++> Call one OK " + meths[j].getReturnType().getName() + " "
                                    + current.getName() + "(" + param[0].getName() + ")");
                        } catch (java.lang.IllegalArgumentException iae) {
                            // did my best but .....
                            log("--> got exception when trying to call ManagedEntityKey "
                                    + current.getName() + "(" + param[0].getName() + ")");
                            log(iae);
                        } catch (Exception e) {
                            // did my best but .....
                            log("--> got exception when trying to call ManagedEntityKey "
                                    + current.getName() + "(" + param[0].getName() + ")");
                            log(e);
                        }
                    }
                }
            }
        }
        
        assertTrue("Not found any valid ManagedEntityKey <methodname>(ManagedEntityValue)", getOne);
    }
    
    /**
     * Try to find and test methods with the format ManagedEntityKeyResults[] methodname(ManagedEntityValue[])
     * If call to the method is successful the isSuccess or getException method are called on all returned ManagedEntityKeyresults
     *<br>
     * Success if at leat one method with the profile MManagedEntityKeyResults[] methodname(ManagedEntityValue[]) found
     * (the method is called anyway (exception or valid retrn value are correct)
     * @throws Exception DOCUMENT ME!
     */
    public void test_Positive_callMethodKeysCreateWithValues()
    throws Exception {
        /*tests*/
        log("==> test_Positive_callMethodKeysCreateWithValues ");
        
        boolean getOne = false;
        boolean foundOne = false;
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            Method[] meths = jas.getClass().getDeclaredMethods();
            Method current = null;
            ArrayList classImpl = null;
            
            for (int j = meths.length - 1; j >= 0; j--) {
                current = meths[j];
                
                Class[] param = current.getParameterTypes();
                
                log("++> Look at " + current.getReturnType().getName() + " " + current.getName()
                + "(" + ((param.length == 1) ? param[0].getName() : "null") + ") ret ="
                        + ManagedEntityKeyResult[].class.isAssignableFrom(current.getReturnType())
                        + " parms = "
                        + ((param.length == 1)
                        ? (ManagedEntityValue[].class.isAssignableFrom(param[0]) ? "true" : "false")
                        : "null"));
                
                if ((param.length == 1) //&& util.isCastableWith(param[0].getComponentType(), "javax.oss.ManagedEntityValue")
                //&& util.isCastableWith(meths[j].getReturnType().getComponentType(), "javax.oss.ManagedEntityKeyResult")) {
                && ManagedEntityValue[].class.isAssignableFrom(param[0])
                && ManagedEntityKeyResult[].class.isAssignableFrom(current.getReturnType())) {
                    log("++> Found " + current.getReturnType().getName() + " " + current.getName()
                    + "(" + param[0].getName() + ")");
                    
                    ManagedEntityValue[] values = new ManagedEntityValue[1];
                    
                    String[] mevTypes = jas.getManagedEntityTypes();
                    
                    for (int k = mevTypes.length - 1; k >= 0; k--) {
                        try {
                            values[0] = (ManagedEntityValue) util
                                    .randomManagedEntityValueAtrributes(jas.makeManagedEntityValue(
                                    mevTypes[k]));
                        } catch (Exception e) {
                            log("--> got exception when trying to generate the array of values "
                                    + current.getReturnType().getName() + " " + current.getName() + "("
                                    + param[0].getName() + ")");
                            log(e);
                        }
                        
                        try {
                            //System.err.println("VP> args[0][0] type is "+args[0];
                            Object res = current.invoke(jas, new Object[] { values });
                            log("++> Call one OK " + meths[j].getReturnType().getName() + " "
                                    + current.getName() + "(" + param[0].getName() + ")");
                            foundOne = true;
                            
                            // Now validate the ManagedEntityKeyResult[]
                            if (res.getClass().isArray()) {
                                ManagedEntityKeyResult[] mekr = (ManagedEntityKeyResult[]) res;
                                
                                for (int l = mekr.length - 1; l >= 0; l--) {
                                    if (mekr[l].isSuccess() || (mekr[l].getException() != null)) {
                                        getOne = true;
                                    }
                                }
                            }
                        } catch (Exception e) {
                            // did my best but .....
                            log("--> got exception when trying to call "
                                    + meths[j].getReturnType().getName() + " " + current.getName()
                                    + "(" + param[0].getName() + ")");
                            log(e);
                        }
                    }
                }
            }
        }
        
        assertTrue("Not found any valid ManagedEntityKeyResult[] <methodname>(ManagedEntityValue[])",
                getOne || !foundOne);
    }
    
    /**
     * Try to find and test methods with the format ManagedEntityValue[] methodname(ManagedEntityKeys[])
     * If call to the method is successful the isSuccess or getException method are called on all returned ManagedEntityKeyresults
     *<br>
     * Success if at leat one method with the profile ManagedEntityValue[] methodname(ManagedEntityKey[]) found
     * (the method is called anyway (exception or valid retrn value are correct)
     * @throws Exception DOCUMENT ME!
     */
    public void test_Positive_callMethodValuesMethWithKeys()
    throws Exception {
        /*tests*/
        log("==> test_Positive_callMethodValuesMethWithKeys ");
        
        boolean getOne = false;
        boolean foundOne = false;
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            Method[] meths = jas.getClass().getDeclaredMethods();
            Method current = null;
            ArrayList classImpl = null;
            
            for (int j = meths.length - 1; j >= 0; j--) {
                current = meths[j];
                
                Class[] param = current.getParameterTypes();
                
                log("++> Look at " + current.getReturnType().getName() + " " + current.getName()
                + "(" + ((param.length == 1) ? param[0].getName() : "null") + ") ret ="
                        + ManagedEntityValue[].class.isAssignableFrom(current.getReturnType())
                        + " parms = "
                        + ((param.length == 1)
                        ? (ManagedEntityKey[].class.isAssignableFrom(param[0]) ? "true" : "false")
                        : "null"));
                
                if ((param.length == 1) //&& util.isCastableWith(param[0].getComponentType(), "javax.oss.ManagedEntityValue")
                //&& util.isCastableWith(meths[j].getReturnType().getComponentType(), "javax.oss.ManagedEntityKeyResult")) {
                && ManagedEntityKey[].class.isAssignableFrom(param[0])
                && ManagedEntityValue[].class.isAssignableFrom(current.getReturnType())) {
                    log("++> Found " + current.getReturnType().getName() + " " + current.getName()
                    + "(" + param[0].getName() + ")");
                    
                    ManagedEntityKey[] keys = new ManagedEntityKey[1];
                    
                    String[] mevTypes = jas.getManagedEntityTypes();
                    
                    for (int k = mevTypes.length - 1; k >= 0; k--) {
                        try {
                            keys[0] = (jas.makeManagedEntityValue(mevTypes[k])).makeManagedEntityKey();
                        } catch (Exception e) {
                            log("--> got exception when trying to generate the array of keys "
                                    + current.getReturnType().getName() + " " + current.getName() + "("
                                    + param[0].getName() + ")");
                            log(e);
                        }
                        
                        try {
                            //System.err.println("VP> args[0][0] type is "+args[0];
                            Object res = current.invoke(jas, new Object[] { keys });
                            log("++> Call one OK " + meths[j].getReturnType().getName() + " "
                                    + current.getName() + "(" + param[0].getName() + ")");
                            foundOne = true;
                            
                            // Now validate the ManagedEntityKeyResult[]
                            if (res.getClass().isArray()) {
                                ManagedEntityValue[] mekr = (ManagedEntityValue[]) res;
                                
                                getOne = true;
                            }
                        } catch (Exception e) {
                            // did my best but .....
                            log("--> got exception when trying to call "
                                    + meths[j].getReturnType().getName() + " " + current.getName()
                                    + "( using " + keys.getClass().getName() + ")");
                            log(e);
                        }
                    }
                }
            }
        }
        
        assertTrue("Not found any valid ManagedEntityValue[] <methodname>(ManagedEntityKey[])",
                getOne || !foundOne);
    }
    
    /**
     * Try to find and test methods with the format void methodname(ManagedEntityValue)
     * If call to the method is successful the isSuccess or getException method are called on all returned ManagedEntityKeyresults
     *<br>
     * Success if at leat one method with the profile void methodname(ManagedEntityValue) found
     * (the method is called anyway (exception or valid retrn value are correct)
     * @throws Exception DOCUMENT ME!
     */
    public void test_Positive_callMethodVoidWithValue()
    throws Exception {
        /*tests*/
        log("==> test_Positive_callMethodVoidWithValue ");
        
        boolean getOne = false;
        boolean foundOne = false;
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            Method[] meths = jas.getClass().getDeclaredMethods();
            Method current = null;
            ArrayList classImpl = null;
            
            for (int j = meths.length - 1; j >= 0; j--) {
                current = meths[j];
                
                Class[] param = current.getParameterTypes();
                
                log("++> Look at " + current.getReturnType().getName() + " " + current.getName()
                + "(" + ((param.length == 1) ? param[0].getName() : "null") + ") ret ="
                        + current.getReturnType() + " parms = "
                        + ((param.length == 1)
                        ? (ManagedEntityValue.class.isAssignableFrom(param[0]) ? "true" : "false")
                        : "null"));
                
                if ((param.length == 1) //&& util.isCastableWith(param[0].getComponentType(), "javax.oss.ManagedEntityValue")
                //&& util.isCastableWith(meths[j].getReturnType().getComponentType(), "javax.oss.ManagedEntityKeyResult")) {
                && ManagedEntityValue.class.isAssignableFrom(param[0])
                && (current.getReturnType() == Void.TYPE)) {
                    log("++> Found void " + current.getName() + "(" + param[0].getName() + ")");
                    
                    ManagedEntityValue value = null;
                    
                    String[] mevTypes = jas.getManagedEntityTypes();
                    
                    for (int k = mevTypes.length - 1; k >= 0; k--) {
                        try {
                            value = (ManagedEntityValue) util.randomManagedEntityValueAtrributes(jas
                                    .makeManagedEntityValue(mevTypes[k]));
                        } catch (Exception e) {
                            log("--> got exception when trying to generate value "
                                    + current.getReturnType().getName() + " " + current.getName() + "("
                                    + mevTypes[k] + ")");
                            log(e);
                        }
                        
                        try {
                            //System.err.println("VP> args[0][0] type is "+args[0];
                            Object res = current.invoke(jas, new Object[] { value });
                            log("++> Call one OK " + current.getName() + "(" + mevTypes[k] + ")");
                            foundOne = true;
                            getOne = true;
                        } catch (Exception e) {
                            // did my best but .....
                            log("--> got exception when trying to call "
                                    + meths[j].getReturnType().getName() + " " + current.getName()
                                    + "(" + mevTypes[k] + ")");
                            log(e);
                        }
                    }
                }
            }
        }
        
        assertTrue("Not found any valid void <methodname>(ManagedEntityValue)", getOne || !foundOne);
    }
    
    /**
     * Try to find and test methods with the format ManagedEntityValue methodname(ManagedEntitykey)
     * If call to the method is successful the isSuccess or getException method are called on all returned ManagedEntityKeyresults
     *<br>
     * Success if at leat one method with the profile ManagedEntityValue methodname(ManagedEntitykey) found
     * (the method is called anyway (exception or valid retrn value are correct)
     * @throws Exception DOCUMENT ME!
     */
    public void test_Positive_callMethodValueWithKey()
    throws Exception {
        /*tests*/
        log("==> test_Positive_callMethodValueWithKey ");
        
        boolean getOne = false;
        boolean foundOne = false;
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            Method[] meths = jas.getClass().getDeclaredMethods();
            Method current = null;
            ArrayList classImpl = null;
            
            for (int j = meths.length - 1; j >= 0; j--) {
                current = meths[j];
                
                Class[] param = current.getParameterTypes();
                
                log("++> Look at " + current.getReturnType().getName() + " " + current.getName()
                + "(" + ((param.length == 1) ? param[0].getName() : "null") + ") ret ="
                        + ManagedEntityValue.class.isAssignableFrom(current.getReturnType())
                        + " parms = "
                        + ((param.length == 1)
                        ? (ManagedEntityKey.class.isAssignableFrom(param[0]) ? "true" : "false") : "null"));
                
                if ((param.length == 1) //&& util.isCastableWith(param[0].getComponentType(), "javax.oss.ManagedEntityValue")
                //&& util.isCastableWith(meths[j].getReturnType().getComponentType(), "javax.oss.ManagedEntityKeyResult")) {
                && ManagedEntityKey.class.isAssignableFrom(param[0])
                && ManagedEntityValue.class.isAssignableFrom(current.getReturnType())) {
                    log("++> Found " + current.getReturnType().getName() + " " + current.getName()
                    + "(" + param[0].getName() + ")");
                    
                    String[] mevTypes = jas.getManagedEntityTypes();
                    ManagedEntityKey key = null;
                    
                    for (int k = mevTypes.length - 1; k >= 0; k--) {
                        try {
                            key = jas.makeManagedEntityValue(mevTypes[k]).makeManagedEntityKey();
                        } catch (Exception e) {
                            log("--> got exception when trying to generate key "
                                    + current.getReturnType().getName() + " " + current.getName()
                                    + "( of type" + mevTypes[k] + ")");
                            log(e);
                        }
                        
                        try {
                            //System.err.println("VP> args[0][0] type is "+args[0];
                            Object res = current.invoke(jas, new Object[] { key });
                            log("++> Call one OK " + meths[j].getReturnType().getName() + " "
                                    + current.getName() + "(" + key.getClass().getName() + ")");
                            foundOne = true;
                            
                            // Now validate the ManagedEntityKeyResult[]
                            if ((res == null)
                            || ManagedEntityValue.class.isAssignableFrom(res.getClass())) {
                                getOne = true;
                            }
                        } catch (Exception e) {
                            // did my best but .....
                            log("--> got exception when trying to call "
                                    + meths[j].getReturnType().getName() + " " + current.getName()
                                    + "(" + param[0].getName() + ")");
                            log(e);
                        }
                    }
                }
            }
        }
        
        assertTrue("Not found any valid ManagedEntityValue <methodname>(ManagedEntitykey)",
                getOne || !foundOne);
    }
    
    /**
     * Try to find and test methods with the format void methodname(ManagedEntitykey)
     * If call to the method is successful the isSuccess or getException method are called on all returned ManagedEntityKeyresults
     *<br>
     * Success if at leat one method with the profile void methodname(ManagedEntitykey) found
     * (the method is called anyway (exception or valid retrn value are correct)
     * @throws Exception DOCUMENT ME!
     */
    public void test_Positive_callMethodVoidWithKey() throws Exception {
        /*tests*/
        log("==> test_Positive_callMethodVoidWithKey ");
        
        boolean getOne = false;
        boolean foundOne = false;
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            Method[] meths = jas.getClass().getDeclaredMethods();
            Method current = null;
            ArrayList classImpl = null;
            
            for (int j = meths.length - 1; j >= 0; j--) {
                current = meths[j];
                
                Class[] param = current.getParameterTypes();
                
                log("++> Look at " + current.getReturnType().getName() + " " + current.getName()
                + "(" + ((param.length == 1) ? param[0].getName() : "null") + ") ret ="
                        + void.class.isAssignableFrom(current.getReturnType()) + " parms = "
                        + ((param.length == 1)
                        ? (ManagedEntityKey.class.isAssignableFrom(param[0]) ? "true" : "false") : "null"));
                
                if ((param.length == 1) //&& util.isCastableWith(param[0].getComponentType(), "javax.oss.ManagedEntityValue")
                //&& util.isCastableWith(meths[j].getReturnType().getComponentType(), "javax.oss.ManagedEntityKeyResult")) {
                && ManagedEntityKey.class.isAssignableFrom(param[0])
                && void.class.isAssignableFrom(current.getReturnType())) {
                    log("++> Found " + current.getReturnType().getName() + " " + current.getName()
                    + "(" + param[0].getName() + ")");
                    
                    String[] mevTypes = jas.getManagedEntityTypes();
                    ManagedEntityKey key = null;
                    
                    for (int k = mevTypes.length - 1; k >= 0; k--) {
                        try {
                            key = jas.makeManagedEntityValue(mevTypes[k]).makeManagedEntityKey();
                        } catch (Exception e) {
                            log("--> got exception when trying to generate key "
                                    + current.getReturnType().getName() + " " + current.getName()
                                    + "( of type" + mevTypes[k] + ")");
                            log(e);
                        }
                        
                        try {
                            //System.err.println("VP> args[0][0] type is "+args[0];
                            Object res = current.invoke(jas, new Object[] { key });
                            log("++> Call one OK " + meths[j].getReturnType().getName() + " "
                                    + current.getName() + "(" + key.getClass().getName() + ")");
                            foundOne = true;
                            getOne = true;
                        } catch (Exception e) {
                            // did my best but .....
                            log("--> got exception when trying to call "
                                    + meths[j].getReturnType().getName() + " " + current.getName()
                                    + "(" + param[0].getName() + ")");
                            log(e);
                        }
                    }
                }
            }
        }
        
        assertTrue("Not found any valid void <methodname>(ManagedEntitykey)", getOne || !foundOne);
    }
    
    /**
     * Verify that the JVT support at least one event type.<br>
     * For each Event type, verify also the corresponding EventPropertyDescriptor using the factory.
     * <br>
     * Success <br>
     * if getEventTypes return no null and at least 1 event type
     * and if (getEventDescriptor) for each type return a valid event propertyDescriptor (correct event type value)
     * and if makeEvent(type) do not throw any exception and returnan instance of the desired event type
     *
     * @throws Exception DOCUMENT ME!
     */
    public void test_Positive_getEventTypes() throws Exception {
        log("==> test_Positive_getEventTypes ");
        
        Class evtType = null;
        
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log(" Current JVTSession =  " + jvtSessionList.get(ind));
            
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] types = jas.getEventTypes();
            assertTrue(jvtSessionList.get(ind) + ".getEventTypes() returns null", types != null);
            assertTrue(jvtSessionList.get(ind) + ".getEventTypes() returns 0 element",
                    types.length != 0);
            
            for (int i = types.length - 1; i >= 0; i--) {
                EventPropertyDescriptor desc = (EventPropertyDescriptor) jas.getEventDescriptor(types[i]);
                assertTrue("EventPropertyDescritor is null for " + types[i], desc != null);
                assertTrue("EventType in descriptor is different form initial type for " + types[i],
                        desc.getEventType().equals(types[i]));
                
                Event event = desc.makeEvent();
                assertTrue("makeEvent for " + types[i] + " returns null", event != null);
                
                try {
                    evtType = Class.forName(types[i]);
                } catch (Exception ex) {
                    log(ex);
                    fail("get Exception:" + ex.getMessage()
                    + " when trying to create an event of the type " + types[i]);
                }
                
                assertTrue("makeEvent for " + types[i]
                        + " returns invalid event instance of event(the returned event is not aassignable to the type)",
                        evtType.isAssignableFrom(event.getClass()));
            }
        }
    }
}
