package ossj.common.tck.tests;

import junit.framework.*;

import ossj.common.tck.*;

import java.util.*;

import javax.jms.*;

import javax.naming.*;

import javax.oss.*;


/**
 * This test checks basic functionality of the TestManagedEntityKey
 *
 * @author Vincent Perrot, Sun Microsystems Inc.
 * @version 1.1
 */
public class TestManagedEntityKey extends ossj.common.tck.JVTTestBase {
    ManagedEntityKey[] keys = null;

    /** Creates new testJVTActivationSession */
    public TestManagedEntityKey(String name) throws Exception {
        super(name);
    }

    /**
     * build the test Suite.
     *
     * @return DOCUMENT ME!
     */
    public static TestSuite suite() {
        TestSuite suite = new TestSuite(TestManagedEntityKey.class);

        return suite;
    }

    protected void setUp() throws Exception {
        //VP    Thread.currentThread().setContextClassLoader(EventLoader.getClassLoader());
        init();

        ArrayList listOfKeys = new ArrayList();

        // Add the JVT keys
        for (int ind = jvtSessionList.size() - 1; ind >= 0; ind--) {
            // get the managedEntityValue for this JVTSession:
            // - get an instance of the session
            log("++> Current JVTSession =  " + jvtSessionList.get(ind));

            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            Hashtable ht = this.util.getManagedEntityKeys(jas);
            String[] types = jas.getManagedEntityTypes();

            for (int i = 0; i < types.length; i++) {
                log("++> for key  " + ht.get(types[i]));
                listOfKeys.add(ht.get(types[i]));
            }
        }

        // Add the CBE keys
        ArrayList cbeInter = util.findInterfaceStartsWith("javax.oss.cbe");

        ArrayList classImpl = null;
        String inter = null;
        Object key = null;

        // test BeanInfo stuff
        for (int j = cbeInter.size() - 1; j >= 0; j--) {
            inter = (String) cbeInter.get(j);
            classImpl = util.findImplementationOf(Class.forName(inter));

            for (int i = classImpl.size() - 1; i >= 0; i--) {
                //                if (util.isCastableWith((Class) classImpl.get(i), "javax.oss.ManagedEntityKey")) {
                if (ManagedEntityKey.class.isAssignableFrom((Class) classImpl.get(i))) {
                    log("++> for key [" + ((Class) classImpl.get(i)).getName() + "] for interface "
                        + inter);

                    try {
                        key = ((Class) classImpl.get(i)).newInstance();
                    } catch (Exception ex) {
                        log("--> unable to instanciate [" + ((Class) classImpl.get(i)).getName()
                            + "]");
                    }

                    if (key != null) {
                        listOfKeys.add(key);
                    }
                }
            }
        }

        keys = (ManagedEntityKey[]) listOfKeys.toArray(new ManagedEntityKey[listOfKeys.size()]);

        /*
                Hashtable ht = this.util.getManagedEntityKeys();
                JVTSession om = this.util.getJVTSession();
                String[] types = om.getManagedEntityTypes();
                keys = new ManagedEntityKey [types.length];
                for( int i=0;i<types.length;i++ )
                {
                    Object obj = ht.get(types[i]);
                    keys[i] = (ManagedEntityKey)obj ;

                }
        */
    }

    protected void tearDown() {
    }

    /**
     * Verify the ApplicationContext attribute of the key implementation.
     * It shall be a ApplicationContext class or derived from it.
     * <br>
     * Success if getApplicaitonContext return a no null and the the object of ApplicationContext Type
     * @throws Exception
     */
    public void test_GetApplicationContext() throws Exception {
        log("==> Test: test_GetApplicationContext");

        for (int i = 0; i < keys.length; i++) {
            ApplicationContext val = keys[i].getApplicationContext();
            assertTrue("Application Context is null for the ManagedEntityKey"
                + keys[i].getClass().getName(), val != null);
            assertTrue(
                "Application Context object is not of type ApplicationContext for ManagedEntityKey "
                + keys[i].getClass().getName(), val instanceof ApplicationContext);
        }
    }

    /**
     * Verify the ApplicationContext attribute of the key implementation.
     * <br>
     * Success if setApplicationContext(makeApplicationContext) doesn't throw any Exception
     * ad getApplicaitonConetxt == value used to set ApplicationContext
     * @throws Exception
     */
    public void test_SetMakeApplicationContext() throws Exception {
        log("==> Test: test_SetMakeApplicationContext");

        for (int i = 0; i < keys.length; i++) {
            ApplicationContext val = keys[i].makeApplicationContext();
            assertTrue("makeApplicationContext returns null for the ManagedEntityKey"
                + keys[i].getClass().getName(), val != null);

            try {
                keys[i].setApplicationContext(val);
            } catch (Exception ex) {
                throw new TCKException("setApplicationContext(makeApplicationContext)",
                    keys[i].getClass().getName(), ex);
            }
            assertTrue("getApplicaitionConetxt returns a value different from the one used to setApplicationContext"
                    + keys[i].getClass().getName(), keys[i].getApplicationContext().equals(val));
        }
    }

    /**
     * Verify the ApplicationDN attribute of the key implementation. It shall not be null.
     *
     * @throws Exception
     */
    public void test_GetApplicationDN() throws Exception {
        log("==> Test: test_GetApplicationDN");

        for (int i = 0; i < keys.length; i++) {
            String val = keys[i].getApplicationDN();
            assertTrue("Application DN is null for the ManagedEntityKey"
                + keys[i].getClass().getName(), val != null);
        }
    }

    /**
     * Verify the primaryKey attribute of the key implementation. It shall not be null.
     * @throws Exception
     */
    public void test_GetPrimaryKey() throws Exception {
        log("==> Test: test_GetPrimaryKey");

        for (int i = 0; i < keys.length; i++) {
            Object val = keys[i].getPrimaryKey();
            assertTrue("Prinary Key is null for the ManagedEntityKey"
                + keys[i].getClass().getName(), val != null);
        }
    }

    /**
     * Verify the type attribute of the key implementation. It shall not be null.
     * @throws Exception
     */
    public void test_GetType() throws Exception {
        log("==> Test: test_GetType");

        for (int i = 0; i < keys.length; i++) {
            String val = keys[i].getType();
            assertTrue("ManagedEntity Value Type  is null for the ManagedEntityKey"
                + keys[i].getClass().getName(), val != null);
        }
    }

    /**
     * Verify the key factory of the key implementation. The generated key shall not be null.
     * And the call to set(make) shall not thorw any exception
     * @throws Exception
     */
    public void test_MakePrimaryKey() throws Exception {
        log("==> Test: test_MakePrimaryKey");

        for (int i = 0; i < keys.length; i++) {
            Object val = keys[i].makePrimaryKey();
            assertTrue("Primary Key made by the ManagedEntityKey " + keys[i].getClass().getName()
                + " is null", val != null);
            keys[i].setPrimaryKey(val);
        }
    }

    /**
     * Verify the ApplicationContext attribute of the key implementation. It shall not support the attribution of a null value.
     * @throws Exception
     */

    //  Spec chapter 3.1.2 all filed in key can be null

    /*
        public void test_SetApplicationContextIsNull() throws Exception {
            log("==> Test: test_SetApplicationContextIsNull");

            for (int i = 0; i < keys.length; i++) {
                try {
                    keys[i].setApplicationContext(null);
                } catch (java.lang.IllegalArgumentException ex) {
                    continue;
                } catch (java.lang.Exception other) {
                    throw new TCKException(null, "ManagedEntitykey", other);
                }

                Assert.fail("ManagedEntityKeys for SetApplicationContext for null"
                    + (keys[i].getClass()).getName());
            }
        }
    */

    /**
     * Verify the ApplicationDN attribute of the key implementation. It shall not support the attribution of a null value.
     * @throws Exception
     */

    //  Spec chapter 3.1.2 all filed in key can be null

    /*
        public void test_SetApplicationDNIsNull() throws Exception {
            log("==> Test: test_SetApplicationDNIsNull");

            for (int i = 0; i < keys.length; i++) {
                try {
                    keys[i].setApplicationDN(null);
                } catch (java.lang.IllegalArgumentException ex) {
                    continue;
                } catch (java.lang.Exception other) {
                    throw new TCKException(null, "ManagedEntityKey", other);
                }

                Assert.fail("ManagedEntityKeys SetApplication DN failed for null "
                    + (keys[i].getClass()).getName());
            }
        }
    */

    /**
     * Verify the primaryKey attribute of the key implementation. It shall not support the attribution of a null value.
     * @throws Exception
     */
    public void test_SetPrimaryKeyIsNull() throws Exception {
        log("==> Test: test_SetPrimaryKeyIsNull");

        for (int i = 0; i < keys.length; i++) {
            try {
                //System.out.println("(( key" + keys[i]);
                keys[i].setPrimaryKey(null);
            } catch (java.lang.IllegalArgumentException ex) {
                continue;
            } catch (java.lang.Exception other) {
                throw new TCKException(null, "ManagedEntityKey", other);
            }

            Assert.fail("ManagedEntityKey setPrimaryKay Failed for Null"
                + (keys[i].getClass()).getName());
        }
    }

    /**
     * Verify the type attribute of the key implementation. It shall not support the attribution of a null value.
     * @throws Exception
     */
    public void test_SetTypeIsNull() throws Exception {
        log("==> Test: test_SetTypeIsNull");

        for (int i = 0; i < keys.length; i++) {
            try {
                keys[i].setType(null);
                Assert.fail("ManagedEntityKey setType Failed for Null "
                    + (keys[i].getClass()).getName());
            } catch (java.lang.IllegalArgumentException ex) {
                continue;
            } catch (java.lang.Exception other) {
                throw new TCKException(null, "ManagedEntityKey", other);
            }
        }
    }

    /**
     * Verify the Aimplementation of the clone and equals meethods.<br>
      * Expected Result:<br>
      *  a = b.clone()<br>
      *  a.equals(a) == true<br>
      *  b.equals(b) == true<br>
      *  a.equals(b) == true<br>
      *  b.equals(a) == true<br>
     * @throws Exception
     */
    public void test_KeyCloneEquals() throws Exception {
        log("==> Test: test_KeyCloneEquals");

        for (int i = 0; i < keys.length; i++) {
            if (keys[i] instanceof Cloneable) {
                Object clone = null;

                try {
                    log("++> clone " + keys[i].getClass().getName());
                    clone = keys[i].clone();
                } catch (java.lang.Exception other) {
                    log(other);
                    throw new TCKException(null, keys[i].getClass().getName(), other);
                }

                assertTrue(keys[i].getClass().getName() + " clone is null", clone != null);
                assertTrue(keys[i].getClass().getName() + " mek.equals(mek) failed",
                    keys[i].equals(keys[i]));
                assertTrue(keys[i].getClass().getName() + " clone.equals(clone) failed",
                    clone.equals(clone));
                assertTrue(keys[i].getClass().getName() + " mek.equals(clone) failed",
                    keys[i].equals(clone));
                assertTrue(keys[i].getClass().getName() + " clone.equals(mek) failed",
                    clone.equals(keys[i]));
            } else {
                log("--> " + keys[i].getClass().getName() + " is not Cloneable");
            }
        }
    }
}
