package ossj.common.tck.tests;

import junit.framework.*;

import ossj.common.tck.*;

import java.lang.reflect.*;

import java.util.*;

import javax.oss.*;


/**
 * Test case for ManagedEntityValues
 *
 *
 * @author
 * @version
 */
public class TestManagedEntityValueSingle extends TestMev {
    /**Flag if staticSetup() was called*/
    private static boolean wasSetup = false;

    /**creates a new testManagedEntityValue*/
    TestManagedEntityValueSingle(String name) throws Exception {
        super(name);
    }

    /**sets up the TestCase*/
    protected void setUp() throws Exception {
        super.setUp();
    }

    /**@return Array of make-able value object types */
    String[] getTypes(JVTSession jas) throws Exception {
        return jas.getManagedEntityTypes();
    }

    /**
        @return All suites of this class
    */
    public static TestSuite suite() throws Exception {
        TestSuite retVal = new TestSuite("ManagedValue Single");
        TestSuite domain = new TestSuite("Domain tests");
        TestSuite other = new TestSuite("Logical tests");

        Class cls = TestManagedEntityValueSingle.class;
        Method[] methods = cls.getDeclaredMethods();

        for (int i = 0; i < methods.length; ++i) {
            String name = methods[i].getName();

            if (name.startsWith(StringTable.TEST_PREFIX)) {
                if (name.startsWith(StringTable.POSITIVE_TEST_PREFIX)
                        || name.startsWith(StringTable.NEGATIVE_TEST_PREFIX)) {
                    domain.addTest(new TestManagedEntityValueSingle(name));
                } else {
                    other.addTest(new TestManagedEntityValueSingle(name));
                }
            }
        }

        retVal.addTest(domain);
        retVal.addTest(other);

        return retVal;
    }

    /*****************************************************************************************
     **
     **   genericTest
     **
     ******************************************************************************************/
    protected void doGeneric(String testName) throws Exception {
        int ind = jvtSessionList.size() - 1;
        Class[] args = { ManagedEntityValue.class };
        ManagedEntityValue mev = null;
        log("==> Managed Entity Value single Test: " + testName);

        for (; ind >= 0; ind--) {
            JVTSession jas = getInstanceOf((String) jvtSessionList.get(ind));
            String[] types = jas.getManagedEntityTypes();

            for (int t = types.length - 1; t >= 0; t--) {
                mev = jas.makeManagedEntityValue(types[t]);
                log("++> for mev [" + mev.getClass().getName() + "]");
                this.getClass().getSuperclass()
                    .getDeclaredMethod(testName, new Class[] { AttributeAccess.class }).invoke(this,
                    new Object[] { mev });
            }
        }
    }

    /*****************************************************************************************
    *
    *   Test Value and Value type and key pattern
    *
    * test if VALUE_TYPE field is declared
    * test it VALUE_TYPE field is public final static String
    * test if the declared type exist (call forName)
    * test if the class is of the declared type (decalreType.isAssignableFrom)
    * test if the VALUE_TYPE ends with "Value"
    * test if the VALUE_TYPE corresponding interface exists
    */
    public void test_ClassNameAndTypeAndValueKeyPair()
        throws Exception {
        doGeneric("test_ClassNameAndTypeAndValueKeyPair");
    }

    /*****************************************************************************************
    **
    **   getAttributeNames
    **
    ******************************************************************************************/
    /**
        tests that attributes names have length > 0 and start with lowercase letter
    */
    public void test_GetAttributeNames_NamesStartWithLowerCase()
        throws Exception {
        doGeneric("test_GetAttributeNames_NamesStartWithLowerCase");
    }

    /**
    tests that all supported optional attibutes are also listed in all attributes and
    attributes names have length > 0 and start with lowercase letter
    */
    public void test_GetOptionalAttributeNames_InAllAttributes()
        throws Exception {
        doGeneric("test_GetOptionalAttributeNames_InAllAttributes");
    }

    /*****************************************************************************************
    **
    **   isPopulated
    **
    ******************************************************************************************/
    /**
        DOMAINTEST
        tests isPopulated for all attributes in getAttributeNames()
    */
    public void test_Positive_IsPopulated_AttributeNameAllDomainValues()
        throws Exception {
        doGeneric("test_Positive_IsPopulated_AttributeNameAllDomainValues");
    }

    /**
        DOMAINTEST
        tests failure of isPopulated(null)
    */
    public void test_Negative_IsPopulated_AttributeNameNullValue()
        throws Exception {
        doGeneric("test_Negative_IsPopulated_AttributeNameNullValue");
    }

    /**
        DOMAINTEST
        tests failure of isPopulated(a) for String a not in getAttributeNames()
    */
    public void test_Negative_IsPopulated_AttributeNameIllegalValue()
        throws Exception {
        doGeneric("test_Negative_IsPopulated_AttributeNameIllegalValue");
    }

    /*****************************************************************************************
    **
    **   getSettableAttributeNames
    **
    ******************************************************************************************/
    /**
        tests that getSettableAttributesNames is a subset of getAttributeNames
    */
    public void test_GetSettableAttributeNames_SubsetOfGetAttributeNames()
        throws Exception {
        doGeneric("test_GetSettableAttributeNames_SubsetOfGetAttributeNames");
    }

    /*****************************************************************************************
    **
    **   unpopulateAttribute
    **
    ******************************************************************************************/
    /**
        DOMAINTEST
        tests succesful invokation of unpopulateAttribute(a) for all a in getAttributeNames()
    */
    public void test_Positive_UnpopulateAttribute_AttributeNameAllDomainValues()
        throws Exception {
        doGeneric("test_Positive_UnpopulateAttribute_AttributeNameAllDomainValues");
    }

    /**
        DOMAINTEST
        tests failure of unpopulatedAttribute(null)
    */
    public void test_Negative_UnpopulateAttribute_AttributeNameNullValue()
        throws Exception {
        doGeneric("test_Negative_UnpopulateAttribute_AttributeNameNullValue");
    }

    /**
        DOMAINTEST
        tests failure of unpopulatedAttribute(a) for a not in getAttributeNames
    */
    public void test_Negative_UnpopulateAttribute_AttributeNameIllegalValue()
        throws Exception {
        doGeneric("test_Negative_UnpopulateAttribute_AttributeNameIllegalValue");
    }

    /*****************************************************************************************
    **
    **   unpopulateAllAttributes
    **
    ******************************************************************************************/
    /**
        tests that unpopulateAlAttributes implies !isPopulated(a) for all a in getAttributeNames()
    */
    public void test_UnpopulateAllAttributes_IsPopulatedFalseForAll()
        throws Exception {
        doGeneric("test_UnpopulateAllAttributes_IsPopulatedFalseForAll");
    }

    /**
        tests that unpopulateAlAttributes implies getPopulatedAttributesNamesIsEmpty() is empty
    */
    public void test_UnpopulateAllAttributes_GetPopulatedAttributesNamesIsEmpty()
        throws Exception {
        doGeneric("test_UnpopulateAllAttributes_GetPopulatedAttributesNamesIsEmpty");
    }

    /**
        tests that unpopulateAlAttributes implies failure of getAttributeValue(a) for all a in getAttributeNames
    */
    public void test_UnpopulateAllAttributes_GetAttributeValueFailsForAll()
        throws Exception {
        doGeneric("test_UnpopulateAllAttributes_GetAttributeValueFailsForAll");
    }

    /*****************************************************************************************
    **
    **   setAttributeValue
    **
    ******************************************************************************************/
    /**
        DOMAINTEST
       tests failure of setAttributeValue(a) for null
    */
    public void test_Negative_SetAttributeValue_AttributeNameNullValue()
        throws Exception {
        doGeneric("test_Negative_SetAttributeValue_AttributeNameNullValue");
    }

    /**
        DOMAINTEST
       tests failure of setAttributeValue(a) for a not in getAttributeNames()
    */
    public void test_Negative_SetAttributeValue_AttributeNameIllegalValue()
        throws Exception {
        doGeneric("test_Negative_SetAttributeValue_AttributeNameIllegalValue");
    }

    /*****************************************************************************************
    **
    **   getAttributeValue
    **
    ******************************************************************************************/
    /**
        DOMAINTEST
        tests failure of getAttributeValue(null)
    */
    public void test_Negative_GetAttributeValue_AttributeNameNullValue()
        throws Exception {
        doGeneric("test_Negative_GetAttributeValue_AttributeNameNullValue");
    }

    /**
        DOMAINTEST
        tests failure of getAttributeValue(a) for a not in getAttributeValues();
    */
    public void test_Negative_GetAttributeValue_AttributeNameIllegalValue()
        throws Exception {
        doGeneric("test_Negative_GetAttributeValue_AttributeNameIllegalValue");
    }

    /*****************************************************************************************
    **
    **   setManagedEntityKey
    **
    ******************************************************************************************/
    /**
        tests a domain value
    */
    public void test_Positive_SetManagedEntityKey_DomainValue()
        throws Exception {
        doGeneric("test_Positive_SetManagedEntityKey_DomainValue");
    }

    /**
        tests a illegal key
    */
    public void test_Positive_SetManagedEntityKey_IllegalKey()
        throws Exception {
        doGeneric("test_Positive_SetManagedEntityKey_IllegalKey");
    }

    /**
        tests a illegal key
    */
    public void test_Positive_SetAttributeValueKEY_IllegalKey()
        throws Exception {
        doGeneric("test_Positive_SetAttributeValueKEY_IllegalKey");
    }

    /**
        tests that succesful setManagedEntityKey(val) implies KEY in getPopulatedAttributeNames()

    */
    public void test_SetManagedEntityKey_GetPopulatedAttributeNamesContainsKEY()
        throws Exception {
        doGeneric("test_SetManagedEntityKey_GetPopulatedAttributeNamesContainsKEY");
    }

    /**
        tests that succesful setManagedEntityKey(val) implies isPopulated(KEY)
    */
    public void test_SetManagedEntityKey_IsPopulatedKEY()
        throws Exception {
        doGeneric("test_SetManagedEntityKey_IsPopulatedKEY");
    }

    /*****************************************************************************************
    **
    **   setManagedEntityKey
    **
    ******************************************************************************************/

    // this method tries to check weather the getattributes return key as attribute or not 
    public void test_GetAttributeNames_ContainsKEY() throws Exception {
        doGeneric("test_GetAttributeNames_ContainsKEY");
    }

    // this method requirement has to be enquired with daniel 
    public void test_GetSettableAttributeNames_NotContainsKEY()
        throws Exception {
        doGeneric("test_GetSettableAttributeNames_NotContainsKEY");
    }

    // this checks for invalid managedentitykey 
    public void test_Positive_ManagedEntityKey_IllegalKey()
        throws Exception {
        doGeneric("test_Positive_ManagedEntityKey_IllegalKey");
    }

    /*****************************************************************************************
    **
    **   setAttributeValue
    **
    ******************************************************************************************/
    /**
        tests setAttributeValue(min/max) for primitive attributes that are not enums
    */

    // problem with creation date 
    public void test_Positive_SetAttributeValue_PrimitiveMINMAXValues()
        throws Exception {
        doGeneric("test_Positive_SetAttributeValue_PrimitiveMINMAXValues");
    }

    /**
     * DOCUMENT ME!
     *
     * @throws Exception DOCUMENT ME!
     */
    public void test_Negative_SetAttributeValue_PrimitiveNullValue()
        throws Exception {
        doGeneric("test_Negative_SetAttributeValue_PrimitiveNullValue");
    }

    // this is a generic method to test the setattribute value function based on the type of the field  
    public void test_Positive_SetAttributeValue_DomainValue()
        throws Exception {
        doGeneric("test_Positive_SetAttributeValue_DomainValue");
    }

    /**
     tests that setAttributeValue(a) => a in getPopulatedAttributes()
    */
    public void test_SetAttributeValue_GetPopulatedAttributesNamesContainsAttributeName()
        throws Exception {
        doGeneric("test_SetAttributeValue_GetPopulatedAttributesNamesContainsAttributeName");
    }

    /**
        tests that setAttributeValue(a) => isPopulated(a)
    */
    public void test_SetAttributeValue_IsPopulated() throws Exception {
        doGeneric("test_SetAttributeValue_IsPopulated");
    }

    /**
        DOMAINTEST
       tests failure of setAttributeValue(a) for an illegal object()
    */
    public void test_Negative_SetAttributeValue_IllegalObject()
        throws Exception {
        doGeneric("test_Negative_SetAttributeValue_IllegalObject");
    }

    /*****************************************************************************************
     **
     **   clone equals
     **
     ******************************************************************************************/
    /**
     * Test clone and equals method implementations
     * Expected Result:
     *  a = b.clone()
     *  a.equals(a) == true
     *  b.equals(b) == true
     *  a.equals(b) == true
     *  b.equals(a) == true
     *
     */
    public void test_Positive_testCloneEqualsImplementation()
        throws Exception {
        doGeneric("test_Positive_testCloneEqualsImplementation");
    }
}
