package ossj.common.tck.tests;

import java.beans.BeanInfo;
import java.beans.Introspector;
import junit.framework.*;

import ossj.common.tck.*;

import java.beans.PropertyDescriptor;

import java.lang.reflect.*;

import java.util.*;

import javax.oss.*;


/**
 * Test case for ManagedEntityValues
 *
 * @author Vincent Perrot, Sun Microsystems Inc.
 * @version
 */
public class TestMev extends ossj.common.tck.JVTTestBase {
    /**
     * creates a new testManagedEntityValue
     */
    TestMev(String name) throws Exception {
        super(name);
    }
    
    /**sets up the TestCase*/
    protected void setUp() throws Exception {
        init();
    }
    
    
    /*****************************************************************************************
     *
     *   Test Value and Value type and key pattern<BR>
     *
     * test if VALUE_TYPE field is declared<BR>
     * test it VALUE_TYPE field is public final static String<BR>
     * test if the declared type exist (call forName)<BR>
     * test if the class is of the declared type (decalreType.isAssignableFrom)<BR>
     * test if the VALUE_TYPE ends with "Value"<BR>
     * test if the VALUE_TYPE corresponding key interface exists<BR>
     * test if the get/set/make<key> methods exist and use/return the correct key type<BR>
     * test if set<key>(make<key>) don't throw exceptions
     */
    public void test_ClassNameAndTypeAndValueKeyPair(AttributeAccess mev)
    throws Exception {
        String className = mev.getClass().getName();
        Field vtField = null;
        
        try {
            vtField = mev.getClass().getField("VALUE_TYPE");
        } catch (NoSuchFieldException nsf) {
            fail("The ManagedEntityValue [" + className
                    + "] do not declare the mandatory public static final VALUE_TYPE field");
        }
        
        assertTrue("VALUE_TYPE field of the ManagedEntityValue [" + className
                + " shall be public final static instead of "
                + Modifier.toString(vtField.getModifiers()),
                Modifier.isFinal(vtField.getModifiers()) && Modifier.isStatic(vtField.getModifiers())
                && Modifier.isPublic(vtField.getModifiers()));
        assertTrue("VALUE_TYPE field of the ManagedEntityValue [" + className
                + " shall be String of " + vtField.getType().getName(),
                String.class.equals(vtField.getType()));
        
        String vtName = null;
        
        try {
            vtName = (String) vtField.get(null);
        } catch (Exception ex) {
            log(ex);
            fail("Unexpected exception when trying to get the VALUE_TYPE field of [" + className
                    + "]");
        }
        
        log("++> [" + className + "] is a [" + vtName + "]");
        
        //exception done for CBEManagedEntityValueImpl
        if (className.endsWith(".CBEManagedEntityValueImpl")) {
            return;
        }
        
        Class declaredType = null;
        
        try {
            declaredType = Class.forName(vtName);
        } catch (Exception ex) {
            log(ex);
            fail("unable to get interface declared [" + vtName
                    + "] in the VALUE_TYPE field of class [" + className + "]");
        }
        
        try {
            declaredType.isAssignableFrom(mev.getClass());
        } catch (Exception ex) {
            log(ex);
            fail("unable to cast using declared [" + vtName
                    + "] in the VALUE_TYPE field the class [" + className + "]");
        }
        
        assertTrue("the type [" + vtName + "] of class [" + className + "]of shall end with Value",
                vtName.endsWith("Value"));
        
        Class key = null;
        String keyName = vtName.substring(0, vtName.lastIndexOf("Value")) + "Key";
        
        try {
            key = Class.forName(keyName);
        } catch (Exception ex) {
            log(ex);
            fail("Unable to get Key [" + keyName + "] pair of Value [" + vtName + "] for class ["
                    + className + "]");
        }
        
        String shortKeyName = keyName.substring(keyName.lastIndexOf(".") + 1);
        Method meth = null;
        
        try {
            meth = mev.getClass().getDeclaredMethod("get" + shortKeyName, (Class[])null);
            assertTrue("The get" + shortKeyName
                    + "() method return the wrong key type (not assignable to " + shortKeyName + ")",
                    key.isAssignableFrom(meth.getReturnType()));
        } catch (Exception ex) {
            log(ex);
            fail("Unable to find the get" + shortKeyName + "() method in [" + className + "]");
        }
        
        try {
            meth = mev.getClass().getDeclaredMethod("set" + shortKeyName, new Class[]{key});
        } catch (Exception ex) {
            log(ex);
            fail("Unable to find the set" + shortKeyName + "(" + shortKeyName + ") method in ["
                    + className + "]");
        }
        
        Method makeMeth = null;
        
        try {
            makeMeth = mev.getClass().getDeclaredMethod("make" + shortKeyName, (Class[])null);
            assertTrue("The make" + shortKeyName
                    + "() method return the wrong key type (not assignable to " + shortKeyName + ")",
                    key.isAssignableFrom(makeMeth.getReturnType()));
        } catch (Exception ex) {
            log(ex);
            fail("Unable to find the make" + shortKeyName + "() method in [" + className + "]");
        }
        
        try {
            Object[] args = new Object[1];
            args[0] = makeMeth.invoke(mev, (Object[])null);
            meth.invoke(mev, args);
        } catch (Exception ex) {
            log(ex);
            fail("Fail to invoke set" + shortKeyName + "( make" + shortKeyName + "()) method in ["
                    + className + "]");
        }
    }
    
    /*****************************************************************************************
     *
     *   Test QUERY_TYPE<BR>
     *<BR>
     * test if QUERY_TYPE field is declared<BR>
     * test it QUERY_TYPE field is public final static String<BR>
     * test if the declared type exist (call forName)<BR>
     * test if the class is of the declared type (decalreType.isAssignableFrom)<BR>
     * test if the QUERY_TYPE corresponding interface exists<BR>
     */
    public void test_Query_Type(AttributeAccess mev) throws Exception {
        String className = mev.getClass().getName();
        Field vtField = null;
        
        try {
            vtField = mev.getClass().getField("QUERY_TYPE");
        } catch (NoSuchFieldException nsf) {
            fail("The QueryValue [" + className
                    + "] do not declare the mandatory public static final QUERY_TYPE field");
        }
        
        assertTrue("QUERY_TYPE field of the QueryValue [" + className
                + " shall be public final static instead of "
                + Modifier.toString(vtField.getModifiers()),
                Modifier.isFinal(vtField.getModifiers()) && Modifier.isStatic(vtField.getModifiers())
                && Modifier.isPublic(vtField.getModifiers()));
        assertTrue("QUERY_TYPE field of the QueryValue [" + className + " shall be String of "
                + vtField.getType().getName(), String.class.equals(vtField.getType()));
        
        String vtName = null;
        
        try {
            vtName = (String) vtField.get(null);
        } catch (Exception ex) {
            log(ex);
            fail("Unexpected exception when trying to get the QUERY_TYPE field of [" + className
                    + "]");
        }
        
        log("++> [" + className + "] is a [" + vtName + "]");
        
        Class declaredType = null;
        
        try {
            declaredType = Class.forName(vtName);
        } catch (Exception ex) {
            log(ex);
            fail("unable to get interface declared [" + vtName
                    + "] in the QUERY_TYPE field of class [" + className + "]");
        }
        
        try {
            declaredType.isAssignableFrom(mev.getClass());
        } catch (Exception ex) {
            log(ex);
            fail("unable to cast using declared [" + vtName
                    + "] in the QUERY_TYPE field the class [" + className + "]");
        }
    }
    
    /*****************************************************************************************
     *
     *   Test UPDATE_TYPE<BR>
     *<BR>
     * test if UPDATE_TYPE field is declared<BR>
     * test it UPDATE_TYPE field is public final static String<BR>
     * test if the declared type exist (call forName)<BR>
     * test if the class is of the declared type (decalreType.isAssignableFrom)<BR>
     * test if the UPDATE_TYPE corresponding interface exists<BR>
     */
    public void test_upv_Type(AttributeAccess mev) throws Exception {
        String className = mev.getClass().getName();
        Field vtField = null;
        
        try {
            vtField = mev.getClass().getField("UPDATE_TYPE");
        } catch (NoSuchFieldException nsf) {
            fail("The UpdateProcedureValue [" + className
                    + "] do not declare the mandatory public static final UPDATE_TYPE field");
        }
        
        assertTrue("UPDATE_TYPE field of the UpdateProcedureValue [" + className
                + " shall be public final static instead of "
                + Modifier.toString(vtField.getModifiers()),
                Modifier.isFinal(vtField.getModifiers()) && Modifier.isStatic(vtField.getModifiers())
                && Modifier.isPublic(vtField.getModifiers()));
        assertTrue("UPDATE_TYPE field of the UpdateProcedureValue [" + className
                + " shall be String of " + vtField.getType().getName(),
                String.class.equals(vtField.getType()));
        
        String vtName = null;
        
        try {
            vtName = (String) vtField.get(null);
        } catch (Exception ex) {
            log(ex);
            fail("Unexpected exception when trying to get the UPDATE_TYPE field of [" + className
                    + "]");
        }
        
        log("++> [" + className + "] is a [" + vtName + "]");
        
        Class declaredType = null;
        
        try {
            declaredType = Class.forName(vtName);
        } catch (Exception ex) {
            log(ex);
            fail("unable to get interface declared [" + vtName
                    + "] in the UPDATE_TYPE field of class [" + className + "]");
        }
        
        try {
            declaredType.isAssignableFrom(mev.getClass());
        } catch (Exception ex) {
            log(ex);
            fail("unable to cast using declared [" + vtName
                    + "] in the UPDATE_TYPE field the class [" + className + "]");
        }
    }
    
    /*****************************************************************************************
     **
     **   getAttributeNames
     **
     ******************************************************************************************/
    /**
     * tests that attributes names have length > 0 and start with lowercase letter<BR>
     * <BR>
     *  - call getAttributeNames check if return array length>0 and != null<BR>
     *  - Fall all attribute names check name length >0 name following bean convention<BR>
     * bean spec: However to support the occasional use of all
     * upper-case names, we check if the first two characters
     * of the name are both upper case and if so leave it alone.
     *
     */
    public void test_GetAttributeNames_NamesStartWithLowerCase(AttributeAccess mev)
    throws Exception {
        String[] names = util.deepCopy(mev.getAttributeNames());
        
        assertTrue(StringTable.TYPE + mev.getClass().getName() + "#: getAttributeNames() is null",
                null != names);
        assertTrue(StringTable.TYPE + mev.getClass().getName() + "#: getAttributeNames() is empty",
                ((names.length > 0)
                || ((names.length == 0) && !ManagedEntityValue.class.isAssignableFrom(mev.getClass()))));
        
        for (int i = 0; i < names.length; ++i) {
            assertTrue(names[i], mev.getClass().getName() + " #:minimum length > 0!",
                    0 < names[i].length());
            
            // bean spec:
            // "However to support the occasional use of all
            // upper-case names, we check if the first two characters
            // of the name are both upper case and if
            // so leave it alone."
            
            char firstLetter = names[i].charAt(0);
            char secondLetter = 'a';
            
            if (names[i].length() > 1) {
                secondLetter = names[i].charAt(1);
            }
            
            // will throw exception if set/get not OK
            PropertyDescriptor prop = null;
            
            try {
                prop = new PropertyDescriptor(names[i], mev.getClass());
            } catch (java.beans.IntrospectionException ex) {
                // tipical case is when get without set
                log(mev.getClass().getName() + " ex.getMessage()" + " use only the getter for "
                        + names[i]);
            }
            assertTrue(names[i], mev.getClass().getName() + " #:name shall be equals to "+Introspector.decapitalize(names[i]),
                    names[i].equals(Introspector.decapitalize(names[i])));
            
            assertTrue(names[i], mev.getClass().getName() + " #:first letter is not lower case",
                    Character.isLowerCase(firstLetter)
                    || (!Character.isLowerCase(firstLetter) && !Character.isLowerCase(secondLetter)));
        }
    }
    
    /**
     * tests same as test_GetAttributeNames_NamesStartWithLowerCase for optional attributes
     * @see #test_GetAttributeNames_NamesStartWithLowerCase
     */
    public void test_GetOptionalAttributeNames_InAllAttributes(AttributeAccess mev)
    throws Exception {
        String[] names = util.deepCopy(mev.getSupportedOptionalAttributeNames());
        Collection col = Arrays.asList(mev.getAttributeNames());
        
        if (names.length > 0) {
            for (int i = names.length - 1; i >= 0; i--) {
                assertTrue(names[i],
                        mev.getClass().getName() + " #:is not preesent in all attributes array!",
                        col.contains(names[i]));
                
                char firstLetter = names[i].charAt(0);
                
                // will throw exception if set/get note OK
                PropertyDescriptor prop = new PropertyDescriptor(names[i], mev.getClass());
                assertTrue(names[i],
                        mev.getClass().getName() + " #:first letter is not lower case",
                        Character.isLowerCase(firstLetter));
            }
        }
    }
    
    /*****************************************************************************************
     **
     **   isPopulated
     **
     ******************************************************************************************/
    /**
     * tests isPopulated for all attributes in getAttributeNames()<br>
     * call ispopulated for all attributes fails if an exception is thrown
     */
    public void test_Positive_IsPopulated_AttributeNameAllDomainValues(AttributeAccess mev)
    throws Exception {
        // use copy of attribute name to ensure implementaion .equals() checks
        String[] vals = util.deepCopy(mev.getAttributeNames());
        
        //tests
        boolean test;
        int i = 0;
        
        for (; i < vals.length; ++i) {
            try {
                test = mev.isPopulated(vals[i]);
            } catch (java.lang.IllegalArgumentException ex) {
                fail(vals[i], mev.getClass().getName());
            } catch (java.lang.Exception other) {
                log(other);
                throw new TCKException(vals[i], mev.getClass().getName(), other);
            }
        }
    }
    
    /**
     * tests failure of isPopulated(null)<br>
     * Successed if java.lang.IllegalArgumentException is thrown
     */
    public void test_Negative_IsPopulated_AttributeNameNullValue(AttributeAccess mev)
    throws Exception {
        try {
            mev.isPopulated(null);
            fail(mev.getClass().getName() + " ispopulated(null): Expected IllegalArgumentException");
        } catch (java.lang.IllegalArgumentException ex) {
            // do nothing, it is OK
        } catch (java.lang.Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /**
     * tests failure of isPopulated(a) for String a not in getAttributeNames()<br>
     * Successed if java.lang.IllegalArgumentException or javax.oss.OssUnsupportedAttributeException is thrown
     */
    public void test_Negative_IsPopulated_AttributeNameIllegalValue(AttributeAccess mev)
    throws Exception {
        try {
            mev.isPopulated(util.getIllegalValue(mev.getAttributeNames(), 1)[0]);
            fail(mev.getClass().getName()
            + " isPopulated(illegal) expected UnsupportedAttributeException or IllegalArgumentException");
        } catch (java.lang.IllegalArgumentException iaex) {
            // OK this is awaited exception
        } catch (javax.oss.OssUnsupportedAttributeException uaex) {
            // OK this is awaited exception
        } catch (java.lang.Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /*****************************************************************************************
     **
     **   getSettableAttributeNames
     **
     ******************************************************************************************/
    /**
     * tests that getSettableAttributesNames is a subset of getAttributeNames<<br>
     * Get the settable attribute list and check that all attribute names are also present in the list returned by getAttributeNames
     */
    public void test_GetSettableAttributeNames_SubsetOfGetAttributeNames(AttributeAccess ac)
    throws Exception {
        ManagedEntityValue mev = (ManagedEntityValue) ac;
        String[] names = mev.getSettableAttributeNames();
        String[] domain = mev.getAttributeNames();
        
        //tests
        for (int i = 0; i < names.length; ++i)
            assertTrue(names[i],
                    mev.getClass().getName() + " settable attribute is not in attribute list",
                    util.contains(domain, names[i]));
    }
    
    /*****************************************************************************************
     **
     **   unpopulateAttribute
     **
     ******************************************************************************************/
    /**
     * tests succesful invocation of unpopulateAttribute(a) for all a in getAttributeNames()
     */
    public void test_Positive_UnpopulateAttribute_AttributeNameAllDomainValues(AttributeAccess mev)
    throws Exception {
        // use copy of attribute name to ensure implementaion .equals() checks
        String[] names = util.deepCopy(mev.getAttributeNames());
        util.shuffle(names);
        
        for (int i = 0; i < names.length; i++) {
            try {
                mev.unpopulateAttribute(names[i]);
            } catch (java.lang.IllegalArgumentException ex) {
                fail(names[i], mev.getClass().getName());
            } catch (java.lang.Exception other) {
                log(other);
                
                //TODO assert or throw ???
                Assert.assertTrue((String) mev.getClass().getName() + ": Unexpected " + other
                        + " when unpopulate " + names[i], false);
                throw new TCKException(null, mev.getClass().getName(), other);
            }
        }
    }
    
    /**
     * tests failure of unpopulatedAttribute(null)<br>
     * Successed if java.lang.IllegalArgumentException is thrown
     */
    public void test_Negative_UnpopulateAttribute_AttributeNameNullValue(AttributeAccess mev)
    throws Exception {
        try {
            mev.unpopulateAttribute(null);
            fail(mev.getClass().getName()
            + " unpopulateAttribute(null) expected IllegalArgumentException");
        } catch (java.lang.IllegalArgumentException ex) {
            // OK this is awaited exception
        } catch (java.lang.Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /**
     * tests failure of unpopulatedAttribute(a) for an argument not in getAttributeNames<br>
     * Successed if java.lang.IllegalArgumentException is thrown
     */
    public void test_Negative_UnpopulateAttribute_AttributeNameIllegalValue(AttributeAccess mev)
    throws Exception {
        try {
            mev.unpopulateAttribute(util.getIllegalValue(mev.getAttributeNames(), 1)[0]);
            fail(mev.getClass().getName()
            + " unpopulateAttribute(illegal) expected IllegalArgumentException");
        } catch (java.lang.IllegalArgumentException ex) {
            // OK this is awaited exception
        } catch (java.lang.Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /*****************************************************************************************
     **
     **   unpopulateAllAttributes
     **
     ******************************************************************************************/
    /**
     * tests that unpopulateAlAttributes implies !isPopulated(a) for all a in getAttributeNames()
     */
    public void test_UnpopulateAllAttributes_IsPopulatedFalseForAll(AttributeAccess mev)
    throws Exception {
        mev.unpopulateAllAttributes();
        
        //tests
        // use copy of attribute name to ensure implementaion .equals() checks
        String[] names = util.deepCopy(mev.getAttributeNames());
        
        for (int i = 0; i < names.length; ++i) {
            assertTrue(names[i],
                    mev.getClass().getName() + " this attribute shall not be populated",
                    !mev.isPopulated(names[i]));
        }
    }
    
    /**
     * tests that unpopulateAlAttributes implies getPopulatedAttributesNamesIsEmpty() is empty
     */
    public void test_UnpopulateAllAttributes_GetPopulatedAttributesNamesIsEmpty(AttributeAccess mev)
    throws Exception {
        mev.unpopulateAllAttributes();
        
        //tests
        String[] names = mev.getPopulatedAttributeNames();
        assertTrue(mev.getClass().getName() + " #:array is null", null != names);
        assertTrue(mev.getClass().getName() + " #:array is not empty", 0 == names.length);
    }
    
    /**
     * tests that unpopulateAlAttributes implies failure of getAttributeValue(a) for all a in getAttributeNames
     */
    public void test_UnpopulateAllAttributes_GetAttributeValueFailsForAll(AttributeAccess mev)
    throws Exception {
        mev.unpopulateAllAttributes();
        
        // use copy of attribute name to ensure implementaion .equals() checks
        String[] names = util.deepCopy(mev.getAttributeNames());
        Object obj;
        
        //tests
        for (int i = names.length - 1; i >= 0; i--) {
            try {
                obj = mev.getAttributeValue(names[i]);
                fail(mev.getClass().getName()
                + " getAttributeValue() when unpopulate expected IllegalStateException for attribute name "
                        + names[i]);
            } catch (java.lang.IllegalStateException ex) {
                continue;
            } catch (java.lang.Exception other) {
                log(other);
                throw new TCKException(names[i], mev.getClass().getName(), other);
            }
            
            // use now the real method implementation.
            try {
                PropertyDescriptor prop = new PropertyDescriptor(names[i], mev.getClass());
                prop.getReadMethod().invoke(mev, (Object[])null);
                fail(mev.getClass().getName()
                + " getAttributeValue() when unpopulate expected IllegalStateException for attribute name "
                        + names[i]);
            } catch (java.lang.IllegalStateException ex) {
                continue;
            } catch (java.lang.Exception other) {
                log(other);
                throw new TCKException(names[i], mev.getClass().getName(), other);
            }
        }
    }
    
    /*****************************************************************************************
     **
     **   setAttributeValue
     **
     ******************************************************************************************/
    /**
     * tests failure of setAttributeValue(null, value) <br>
     * Suceesed if java.lang.IllegalArgumentException is thrown
     */
    public void test_Negative_SetAttributeValue_AttributeNameNullValue(AttributeAccess mev)
    throws Exception {
        Object value = new Object();
        
        //tests
        try {
            mev.setAttributeValue(null, value);
            fail(mev.getClass().getName()
            + " setAttributeValue(null, value) expected IllegalArgumentException");
        } catch (java.lang.IllegalArgumentException ex) {
            // OK this is awaited exception
        } catch (java.lang.Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /**
     * tests failure of setAttributeValue(invalid_argument, value)
     * Suceesed if java.lang.IllegalArgumentException is thrown
     */
    public void test_Negative_SetAttributeValue_AttributeNameIllegalValue(AttributeAccess mev)
    throws Exception {
        Object value = new Object();
        
        //tests
        try {
            mev.setAttributeValue(util.getIllegalValue(mev.getAttributeNames(), 1)[0], value);
            fail(mev.getClass().getName()
            + " setAttributeValue(illegal, value) expected IllegalArgumentException");
        } catch (java.lang.IllegalArgumentException ex) {
            // OK this is awaited exception
        } catch (java.lang.Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /*****************************************************************************************
     **
     **   getAttributeValue
     **
     ******************************************************************************************/
    /**
     * tests failure of getAttributeValue(null)
     * Suceesed if java.lang.IllegalArgumentException is thrown
     */
    public void test_Negative_GetAttributeValue_AttributeNameNullValue(AttributeAccess mev)
    throws Exception {
        try {
            mev.getAttributeValue(null);
            fail(mev.getClass().getName()
            + " getAttributeValue(null) expected IllegalArgumentException");
        } catch (java.lang.IllegalArgumentException ex) {
            // OK this is awaited exception
        } catch (java.lang.Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /**
     * tests failure of getAttributeValue(invalid_argument)
     */
    public void test_Negative_GetAttributeValue_AttributeNameIllegalValue(AttributeAccess mev)
    throws Exception {
        try {
            mev.getAttributeValue(util.getIllegalValue(mev.getAttributeNames(), 1)[0]);
            fail(mev.getClass().getName()
            + " getAttributeValue(illegal) expected IllegalArgumentException");
        } catch (java.lang.IllegalArgumentException ex) {
            // OK this is awaited exception
        } catch (java.lang.Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /*****************************************************************************************
     **
     **   setManagedEntityKey
     **
     ******************************************************************************************/
    /**
     * tests based key management<br>
     * Call setManagedEntityKey(mev.makeManagedEntityKey());
     *
     */
    public void test_Positive_SetManagedEntityKey_DomainValue(AttributeAccess ac)
    throws Exception {
        ManagedEntityValue mev = (ManagedEntityValue) ac;
        
        try {
            mev.setManagedEntityKey(mev.makeManagedEntityKey());
        } catch (java.lang.IllegalArgumentException ex) {
            fail(mev.getClass().getName() + " fail to set managed entity key.");
        } catch (java.lang.Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /**
     * tests illegal keys<br>
     * Illegal keys are:
     * <UL>
     * <LI> type=""
     * <LI> type=invalid_prefix+getType()
     * <LI> promary key = new object()
     * <LI> primaryKey-newObject + invalid type
     * </UL>
     *
     */
    public void test_Positive_SetManagedEntityKey_IllegalKey(AttributeAccess ac)
    throws Exception {
        //tests
        ManagedEntityValue mev = (ManagedEntityValue) ac;
        Vector illegalsKeys = util.getIllegalKeys(mev);
        
        try {
            for (Enumeration e = illegalsKeys.elements(); e.hasMoreElements();) {
                mev.setManagedEntityKey((ManagedEntityKey) e.nextElement());
            }
        } catch (java.lang.IllegalArgumentException ex) {
            //TODO VP shall sucess here????
            log(ex);
            fail(mev.getClass().getName());
        } catch (java.lang.Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /**
     * tests a illegal key
     */
    public void test_Positive_SetAttributeValueKEY_IllegalKey(AttributeAccess ac)
    throws Exception {
        ManagedEntityValue mev = (ManagedEntityValue) ac;
        Vector illegalsKeys = util.getIllegalKeys(mev);
        
        try {
            for (Enumeration e = illegalsKeys.elements(); e.hasMoreElements();) {
                // use copy to ensure .equals checking in the implementation
                mev.setAttributeValue(new String(ManagedEntityValue.KEY),
                        (ManagedEntityKey) e.nextElement());
            }
        } catch (java.lang.IllegalArgumentException ex) {
            //TODO VP shall sucess here????
            log(ex);
            fail(mev.getClass().getName());
        } catch (java.lang.Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /**
     *  tests that succesful setManagedEntityKey(val) implies KEY in getPopulatedAttributeNames()
     */
    public void test_SetManagedEntityKey_GetPopulatedAttributeNamesContainsKEY(AttributeAccess ac)
    throws Exception {
        ManagedEntityValue mev = (ManagedEntityValue) ac;
        
        try {
            mev.setManagedEntityKey(mev.makeManagedEntityKey());
            
            //tests
            String[] names = mev.getPopulatedAttributeNames();
            assertTrue(mev.getClass().getName() + " #:array is null", null != names);
            assertTrue(mev.getClass().getName() + " #:KEY not found ",
                    util.contains(names, ManagedEntityValue.KEY));
        } catch (java.lang.Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /**
     * tests that succesful setManagedEntityKey(val) implies isPopulated(KEY)
     */
    public void test_SetManagedEntityKey_IsPopulatedKEY(AttributeAccess ac)
    throws Exception {
        ManagedEntityValue mev = (ManagedEntityValue) ac;
        
        try {
            mev.setManagedEntityKey(mev.makeManagedEntityKey());
            assertTrue(mev.getClass().getName() + " key Attribute shall be populated.",
                    mev.isPopulated(ManagedEntityValue.KEY));
        } catch (java.lang.Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /*****************************************************************************************
     **
     **   setManagedEntityKey
     **
     ******************************************************************************************/
    /**
     * this method tries to check weather the getattributes return KEY as attribute or not
     */
    public void test_GetAttributeNames_ContainsKEY(AttributeAccess ac)
    throws Exception {
        ManagedEntityValue mev = (ManagedEntityValue) ac;
        String[] names = mev.getAttributeNames();
        
        //tests
        assertTrue(mev.getClass().getName() + " #:expected non null value", names != null);
        assertTrue(mev.getClass().getName() + " #:attribute KEY not found",
                util.contains(names, ManagedEntityValue.KEY));
    }
    
    /**
     * The KEY attribute is usually set on the server side,
     * so this test just looks at the absence of the "KEY" attribute from the settables attribute list
     */
    public void test_GetSettableAttributeNames_NotContainsKEY(AttributeAccess ac)
    throws Exception {
        ManagedEntityValue mev = (ManagedEntityValue) ac;
        String[] names = mev.getSettableAttributeNames();
        assertTrue(mev.getClass().getName() + "#: attribute found",
                !util.contains(names, ManagedEntityValue.KEY));
    }
    
    // this checks for invalid managedentitykey
    public void test_Positive_ManagedEntityKey_IllegalKey(AttributeAccess ac)
    throws Exception {
        ManagedEntityValue mev = (ManagedEntityValue) ac;
        Vector v = util.getIllegalKeys(mev);
        
        try {
            for (Enumeration e = v.elements(); e.hasMoreElements();)
                mev.setManagedEntityKey((ManagedEntityKey) e.nextElement());
        } catch (java.lang.IllegalArgumentException ex) {
            fail(mev.getClass().getName() + ": " + mev.getClass().getName());
        } catch (Exception other) {
            log(other);
            throw new TCKException(null, mev.getClass().getName(), other);
        }
    }
    
    /*****************************************************************************************
     **
     **   setAttributeValue
     **
     ******************************************************************************************/
    /**
     * tests setAttributeValue(min/max) for primitive attributes that are not enums
     */
    
    // problem with creation date
    public void test_Positive_SetAttributeValue_PrimitiveMINMAXValues(AttributeAccess mev)
    throws Exception {
        // use copy of attribute name to ensure implementaion .equals() checks
        String[] names = util.deepCopy(mev.getAttributeNames());
        Class cls;
        
        for (int i = 0; i < names.length; ++i) {
            cls = util.getAttributeType(mev, names[i]);
            
            if (cls.isPrimitive()) {
                try {
                    mev.setAttributeValue(names[i], util.getMaxAsObject(cls));
                } catch (java.lang.IllegalArgumentException ex) {
                    /* the max may be illegal for enum mev.getClass().getName() (set as int) for example so remove this test
                        fail( names[i], mev.getClass().getName()+"#: MAX value failed" );
                     */
                } catch (Exception other) {
                    log(other);
                    throw new TCKException(names[i],
                            mev.getClass().getName() + "#: MAX value failed", other);
                }
                
                try {
                    mev.setAttributeValue(names[i], util.getMinAsObject(cls));
                } catch (java.lang.IllegalArgumentException ex) {
                    /* the min may be illegal for enum mev.getClass().getName() (set as int) for example so remove this test
                        fail( names[i], mev.getClass().getName()+"#: MIN value failed" );
                     */
                } catch (Exception other) {
                    log(other);
                    throw new TCKException(names[i],
                            mev.getClass().getName() + "#: MIN value failed", other);
                }
            }
        }
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param mev DOCUMENT ME!
     *
     * @throws Exception DOCUMENT ME!
     * @throws TCKException DOCUMENT ME!
     */
    public void test_Negative_SetAttributeValue_PrimitiveNullValue(AttributeAccess mev)
    throws Exception {
        // use copy of attribute name to ensure implementaion .equals() checks
        String[] names = util.deepCopy(mev.getAttributeNames());
        Class cls;
        
        //tests
        for (int i = 0; i < names.length; ++i) {
            cls = util.getAttributeType(mev, names[i]);
            
            if (cls.isPrimitive()) {
                try {
                    mev.setAttributeValue(names[i], null);
                    fail(names[i] + " type=" + mev.getClass().getName()
                    + " #: expected IllegalArgument Exception");
                } catch (java.lang.IllegalArgumentException ex) {
                    continue;
                } catch (Exception other) {
                    log(other);
                    throw new TCKException(names[i], mev.getClass().getName(), other);
                }
            }
        }
    }
    
    // this is a generic method to test the setattribute value function based on the type of the field
    public void test_Positive_SetAttributeValue_DomainValue(AttributeAccess mev)
    throws Exception {
        // use copy of attribute name to ensure implementaion .equals() checks
        String[] names = util.deepCopy(mev.getAttributeNames());
        assertTrue(mev.getClass().getName() + "#: getAttributeNames is null", null != names);
        util.shuffle(names);
        
        Object value = null;
        Class cls = null;
        
        //tests
        for (int i = 0; i < names.length; ++i) {
            cls = util.getAttributeType(mev, names[i]);
            
            if (!cls.isPrimitive()) {
                if (ManagedEntityValue.class.isAssignableFrom(mev.getClass())
                && names[i].equals(ManagedEntityValue.KEY)) {
                    value = ((ManagedEntityValue) mev).makeManagedEntityKey();
                } else {
                    value = util.randomValue(names[i], mev);
                }
                
                if (MultiValueList.class.isAssignableFrom(cls)) {
                    int mod = ((MultiValueList) value).getModifier();
                    assertTrue("invalid modifier value for " + cls.getName(),
                            (mod != MultiValueList.NONE) || (mod != MultiValueList.SET)
                            || (mod != MultiValueList.ADD) || (mod != MultiValueList.REMOVE));
                    
                    Method[] meths = cls.getMethods();
                    ArrayList methNames = new ArrayList();
                    
                    for (int o = meths.length - 1; o >= 0; o--) {
                        methNames.add(meths[o].getName());
                    }
                    
                    assertTrue("the MultiValueList class " + cls.getName()
                    + " do not contain the mandatory method set", methNames.contains("set"));
                    assertTrue("the MultiValueList class " + cls.getName()
                    + " do not contain the mandatory method add", methNames.contains("add"));
                    assertTrue("the MultiValueList class " + cls.getName()
                    + " do not contain the mandatory method remove",
                            methNames.contains("remove"));
                    assertTrue("the MultiValueList class " + cls.getName()
                    + " do not contain the mandatory method get", methNames.contains("get"));
                    assertTrue("the MultiValueList class " + cls.getName()
                    + " do not contain the mandatory method make", methNames.contains("make"));
                    assertTrue("the MultiValueList class " + cls.getName()
                    + " do not contain the mandatory method clone", methNames.contains("clone"));
                    assertTrue("the MultiValueList class " + cls.getName()
                    + " do not contain the mandatory method reset", methNames.contains("reset"));
                }
                
                try {
                    mev.setAttributeValue(names[i], value);
                } catch (java.lang.IllegalArgumentException ex) {
                    log(ex);
                    
                    // VP Note: not necessarily a failure as some agument like "state"
                    // can receive only a subset of valid values
                    // and IllegalArgument Exception is valid in the interface
                    // fail(names[i], mev.getClass().getName());
                } catch (Exception other) {
                    log(other);
                    throw new TCKException(names[i], mev.getClass().getName(), other);
                }
                
                try {
                    PropertyDescriptor prop = new PropertyDescriptor(names[i], mev.getClass());
                    Object[] args = new Object[1];
                    args[0] = value;
                    prop.getWriteMethod().invoke(mev, args);
                } catch (java.lang.reflect.InvocationTargetException ex) {
                    log(ex);
                    
                    if (!(ex.getTargetException() instanceof java.lang.IllegalArgumentException)) {
                        // VP Note: not necessarily a failure as some agument like "state"
                        // can receive only a subset of valid values
                        // and IllegalArgument Exception is valid in the interface
                        // fail(names[i], mev.getClass().getName());
                        throw new TCKException(names[i], mev.getClass().getName(), ex);
                    }
                } catch (Exception other) {
                    log(other);
                    throw new TCKException(names[i], mev.getClass().getName(), other);
                }
            }
        }
    }
    
    /**
     * tests that setAttributeValue(a) => a in getPopulatedAttributes()
     */
    public void test_SetAttributeValue_GetPopulatedAttributesNamesContainsAttributeName(
            AttributeAccess mev) throws Exception {
        // use copy of attribute name to ensure implementaion .equals() checks
        String[] names = util.deepCopy(mev.getAttributeNames());
        util.shuffle(names);
        
        Object value = null;
        
        //tests
        for (int i = names.length - 1; i >= 0; --i) {
            try {
                if (ManagedEntityValue.class.isAssignableFrom(mev.getClass())
                && names[i].equals(ManagedEntityValue.KEY)) {
                    value = ((ManagedEntityValue) mev).makeManagedEntityKey();
                } else {
                    value = util.randomValue(names[i], mev);
                }
                
                mev.setAttributeValue(names[i], value);
                
                String[] popNames = mev.getPopulatedAttributeNames();
                
                assertTrue(names[i], mev.getClass().getName() + " shall be populated",
                        util.contains(popNames, names[i]));
            } catch (java.lang.IllegalArgumentException iae) {
                // VP Note: not necessarily a failure as some agument like "state"
                // can receive only a subset of valid values
                // and IllegalArgument Exception is valid in the interface
                // fail(names[i], mev.getClass().getName());
                log("IllegalArgumentException, is a valid exception for this test, continue");
                log(iae);
            } catch (java.lang.IllegalStateException ise) {
                // VP Note: not necessarily a failure as some agument like "reportPeriod"
                // are valid only is reportByFIle have been set before
                // fail(names[i], mev.getClass().getName());
                log("IllegalStateException, is a valid exception for this test, continue");
                log(ise);
            } catch (Exception other) {
                log("--> get Exception type:" + other.getClass().getName());
                log(other);
                throw new TCKException(names[i], mev.getClass().getName(), other);
            }
        }
    }
    
    /**
     * tests that setAttributeValue(a) => isPopulated(a)
     */
    public void test_SetAttributeValue_IsPopulated(AttributeAccess mev)
    throws Exception {
        // use copy of attribute name to ensure implementaion .equals() checks
        String[] names = util.deepCopy(mev.getAttributeNames());
        util.shuffle(names);
        
        Object value = null;
        
        //tests
        for (int i = 0; i < names.length; ++i) {
            try {
                if (ManagedEntityValue.class.isAssignableFrom(mev.getClass())
                && names[i].equals(ManagedEntityValue.KEY)) {
                    value = ((ManagedEntityValue) mev).makeManagedEntityKey();
                } else {
                    value = util.randomValue(names[i], mev);
                }
                
                mev.setAttributeValue(names[i], value);
                assertTrue(names[i], mev.getClass().getName() + " shall be populated",
                        mev.isPopulated(names[i]));
            } catch (java.lang.IllegalArgumentException other) {
                /* it is a valid exception to be thrown
                    for example when a String is used as State argument
                    for exemple and get only a subset of valid strings.
                 */
            } catch (java.lang.IllegalStateException ise) {
                /* VP Note: not necessarily a failure as some agument like "reportPeriod"
                   are valid only is reportByFIle have been set before
                 */
            } catch (Exception other) {
                log(other);
                throw new TCKException(names[i], mev.getClass().getName(), other);
            }
        }
    }
    
    /**
     * DOMAINTEST
     * tests failure of setAttributeValue(a) for an illegal object()
     */
    public void test_Negative_SetAttributeValue_IllegalObject(AttributeAccess mev)
    throws Exception {
        // use copy of attribute name to ensure implementaion .equals() checks
        String[] names = util.deepCopy(mev.getAttributeNames());
        
        //tests
        for (int i = 0; i < names.length; i++) {
            if (!(Object.class.equals(util.getAttributeType(mev, names[i])))) {
                try {
                    mev.setAttributeValue(names[i], new Object());
                    fail(mev.getClass().getName()
                    + "#: expected java.lang.IllegalArgumentException");
                } catch (java.lang.IllegalArgumentException ex) {
                    continue;
                } catch (java.lang.Exception other) {
                    log(other);
                    throw new TCKException(names[i], mev.getClass().getName(), other);
                }
            }
        }
    }
    
    /**
     * Test clone and equals methods CBE implementations<br>
     * Expected Result:<br>
     *  a = b.clone()<br>
     *  a.equals(a) == true<br>
     *  b.equals(b) == true<br>
     *  a.equals(b) == true<br>
     *  b.equals(a) == true<br>
     *
     */
    public void test_Positive_testCloneEqualsImplementation(AttributeAccess mev)
    throws Exception {
        test_Positive_testCloneEqualsImplementation((Object) mev);
    }
    
    /**
     * Test clone and equals methods CBE implementations<br>
     * Expected Result:<pre>
     *  new a;
     *  a = b.clone()
     *  a.equals(a) == true
     *  b.equals(b) == true
     *  a.equals(b) == true
     *  b.equals(a) == true
     *
     * set random attribute in a
     *  a = b.clone()
     *  a.equals(a) == true
     *  b.equals(b) == true
     *  a.equals(b) == true
     *  b.equals(a) == true
     * </PRE>
     *
     * @throws Exception DOCUMENT ME!
     * @throws TCKException DOCUMENT ME!
     */
    public void test_Positive_testCloneEqualsImplementation(Object mev)
    throws Exception {
        if (mev.getClass().isInterface()) {
            return;
        }
        
        Method meth = mev.getClass().getDeclaredMethod("clone", (Class[])null);
        
        if ((meth != null) || mev instanceof Cloneable) {
            Object clone = null;
            
            try {
                log("++> clone " + mev.getClass().getName());
                
                if (NamedQueryValue.class.isAssignableFrom(mev.getClass())) {
                    clone = ((NamedQueryValue) mev).clone();
                } else if (ManagedEntityValue.class.isAssignableFrom(mev.getClass())) {
                    clone = ((ManagedEntityValue) mev).clone();
                } else {
                    clone =  meth.invoke(mev, (Object[])null);
                }
                
            } catch (java.lang.Exception other) {
                log(other);
                throw new TCKException(null, mev.getClass().getName(), other);
            }
            
            log("++> the clone = " + clone);
            
            assertTrue(mev.getClass().getName() + " clone is null", clone != null);
            assertTrue(mev.getClass().getName() + " mev.equals(mev) failed", mev.equals(mev));
            assertTrue(mev.getClass().getName() + " clone.equals(clone) failed", clone.equals(clone));
            assertTrue(mev.getClass().getName() + " mev.equals(clone) failed", mev.equals(clone));
            assertTrue(mev.getClass().getName() + " clone.equals(mev) failed", clone.equals(mev));
            
            if (AttributeAccess.class.isAssignableFrom(mev.getClass())) {
                AttributeAccess mymev = (AttributeAccess) mev;
                String[] attNames = mymev.getAttributeNames();
                if (attNames.length == 0) return;
                util.randomManagedEntityValueAtrributes(mymev);
                
                try {
                    log("++> random att and clone " + mymev.getClass().getName());
                    clone =  meth.invoke(mev, (Object[])null);
                } catch (java.lang.Exception other) {
                    log(other);
                    throw new TCKException(null, mymev.getClass().getName(), other);
                }
                
                log("++> the clone = " + clone);
                
                assertTrue("mev+att" + mev.getClass().getName() + " clone is null", clone != null);
                assertTrue("mev+att" + mev.getClass().getName() + " mev.equals(mev) failed", mymev.equals(mymev));
                assertTrue("mev+att" + mev.getClass().getName() + " clone.equals(clone) failed", clone.equals(clone));
                assertTrue("mev+att" + mev.getClass().getName() + " mev.equals(clone) failed", mymev.equals(clone));
                assertTrue("mev+att" + mev.getClass().getName() + " clone.equals(mev) failed", clone.equals(mymev));
            }
            
        } else {
            log("--> " + mev.getClass().getName() + " is not Cloneable");
        }
    }
    
    /*
         private PropertyDescriptor[] getAllDeclaredProperties(Object instance) {
     
            // or use the beanInfo.....
            //        beanInfo = Introspector.getBeanInfo(instance.getClass(), Introspector.USE_ALL_BEANINFO);
            //        props = beanInfo.getPropertyDescriptors();
            //
            ArrayList tmp = new ArrayList();
     
            if (instance instanceof ManagedEntityValue) {
                try {
                    String[] prop = (String[]) instance.getClass().getMethod("getAttributeNames", null)
                                                       .invoke(instance, null);
     
                    for (int i = prop.length - 1; i >= 0; i--)
                        tmp.add(new PropertyDescriptor(prop[i], instance.getClass()));
     
                    prop = (String[]) instance.getClass()
                                              .getMethod("getSupportedOptionalAttributeNames", null)
                                              .invoke(instance, null);
     
                    for (int j = prop.length - 1; j >= 0; j--)
                        tmp.add(new PropertyDescriptor(prop[j], instance.getClass()));
                } catch (Exception e) {
                    log("--> getAllDeclaredProperties Exception:");
                    log(e);
                }
            }
     
            return (PropertyDescriptor[]) tmp.toArray(new PropertyDescriptor[tmp.size()]);
        }
     
     */
}
