// @(#)file      CommonAPIXMLTests.java
// @(#)author    Vincent Perrot, Sun Microsystems, Inc.
// @(#)version   1.12
// @(#)date      02/03/12
// 
// Copyright 2002 Sun Microsystems, Inc. All rights reserved.
// This software is the proprietary information of Sun Microsystems, Inc.
// Use is subject to license terms.
// 
// Copyright 2002 Sun Microsystems, Inc. Tous droits rservs.
// Ce logiciel est propriet de Sun Microsystems, Inc.
// Distribu par des licences qui en restreignent l'utilisation.
// 
package ossj.common.tck.tests;

import junit.framework.*;

import ossj.common.tck.*;

import java.lang.reflect.Method;

import java.util.ArrayList;

import javax.jms.*;


/**
 * XmlTests Class
 *
 * Implement the XML Mesaging Mandatory Test of the JSR144 specification.
 *
 * @author Copyright 2002 Sun Microsystems Inc., Inc. All rights reserved.
 * @version 1.12
 */
public class XmlTests extends ossj.common.tck.JVTTestBase {
    public boolean done = false;
    public TestResult result;
    private TemporaryQueue replyQueue = null;
    private QueueSender qsender = null;
    private QueueSession qsession = null;
    private QueueReceiver qreceiver = null;

    /**
     * Creates a new CommonAPIXMLTests object.
     *
     * @param name the test name
     *
     * @throws Exception any
     */
    public XmlTests(String name) throws Exception {
        super(name);
        getEnvironment();
        qsession = util.createQueueSession();
        replyQueue = qsession.createTemporaryQueue();
        qsender = util.createQueueSender();
        qreceiver = qsession.createReceiver(replyQueue);
    }

    /**
     * Initialize every test
     */
    public void setUp() {
    }

    /**
     * 
     */
    public void tearDown() {
    }

    /**
     * Build the suite of test
     *
     * @return the test suite
     *
     * @throws Exception ...
     */
    public static TestSuite suite() throws Exception {
        TestSuite retVal = new TestSuite("Xml Tests");
        TestSuite domain = new TestSuite("Domain tests");
        TestSuite other = new TestSuite("Logical tests");

        Class cls = XmlTests.class;
        Method[] methods = cls.getDeclaredMethods();

        for (int i = 0; i < methods.length; ++i) {
            String name = methods[i].getName();

            if (name.startsWith(StringTable.TEST_PREFIX)) {
                if (name.startsWith(StringTable.POSITIVE_TEST_PREFIX)
                        || name.startsWith(StringTable.NEGATIVE_TEST_PREFIX)) {
                    domain.addTest(new XmlTests(name));
                } else {
                    other.addTest(new XmlTests(name));
                }
            }
        }

        retVal.addTest(other);
        retVal.addTest(domain);

        return retVal;
    }

    //    	<getManagedEntityTypesRequest xmlns="http://ossj.org/xml/Common/v1-2"/>

    /**
     * Test if the given JNDI name follows:
     * System/System1/ApplicationType/Common/Application/1-2;1-2;ReferenceImplementation/Comp/name
     * or
     * System/System1/ApplicationType/Common/Application/1-2;1-2;ReferenceImplementation/Resource/name
     *
     * @throws Exception
     *
     */
    public void test_xml_Queue_JndiName() throws Exception {
        /*test*/
        log("==> test_xml_Queue_JndiName");
        assertTrue(util.isValidJndiName(util.getQueueFactoryName()),
            (util.isValidJndiName(util.getQueueFactoryName()) == null));
        assertTrue(util.isValidJndiName(util.getRequestQueueName()),
            (util.isValidJndiName(util.getRequestQueueName()) == null));

        // test here the queue names....
    }

    /**
    *
    * getManagedEntityTypes, instantiated from TestBasics <br>
    * verify that the JVT support at least one managed Entity type using getManagedEntityTypes
    * <P>
    * Success if the the return array is not null and not empty
    */
    public void test_xml_GetManagedEntityTypes_ContainsACTIVATE()
        throws Exception {
        /*test*/
        log("==> test_xml_GetManagedEntityTypes_ContainsACTIVATE ");

        String request = "<getManagedEntityTypesRequest xmlns=\"http://ossj.org/xml/Common/v1-2\"/>";
        logSend(addXmlRootElement(request));

        String resp = sendRequest(addXmlRootElement(request));
        logReceive(resp);

        if (isException(resp)) {
            fail("Get exception when sending " + request + ": " + resp);
        }

        String[] managedEntityTypes = getStringValues(resp);
        assertTrue("getManagedEntityTypes returns null ", managedEntityTypes != null);
        assertTrue("getManagedEntityTypes returns empty types list ", managedEntityTypes.length != 0);
    }

    /**
    *
    * verify that JVT support at least one query type using getNamedQueryTypes.<br>
    * getNamedQueryTypes instantiated from TestBasics
    * <P>
    * Success if the the return array is not null and not empty
    */
    public void test_xml_GetQueryTypes_IsNotNull() throws Exception {
        /*tests*/
        log("==> test_xml_GetQueryTypes_IsNotNull ");

        String request = "<getNamedQueryTypesRequest xmlns=\"http://ossj.org/xml/Common/v1-2\"/>";
        logSend(request);

        String resp = sendRequest(addXmlRootElement(request));
        logReceive(resp);

        if (isException(resp)) {
            fail("Get exception when sending " + request + ": " + resp);
        }

        String[] types = getStringValues(resp);
        assertTrue("getNamedQueryTypes returns null", types != null);
        //Fix issue #51
        //assertTrue("getNamedQueryTypes returns empty types list", types.length != 0);
    }

    /**
     *
     * verify that JVT support at least one query type using getUpdateProcedureTypes.<br>
     * getUpdateProcedureTypes instantiated from TestBasics
     * <P>
     * Success if the the return array is not null
     */
    public void test_xml_GetUpvTypes_IsNotNull() throws Exception {
        /*tests*/
        log("==> test_xml_GetUpvTypes_IsNotNull ");

        String request = "<getUpdateProcedureTypesRequest xmlns=\"http://ossj.org/xml/Common/v1-2\"/>";
        logSend(request);

        String resp = sendRequest(addXmlRootElement(request));
        logReceive(resp);

        if (isException(resp)) {
            fail("Get exception when sending " + request + ": " + resp);
        }

        String[] types = getStringValues(resp);
        assertTrue("getUpdateProcedureTypes returns null", types != null);

        //assertTrue("getUpdateProcedureTypes returns empty types list", types.length != 0);
    }

    /**
     * GetSupportedOperations instantiated from TestBasics.<br>
    * just call getSupportedOptionalOperations
    * <P>
    * Success if no exceptiuon is thrown.
    */
    public void test_xml_GetSupportedOperations_IsSubsetOfJVTSessionOption()
        throws Exception {
        //    	tests
        log("==> test_xml_GetSupportedOperations_IsSubsetOfJVTSessionOption ");

        String request = "<getSupportedOptionalOperationsRequest xmlns=\"http://ossj.org/xml/Common/v1-2\"/>";
        logSend(request);

        String resp = sendRequest(addXmlRootElement(request));
        logReceive(resp);

        if (isException(resp)) {
            fail("Get exception when sending " + request + ": " + resp);
        }

        String[] operations = getStringValues(resp);

        //        	String[] optionalOperations = this.util.getOptionalOperations();
        //        	assertTrue("getSupportedOperations returned null", null!=operations);
        //        	for( int i=0;i<operations.length;++i ){
        //            	assertTrue( "Supported operation "+operations[i]+" is not in JVTActivationSessionOption", 
        //                        this.util.contains(optionalOperations, operations[i]) );
        //        	}
    }

    /**
     * Query managed entities using null query type
     * <br>
     * Success if querry(null) throws OssIllegalArgumentException
     *
     * @throws Exception DOCUMENT ME!
     * @throws TCKException DOCUMENT ME!
     */
    public void test_Negative_xml_query_NullQuery() throws Exception {
        log("==> test_xml_Negative_query_NullQuery ");

        String request = "<queryRequest xmlns=\"http://ossj.org/xml/Common/v1-2\"/>";
        logSend(request);

        String resp = sendRequest(addXmlRootElement(request));
        logReceive(resp);

        if (!isException(resp) || (resp.indexOf("<ossIllegalArgumentException") < 0)) {
            fail("Expected OssIllegalArgumentException when sending " + request + " instead of: "
                + resp);
        }
    }

    /**
     * Update procedure using null  type
     * <br>
     * Success if update(null) throws OssIllegalArgumentException
     *
     * @throws Exception DOCUMENT ME!
     * @throws TCKException DOCUMENT ME!
     */
    public void test_Negative_xml_update_NullUpv() throws Exception {
        log("==> test_xml_Negative_update_NullUpv ");

        String request = "<updateRequest xmlns=\"http://ossj.org/xml/Common/v1-2\"/>";
        logSend(request);

        String resp = sendRequest(addXmlRootElement(request));
        logReceive(resp);

        if (!isException(resp) || (resp.indexOf("<ossIllegalArgumentException") < 0)) {
            fail("Expected OssIllegalArgumentException when sending " + request + " instead of: "
                + resp);
        }
    }

    /**
     * Verify that the JVT support at least one event type.<br>
     * For each Event type, verify also the corresponding EventPropertyDescriptor using the factory.
     * <br>
     * Success <br>
     * if getEventTypes return no null and at least 1 event type
     * and if (getEventDescriptor) for each type return a valid event propertyDescriptor (correct event type value)
     * and if makeEvent(type) do not throw any exception and returnan instance of the desired event type
     *
     * @throws Exception DOCUMENT ME!
     */
    public void test_Positive_xml_getEventTypes() throws Exception {
        log("==> test_xml_Positive_getEventTypes ");

        String request = "<getEventTypesRequest xmlns=\"http://ossj.org/xml/Common/v1-2\"/>";
        logSend(request);

        String resp = sendRequest(addXmlRootElement(request));
        logReceive(resp);

        if (isException(resp)) {
            fail("Get exception when sending " + request + ": " + resp);
        }

        String[] types = getStringValues(resp);
        assertTrue("getEventTypes() returns null", types != null);
        //Fix Issue#51
        //assertTrue("getEventTypes() returns 0 element", types.length != 0);

        for (int i = types.length - 1; i >= 0; i--) {
            request = "<getEventDescriptorRequest xmlns=\"http://ossj.org/xml/Common/v1-2\">\n";
            request += "<eventType>";
            request += types[i];
            request += "</eventType>\n";
            request += "</getEventDescriptorRequest>\n";

            logSend(request);
            resp = sendRequest(addXmlRootElement(request));
            logReceive(resp);

            if (isException(resp)) {
                fail("Get exception when sending " + request + ": " + resp);
            }

            assertTrue("EventPropertyDescritor is null for " + types[i],
                resp.indexOf("<eventPropertyDescriptor") >= 0);
            assertTrue("EventType in descriptor is different form initial type for " + types[i],
                resp.indexOf(types[i]) >= 0);
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param rootElementTag DOCUMENT ME!
     * @param xmlDoc DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    private String addXmlRootElement(String xmlDoc) {
        if (xmlDoc != null) {
            return "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n" + xmlDoc;
        } else {
            return null;
        }
    }

    private void logSend(String message) {
        log("---> Send:\n");
        log(message);
        log("\n---------------------------------------------------------\n");
    }

    private void logReceive(String message) {
        log("<--- Receive:\n");
        log(message);
        log("\n---------------------------------------------------------\n");
    }

    private boolean isException(String message) {
        if ((message.indexOf("<ossIllegalArgumentException") >= 0)
                || (message.indexOf("<ossIllegalAttributeValueException") >= 0)
                || (message.indexOf("<ossIllegalStateException") >= 0)
                || (message.indexOf("<ossResyncRequiredException") >= 0)
                || (message.indexOf("<ossSetException") >= 0)
                || (message.indexOf("<ossUnsupportedEttributeException") >= 0)
                || (message.indexOf("<ossUnsupportedOperationException") >= 0)
                || (message.indexOf("<remoteException") >= 0)
                || (message.indexOf("<createException") >= 0)) {
            return true;
        } else {
            return false;
        }
    }

    /*<strings>
      <item>ossj.common.ex.ManagedEntityEx2ValueImpl</item>
      <item>ossj.common.ex.ManagedEntityEx1ValueImpl</item>
    </strings>
        */
    private String[] getStringValues(String message) {
        ArrayList values = new ArrayList();

        if (message.indexOf("<strings/>") >= 0) {
            return new String[0];
        }

        if (message.indexOf("<strings") < 0) {
            // may be return null; ???
            return new String[0];
        }

        //      log("message:\n"+message);
        //      log("strings="+message.indexOf("<strings>" )
        //      log("/strings="+message.indexOf("</strings>" )
        String temp = message.substring(message.indexOf("<strings>") + 9,
                message.indexOf("</strings>"));

        if (temp.indexOf("<item") < 0) {
            return new String[0];
        }

        while (temp.indexOf("<item") >= 0) {
            log("start: " + temp);

            if (temp.substring(0, 6).equals("<item/>")) {
                temp = temp.substring(7);
                log(temp);
                values.add("");
            }

            temp = temp.substring(temp.indexOf("<item>") + 6);
            values.add(temp.substring(0, temp.indexOf("</item>")));

            //log("add:"+temp.substring(0, temp.indexOf("</item>")));
            temp = temp.substring(temp.indexOf("</item>") + 7);

            //log("remaining temp = "+temp);
        }

        return (String[]) values.toArray(new String[0]);
    }

    private boolean isValidateXml(String xml) {
        return true;
    }

    private String sendRequest(String s) {
        Object obj = null;
        String s2 = null;

        try {
            //log("session "+qsession.toString());
            TextMessage textmessage = qsession.createTextMessage(s);
            textmessage.setJMSReplyTo(replyQueue);

            //logSend(s);
            qsender.send(textmessage);
        } catch (JMSException jmsexception) {
            jmsexception.printStackTrace();
            log("get exception when sending request:" + jmsexception.getMessage());
        }

        try {
            boolean flag = true;

            while (flag) {
                Message message = qreceiver.receive();
                s2 = getTxtMsg(message);

                //logReceive(s2);
                if (s2.endsWith(">")) {
                    flag = false;
                }
            }
        } catch (JMSException jmsexception) {
            jmsexception.printStackTrace();
            log("get exception when waiting reply:" + jmsexception.getMessage());
        }

        return s2;
    }

    private String getTxtMsg(Message message) {
        String s = null;

        try {
            s = ((TextMessage) message).getText();
        } catch (JMSException jmsexception) {
            log("get exception when geting the text in JMS message:" + jmsexception.getMessage());
        }

        return s;
    }
}
